=== modified file 'bzrlib/__init__.py'
--- bzrlib/__init__.py	2011-05-27 21:41:02 +0000
+++ bzrlib/__init__.py	2011-06-27 15:42:09 +0000
@@ -52,7 +52,7 @@
 # Python version 2.0 is (2, 0, 0, 'final', 0)."  Additionally we use a
 # releaselevel of 'dev' for unreleased under-development code.
 
-version_info = (2, 4, 0, 'dev', 4)
+version_info = (2, 4, 0, 'dev', 5)
 
 # API compatibility version
 api_minimum_version = (2, 4, 0)

=== modified file 'bzrlib/annotate.py'
--- bzrlib/annotate.py	2011-05-03 15:04:00 +0000
+++ bzrlib/annotate.py	2011-06-27 15:52:06 +0000
@@ -104,7 +104,10 @@
         # bugfixes etc.
         current_rev = Revision(CURRENT_REVISION)
         current_rev.parent_ids = tree.get_parent_ids()
-        current_rev.committer = branch.get_config().username()
+        try:
+            current_rev.committer = branch.get_config().username()
+        except errors.NoWhoami:
+            current_rev.committer = 'local user'
         current_rev.message = "?"
         current_rev.timestamp = round(time.time(), 3)
         current_rev.timezone = osutils.local_time_offset()

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2011-06-01 13:43:12 +0000
+++ bzrlib/branch.py	2011-06-20 11:04:42 +0000
@@ -513,7 +513,7 @@
         rev_iter = iter(merge_sorted_revisions)
         if start_revision_id is not None:
             for node in rev_iter:
-                rev_id = node.key[-1]
+                rev_id = node.key
                 if rev_id != start_revision_id:
                     continue
                 else:
@@ -525,19 +525,19 @@
         if stop_revision_id is None:
             # Yield everything
             for node in rev_iter:
-                rev_id = node.key[-1]
+                rev_id = node.key
                 yield (rev_id, node.merge_depth, node.revno,
                        node.end_of_merge)
         elif stop_rule == 'exclude':
             for node in rev_iter:
-                rev_id = node.key[-1]
+                rev_id = node.key
                 if rev_id == stop_revision_id:
                     return
                 yield (rev_id, node.merge_depth, node.revno,
                        node.end_of_merge)
         elif stop_rule == 'include':
             for node in rev_iter:
-                rev_id = node.key[-1]
+                rev_id = node.key
                 yield (rev_id, node.merge_depth, node.revno,
                        node.end_of_merge)
                 if rev_id == stop_revision_id:
@@ -549,7 +549,7 @@
             ancestors = graph.find_unique_ancestors(start_revision_id,
                                                     [stop_revision_id])
             for node in rev_iter:
-                rev_id = node.key[-1]
+                rev_id = node.key
                 if rev_id not in ancestors:
                     continue
                 yield (rev_id, node.merge_depth, node.revno,
@@ -565,7 +565,7 @@
             reached_stop_revision_id = False
             revision_id_whitelist = []
             for node in rev_iter:
-                rev_id = node.key[-1]
+                rev_id = node.key
                 if rev_id == left_parent:
                     # reached the left parent after the stop_revision
                     return

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2011-06-01 16:50:59 +0000
+++ bzrlib/builtins.py	2011-06-27 15:42:09 +0000
@@ -1740,9 +1740,13 @@
             b = wt.branch
             last_revision = wt.last_revision()
 
-        revision_ids = b.repository.get_ancestry(last_revision)
-        revision_ids.pop(0)
-        for revision_id in revision_ids:
+        self.add_cleanup(b.repository.lock_read().unlock)
+        graph = b.repository.get_graph()
+        revisions = [revid for revid, parents in
+            graph.iter_ancestry([last_revision])]
+        for revision_id in reversed(revisions):
+            if _mod_revision.is_null(revision_id):
+                continue
             self.outf.write(revision_id + '\n')
 
 

=== modified file 'bzrlib/bundle/serializer/__init__.py'
--- bzrlib/bundle/serializer/__init__.py	2010-11-05 20:54:32 +0000
+++ bzrlib/bundle/serializer/__init__.py	2011-06-16 11:58:22 +0000
@@ -159,10 +159,9 @@
         forced_bases = {revision_id:base_revision_id}
         if base_revision_id is NULL_REVISION:
             base_revision_id = None
-        revision_ids = set(repository.get_ancestry(revision_id,
-                           topo_sorted=False))
-        revision_ids.difference_update(repository.get_ancestry(
-            base_revision_id, topo_sorted=False))
+        graph = repository.get_graph()
+        revision_ids = graph.find_unique_ancestors(revision_id,
+            [base_revision_id])
         revision_ids = list(repository.get_graph().iter_topo_order(
             revision_ids))
         revision_ids.reverse()

=== modified file 'bzrlib/commands.py'
--- bzrlib/commands.py	2011-05-19 09:32:38 +0000
+++ bzrlib/commands.py	2011-06-27 15:36:58 +0000
@@ -36,6 +36,7 @@
     cmdline,
     debug,
     errors,
+    i18n,
     option,
     osutils,
     trace,
@@ -44,6 +45,7 @@
 """)
 
 from bzrlib.hooks import Hooks
+from bzrlib.i18n import gettext
 # Compatibility - Option used to be in commands.
 from bzrlib.option import Option
 from bzrlib.plugin import disable_plugins, load_plugins
@@ -408,6 +410,7 @@
     takes_options = []
     encoding_type = 'strict'
     invoked_as = None
+    l10n = True
 
     hidden = False
 
@@ -485,9 +488,18 @@
             usage help (e.g. Purpose, Usage, Options) with a
             message explaining how to obtain full help.
         """
+        if self.l10n and not i18n.installed():
+            i18n.install()  # Install i18n only for get_help_text for now.
         doc = self.help()
-        if not doc:
-            doc = "No help for this command."
+        if doc:
+            # Note: If self.gettext() translates ':Usage:\n', the section will
+            # be shown after "Description" section and we don't want to
+            # translate the usage string.
+            # Though, bzr export-pot don't exports :Usage: section and it must
+            # not be translated.
+            doc = self.gettext(doc)
+        else:
+            doc = gettext("No help for this command.")
 
         # Extract the summary (purpose) and sections out from the text
         purpose,sections,order = self._get_help_parts(doc)
@@ -500,11 +512,11 @@
 
         # The header is the purpose and usage
         result = ""
-        result += ':Purpose: %s\n' % purpose
+        result += gettext(':Purpose: %s\n') % (purpose,)
         if usage.find('\n') >= 0:
-            result += ':Usage:\n%s\n' % usage
+            result += gettext(':Usage:\n%s\n') % (usage,)
         else:
-            result += ':Usage:   %s\n' % usage
+            result += gettext(':Usage:   %s\n') % (usage,)
         result += '\n'
 
         # Add the options
@@ -512,7 +524,8 @@
         # XXX: optparse implicitly rewraps the help, and not always perfectly,
         # so we get <https://bugs.launchpad.net/bzr/+bug/249908>.  -- mbp
         # 20090319
-        options = option.get_optparser(self.options()).format_option_help()
+        parser = option.get_optparser(self.options())
+        options = parser.format_option_help()
         # FIXME: According to the spec, ReST option lists actually don't
         # support options like --1.14 so that causes syntax errors (in Sphinx
         # at least).  As that pattern always appears in the commands that
@@ -522,10 +535,7 @@
         if not plain and options.find('  --1.14  ') != -1:
             options = options.replace(' format:\n', ' format::\n\n', 1)
         if options.startswith('Options:'):
-            result += ':' + options
-        elif options.startswith('options:'):
-            # Python 2.4 version of optparse
-            result += ':Options:' + options[len('options:'):]
+            result += gettext(':Options:%s') % (options[len('options:'):],)
         else:
             result += options
         result += '\n'
@@ -536,26 +546,26 @@
             if sections.has_key(None):
                 text = sections.pop(None)
                 text = '\n  '.join(text.splitlines())
-                result += ':%s:\n  %s\n\n' % ('Description',text)
+                result += gettext(':Description:\n  %s\n\n') % (text,)
 
             # Add the custom sections (e.g. Examples). Note that there's no need
             # to indent these as they must be indented already in the source.
             if sections:
                 for label in order:
-                    if sections.has_key(label):
-                        result += ':%s:\n%s\n' % (label,sections[label])
+                    if label in sections:
+                        result += ':%s:\n%s\n' % (label, sections[label])
                 result += '\n'
         else:
-            result += ("See bzr help %s for more details and examples.\n\n"
+            result += (gettext("See bzr help %s for more details and examples.\n\n")
                 % self.name())
 
         # Add the aliases, source (plug-in) and see also links, if any
         if self.aliases:
-            result += ':Aliases:  '
+            result += gettext(':Aliases:  ')
             result += ', '.join(self.aliases) + '\n'
         plugin_name = self.plugin_name()
         if plugin_name is not None:
-            result += ':From:     plugin "%s"\n' % plugin_name
+            result += gettext(':From:     plugin "%s"\n') % plugin_name
         see_also = self.get_see_also(additional_see_also)
         if see_also:
             if not plain and see_also_as_links:
@@ -567,11 +577,10 @@
                         see_also_links.append(item)
                     else:
                         # Use a Sphinx link for this entry
-                        link_text = ":doc:`%s <%s-help>`" % (item, item)
+                        link_text = gettext(":doc:`%s <%s-help>`") % (item, item)
                         see_also_links.append(link_text)
                 see_also = see_also_links
-            result += ':See also: '
-            result += ', '.join(see_also) + '\n'
+            result += gettext(':See also: %s') % ', '.join(see_also) + '\n'
 
         # If this will be rendered as plain text, convert it
         if plain:
@@ -658,13 +667,14 @@
     def run_argv_aliases(self, argv, alias_argv=None):
         """Parse the command line and run with extra aliases in alias_argv."""
         args, opts = parse_args(self, argv, alias_argv)
+        self._setup_outf()
 
         # Process the standard options
         if 'help' in opts:  # e.g. bzr add --help
-            sys.stdout.write(self.get_help_text())
+            self.outf.write(self.get_help_text())
             return 0
         if 'usage' in opts:  # e.g. bzr add --usage
-            sys.stdout.write(self.get_help_text(verbose=False))
+            self.outf.write(self.get_help_text(verbose=False))
             return 0
         trace.set_verbosity_level(option._verbosity_level)
         if 'verbose' in self.supported_std_options:
@@ -685,8 +695,6 @@
         all_cmd_args = cmdargs.copy()
         all_cmd_args.update(cmdopts)
 
-        self._setup_outf()
-
         try:
             return self.run(**all_cmd_args)
         finally:
@@ -751,6 +759,14 @@
             return None
         return getdoc(self)
 
+    def gettext(self, message):
+        """Returns the gettext function used to translate this command's help.
+
+        Commands provided by plugins should override this to use their
+        own i18n system.
+        """
+        return i18n.gettext_per_paragraph(message)
+
     def name(self):
         """Return the canonical name for this command.
 
@@ -1041,8 +1057,8 @@
     argv = _specified_or_unicode_argv(argv)
     trace.mutter("bzr arguments: %r", argv)
 
-    opt_lsprof = opt_profile = opt_no_plugins = opt_builtin =  \
-                opt_no_aliases = False
+    opt_lsprof = opt_profile = opt_no_plugins = opt_builtin = \
+            opt_no_l10n = opt_no_aliases = False
     opt_lsprof_file = opt_coverage_dir = None
 
     # --no-plugins is handled specially at a very early stage. We need
@@ -1065,6 +1081,8 @@
             opt_no_plugins = True
         elif a == '--no-aliases':
             opt_no_aliases = True
+        elif a == '--no-l10n':
+            opt_no_l10n = True
         elif a == '--builtin':
             opt_builtin = True
         elif a == '--concurrency':
@@ -1106,6 +1124,8 @@
 
     cmd = argv.pop(0)
     cmd_obj = get_cmd_object(cmd, plugins_override=not opt_builtin)
+    if opt_no_l10n:
+        cmd.l10n = False
     run = cmd_obj.run_argv_aliases
     run_argv = [argv, alias_argv]
 

=== modified file 'bzrlib/commit.py'
--- bzrlib/commit.py	2011-05-17 14:27:30 +0000
+++ bzrlib/commit.py	2011-06-19 02:48:01 +0000
@@ -52,7 +52,6 @@
 from bzrlib import (
     debug,
     errors,
-    revision,
     trace,
     tree,
     ui,

=== modified file 'bzrlib/config.py'
--- bzrlib/config.py	2011-06-01 12:31:04 +0000
+++ bzrlib/config.py	2011-06-20 14:03:40 +0000
@@ -65,9 +65,8 @@
 import os
 import string
 import sys
-import weakref
-
-from bzrlib import commands
+
+
 from bzrlib.decorators import needs_write_lock
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
@@ -75,7 +74,6 @@
 import re
 from cStringIO import StringIO
 
-import bzrlib
 from bzrlib import (
     atomicfile,
     bzrdir,
@@ -95,6 +93,8 @@
 from bzrlib.util.configobj import configobj
 """)
 from bzrlib import (
+    commands,
+    hooks,
     registry,
     )
 from bzrlib.symbol_versioning import (
@@ -145,7 +145,6 @@
                                         interpolation=False,
                                         **kwargs)
 
-
     def get_bool(self, section, key):
         return self[section].as_bool(key)
 
@@ -159,7 +158,7 @@
         return self[section][name]
 
 
-# FIXME: Until we can guarantee that each config file is loaded once and and
+# FIXME: Until we can guarantee that each config file is loaded once and
 # only once for a given bzrlib session, we don't want to re-read the file every
 # time we query for an option so we cache the value (bad ! watch out for tests
 # needing to restore the proper value).This shouldn't be part of 2.4.0 final,
@@ -370,18 +369,22 @@
                               % (option_name,))
             else:
                 value = self._expand_options_in_string(value)
+        for hook in OldConfigHooks['get']:
+            hook(self, option_name, value)
         return value
 
-    def get_user_option_as_bool(self, option_name, expand=None):
-        """Get a generic option as a boolean - no special process, no default.
+    def get_user_option_as_bool(self, option_name, expand=None, default=None):
+        """Get a generic option as a boolean.
 
+        :param expand: Allow expanding references to other config values.
+        :param default: Default value if nothing is configured
         :return None if the option doesn't exist or its value can't be
             interpreted as a boolean. Returns True or False otherwise.
         """
         s = self.get_user_option(option_name, expand=expand)
         if s is None:
             # The option doesn't exist
-            return None
+            return default
         val = ui.bool_from_string(s)
         if val is None:
             # The value can't be interpreted as a boolean
@@ -492,10 +495,10 @@
         if policy is None:
             policy = self._get_signature_checking()
             if policy is not None:
+                #this warning should go away once check_signatures is
+                #implemented (if not before)
                 trace.warning("Please use create_signatures,"
                               " not check_signatures to set signing policy.")
-            if policy == CHECK_ALWAYS:
-                return True
         elif policy == SIGN_ALWAYS:
             return True
         return False
@@ -544,7 +547,7 @@
         return tools
 
     def find_merge_tool(self, name):
-        # We fake a defaults mechanism here by checking if the given name can 
+        # We fake a defaults mechanism here by checking if the given name can
         # be found in the known_merge_tools if it's not found in the config.
         # This should be done through the proposed config defaults mechanism
         # when it becomes available in the future.
@@ -554,6 +557,76 @@
         return command_line
 
 
+class _ConfigHooks(hooks.Hooks):
+    """A dict mapping hook names and a list of callables for configs.
+    """
+
+    def __init__(self):
+        """Create the default hooks.
+
+        These are all empty initially, because by default nothing should get
+        notified.
+        """
+        super(_ConfigHooks, self).__init__('bzrlib.config', 'ConfigHooks')
+        self.add_hook('load',
+                      'Invoked when a config store is loaded.'
+                      ' The signature is (store).',
+                      (2, 4))
+        self.add_hook('save',
+                      'Invoked when a config store is saved.'
+                      ' The signature is (store).',
+                      (2, 4))
+        # The hooks for config options
+        self.add_hook('get',
+                      'Invoked when a config option is read.'
+                      ' The signature is (stack, name, value).',
+                      (2, 4))
+        self.add_hook('set',
+                      'Invoked when a config option is set.'
+                      ' The signature is (stack, name, value).',
+                      (2, 4))
+        self.add_hook('remove',
+                      'Invoked when a config option is removed.'
+                      ' The signature is (stack, name).',
+                      (2, 4))
+ConfigHooks = _ConfigHooks()
+
+
+class _OldConfigHooks(hooks.Hooks):
+    """A dict mapping hook names and a list of callables for configs.
+    """
+
+    def __init__(self):
+        """Create the default hooks.
+
+        These are all empty initially, because by default nothing should get
+        notified.
+        """
+        super(_OldConfigHooks, self).__init__('bzrlib.config', 'OldConfigHooks')
+        self.add_hook('load',
+                      'Invoked when a config store is loaded.'
+                      ' The signature is (config).',
+                      (2, 4))
+        self.add_hook('save',
+                      'Invoked when a config store is saved.'
+                      ' The signature is (config).',
+                      (2, 4))
+        # The hooks for config options
+        self.add_hook('get',
+                      'Invoked when a config option is read.'
+                      ' The signature is (config, name, value).',
+                      (2, 4))
+        self.add_hook('set',
+                      'Invoked when a config option is set.'
+                      ' The signature is (config, name, value).',
+                      (2, 4))
+        self.add_hook('remove',
+                      'Invoked when a config option is removed.'
+                      ' The signature is (config, name).',
+                      (2, 4))
+OldConfigHooks = _OldConfigHooks()
+
+
 class IniBasedConfig(Config):
     """A configuration policy that draws from ini files."""
 
@@ -619,8 +692,12 @@
             self._parser = ConfigObj(co_input, encoding='utf-8')
         except configobj.ConfigObjError, e:
             raise errors.ParseConfigError(e.errors, e.config.filename)
+        except UnicodeDecodeError:
+            raise errors.ConfigContentError(self.file_name)
         # Make sure self.reload() will use the right file name
         self._parser.filename = self.file_name
+        for hook in OldConfigHooks['load']:
+            hook(self)
         return self._parser
 
     def reload(self):
@@ -629,6 +706,8 @@
             raise AssertionError('We need a file name to reload the config')
         if self._parser is not None:
             self._parser.reload()
+        for hook in ConfigHooks['load']:
+            hook(self)
 
     def _get_matching_sections(self):
         """Return an ordered list of (section_name, extra_path) pairs.
@@ -806,6 +885,8 @@
         except KeyError:
             raise errors.NoSuchConfigOption(option_name)
         self._write_config_file()
+        for hook in OldConfigHooks['remove']:
+            hook(self, option_name)
 
     def _write_config_file(self):
         if self.file_name is None:
@@ -817,6 +898,8 @@
         atomic_file.commit()
         atomic_file.close()
         osutils.copy_ownership_from_path(self.file_name)
+        for hook in OldConfigHooks['save']:
+            hook(self)
 
 
 class LockableConfig(IniBasedConfig):
@@ -950,7 +1033,8 @@
         self.reload()
         self._get_parser().setdefault(section, {})[option] = value
         self._write_config_file()
-
+        for hook in OldConfigHooks['set']:
+            hook(self, option, value)
 
     def _get_sections(self, name=None):
         """See IniBasedConfig._get_sections()."""
@@ -1152,6 +1236,8 @@
         # the allowed values of store match the config policies
         self._set_option_policy(location, option, store)
         self._write_config_file()
+        for hook in OldConfigHooks['set']:
+            hook(self, option, value)
 
 
 class BranchConfig(Config):
@@ -1613,6 +1699,8 @@
             self._config = ConfigObj(self._input, encoding='utf-8')
         except configobj.ConfigObjError, e:
             raise errors.ParseConfigError(e.errors, e.config.filename)
+        except UnicodeError:
+            raise errors.ConfigContentError(self._filename)
         return self._config
 
     def _save(self):
@@ -1635,7 +1723,7 @@
         section[option_name] = value
         self._save()
 
-    def get_credentials(self, scheme, host, port=None, user=None, path=None, 
+    def get_credentials(self, scheme, host, port=None, user=None, path=None,
                         realm=None):
         """Returns the matching credentials from authentication.conf file.
 
@@ -2054,7 +2142,10 @@
                 section_obj = configobj[section]
             except KeyError:
                 return default
-        return section_obj.get(name, default)
+        value = section_obj.get(name, default)
+        for hook in OldConfigHooks['get']:
+            hook(self, name, value)
+        return value
 
     def set_option(self, value, name, section=None):
         """Set the value associated with a named option.
@@ -2068,6 +2159,8 @@
             configobj[name] = value
         else:
             configobj.setdefault(section, {})[name] = value
+        for hook in OldConfigHooks['set']:
+            hook(self, name, value)
         self._set_configobj(configobj)
 
     def remove_option(self, option_name, section_name=None):
@@ -2076,26 +2169,42 @@
             del configobj[option_name]
         else:
             del configobj[section_name][option_name]
+        for hook in OldConfigHooks['remove']:
+            hook(self, option_name)
         self._set_configobj(configobj)
 
     def _get_config_file(self):
         try:
-            return StringIO(self._transport.get_bytes(self._filename))
+            f = StringIO(self._transport.get_bytes(self._filename))
+            for hook in OldConfigHooks['load']:
+                hook(self)
+            return f
         except errors.NoSuchFile:
             return StringIO()
 
+    def _external_url(self):
+        return urlutils.join(self._transport.external_url(), self._filename)
+
     def _get_configobj(self):
         f = self._get_config_file()
         try:
-            return ConfigObj(f, encoding='utf-8')
+            try:
+                conf = ConfigObj(f, encoding='utf-8')
+            except configobj.ConfigObjError, e:
+                raise errors.ParseConfigError(e.errors, self._external_url())
+            except UnicodeDecodeError:
+                raise errors.ConfigContentError(self._external_url())
         finally:
             f.close()
+        return conf
 
     def _set_configobj(self, configobj):
         out_file = StringIO()
         configobj.write(out_file)
         out_file.seek(0)
         self._transport.put_file(self._filename, out_file)
+        for hook in OldConfigHooks['save']:
+            hook(self)
 
 
 class Option(object):
@@ -2189,14 +2298,10 @@
         """Loads the Store from persistent storage."""
         raise NotImplementedError(self.load)
 
-    def _load_from_string(self, str_or_unicode):
+    def _load_from_string(self, bytes):
         """Create a store from a string in configobj syntax.
 
-        :param str_or_unicode: A string representing the file content. This will
-            be encoded to suit store needs internally.
-
-        This is for tests and should not be used in production unless a
-        convincing use case can be demonstrated :)
+        :param bytes: A string representing the file content.
         """
         raise NotImplementedError(self._load_from_string)
 
@@ -2271,25 +2376,25 @@
             return
         content = self.transport.get_bytes(self.file_name)
         self._load_from_string(content)
+        for hook in ConfigHooks['load']:
+            hook(self)
 
-    def _load_from_string(self, str_or_unicode):
+    def _load_from_string(self, bytes):
         """Create a config store from a string.
 
-        :param str_or_unicode: A string representing the file content. This will
-            be utf-8 encoded internally.
-
-        This is for tests and should not be used in production unless a
-        convincing use case can be demonstrated :)
+        :param bytes: A string representing the file content.
         """
         if self.is_loaded():
             raise AssertionError('Already loaded: %r' % (self._config_obj,))
-        co_input = StringIO(str_or_unicode.encode('utf-8'))
+        co_input = StringIO(bytes)
         try:
             # The config files are always stored utf8-encoded
             self._config_obj = ConfigObj(co_input, encoding='utf-8')
         except configobj.ConfigObjError, e:
             self._config_obj = None
             raise errors.ParseConfigError(e.errors, self.external_url())
+        except UnicodeDecodeError:
+            raise errors.ConfigContentError(self.external_url())
 
     def save(self):
         if not self.is_loaded():
@@ -2298,6 +2403,8 @@
         out = StringIO()
         self._config_obj.write(out)
         self.transport.put_bytes(self.file_name, out.getvalue())
+        for hook in ConfigHooks['save']:
+            hook(self)
 
     def external_url(self):
         # FIXME: external_url should really accepts an optional relpath
@@ -2587,6 +2694,8 @@
                 opt = None
             if opt is not None:
                 value = opt.get_default()
+        for hook in ConfigHooks['get']:
+            hook(self, name, value)
         return value
 
     def _get_mutable_section(self):
@@ -2604,11 +2713,15 @@
         """Set a new value for the option."""
         section = self._get_mutable_section()
         section.set(name, value)
+        for hook in ConfigHooks['set']:
+            hook(self, name, value)
 
     def remove(self, name):
         """Remove an existing option."""
         section = self._get_mutable_section()
         section.remove(name)
+        for hook in ConfigHooks['remove']:
+            hook(self, name)
 
     def __repr__(self):
         # Mostly for debugging use
@@ -2701,6 +2814,8 @@
                         ' the configuration file'),
         ]
 
+    _see_also = ['configuration']
+
     @commands.display_command
     def run(self, name=None, all=False, directory=None, scope=None,
             remove=False):

=== modified file 'bzrlib/conflicts.py'
--- bzrlib/conflicts.py	2011-05-27 00:22:26 +0000
+++ bzrlib/conflicts.py	2011-06-21 15:19:18 +0000
@@ -507,12 +507,11 @@
         if path_to_create is not None:
             tid = tt.trans_id_tree_path(path_to_create)
             transform.create_from_tree(
-                tt, tt.trans_id_tree_path(path_to_create),
-                self._revision_tree(tt._tree, revid), file_id)
+                tt, tid, self._revision_tree(tt._tree, revid), file_id)
             tt.version_file(file_id, tid)
-
+        else:
+            tid = tt.trans_id_file_id(file_id)
         # Adjust the path for the retained file id
-        tid = tt.trans_id_file_id(file_id)
         parent_tid = tt.get_tree_parent(tid)
         tt.adjust_path(osutils.basename(path), parent_tid, tid)
         tt.apply()
@@ -583,7 +582,7 @@
         :param tt: The TreeTransform where the conflict is resolved.
         :param suffix_to_remove: Either 'THIS' or 'OTHER'
 
-        The resolution is symmetric, when taking THIS, OTHER is deleted and
+        The resolution is symmetric: when taking THIS, OTHER is deleted and
         item.THIS is renamed into item and vice-versa.
         """
         try:
@@ -596,12 +595,23 @@
             # never existed or was already deleted (including the case
             # where the user deleted it)
             pass
-        # Rename 'item.suffix_to_remove' (note that if
-        # 'item.suffix_to_remove' has been deleted, this is a no-op)
-        this_tid = tt.trans_id_file_id(self.file_id)
-        parent_tid = tt.get_tree_parent(this_tid)
-        tt.adjust_path(osutils.basename(self.path), parent_tid, this_tid)
-        tt.apply()
+        try:
+            this_path = tt._tree.id2path(self.file_id)
+        except errors.NoSuchId:
+            # The file is not present anymore. This may happen if the user
+            # deleted the file either manually or when resolving a conflict on
+            # the parent.  We may raise some exception to indicate that the
+            # conflict doesn't exist anymore and as such doesn't need to be
+            # resolved ? -- vila 20110615 
+            this_tid = None
+        else:
+            this_tid = tt.trans_id_tree_path(this_path)
+        if this_tid is not None:
+            # Rename 'item.suffix_to_remove' (note that if
+            # 'item.suffix_to_remove' has been deleted, this is a no-op)
+            parent_tid = tt.get_tree_parent(this_tid)
+            tt.adjust_path(osutils.basename(self.path), parent_tid, this_tid)
+            tt.apply()
 
     def action_take_this(self, tree):
         self._resolve_with_cleanups(tree, 'OTHER')

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2011-06-02 16:57:09 +0000
+++ bzrlib/errors.py	2011-06-20 13:58:14 +0000
@@ -1770,6 +1770,15 @@
     _fmt = "Working tree has conflicts."
 
 
+class ConfigContentError(BzrError):
+
+    _fmt = "Config file %(filename)s is not UTF-8 encoded\n"
+
+    def __init__(self, filename):
+        BzrError.__init__(self)
+        self.filename = filename
+
+
 class ParseConfigError(BzrError):
 
     _fmt = "Error(s) parsing config file %(filename)s:\n%(errors)s"

=== modified file 'bzrlib/export/dir_exporter.py'
--- bzrlib/export/dir_exporter.py	2011-06-13 16:34:53 +0000
+++ bzrlib/export/dir_exporter.py	2011-06-28 13:55:39 +0000
@@ -96,7 +96,11 @@
         if force_mtime is not None:
             mtime = force_mtime
         else:
-            mtime = tree.get_file_mtime(tree.path2id(relpath), relpath)
+            if subdir is None:
+                file_id = tree.path2id(relpath)
+            else:
+                file_id = tree.path2id(osutils.pathjoin(subdir, relpath))
+            mtime = tree.get_file_mtime(file_id, relpath)
         os.utime(fullpath, (mtime, mtime))
 
         yield

=== modified file 'bzrlib/export_pot.py'
--- bzrlib/export_pot.py	2011-05-12 12:19:35 +0000
+++ bzrlib/export_pot.py	2011-05-28 12:17:22 +0000
@@ -76,10 +76,12 @@
            'msgid %s\n' % _normalize(s) +
            'msgstr ""\n')
 
-def _poentry_per_paragraph(outf, path, lineno, msgid):
+def _poentry_per_paragraph(outf, path, lineno, msgid, filter=lambda x: False):
     # TODO: How to split long help?
     paragraphs = msgid.split('\n\n')
     for p in paragraphs:
+        if filter(p):
+            continue
         _poentry(outf, path, lineno, p)
         lineno += p.count('\n') + 2
 
@@ -145,7 +147,7 @@
                      'help of %r option of %r command' % (name, cmd.name()))
 
 
-def _write_command_help(outf, cmd_name, cmd):
+def _write_command_help(outf, cmd):
     path = inspect.getfile(cmd.__class__)
     if path.endswith('.pyc'):
         path = path[:-1]
@@ -155,9 +157,16 @@
     lineno = offsets[cmd.__doc__]
     doc = inspect.getdoc(cmd)
 
-    _poentry_per_paragraph(outf, path, lineno, doc)
+    def filter(p):
+        # ':Usage:' has special meaning in help topics.
+        # This is usage example of command and should not be translated.
+        if p.splitlines()[0] == ':Usage:':
+            return True
+
+    _poentry_per_paragraph(outf, path, lineno, doc, filter)
     _command_options(outf, path, cmd)
 
+
 def _command_helps(outf):
     """Extract docstrings from path.
 
@@ -172,7 +181,7 @@
         if command.hidden:
             continue
         note("Exporting messages from builtin command: %s", cmd_name)
-        _write_command_help(outf, cmd_name, command)
+        _write_command_help(outf, command)
 
     plugin_path = plugin.get_core_plugin_path()
     core_plugins = glob(plugin_path + '/*/__init__.py')
@@ -189,7 +198,7 @@
             continue
         note("Exporting messages from plugin command: %s in %s",
              cmd_name, command.plugin_name())
-        _write_command_help(outf, cmd_name, command)
+        _write_command_help(outf, command)
 
 
 def _error_messages(outf):

=== modified file 'bzrlib/graph.py'
--- bzrlib/graph.py	2011-05-20 13:28:35 +0000
+++ bzrlib/graph.py	2011-06-20 11:03:53 +0000
@@ -1980,7 +1980,10 @@
         return set([h[0] for h in head_keys])
 
     def merge_sort(self, tip_revision):
-        return self._graph.merge_sort((tip_revision,))
+        nodes = self._graph.merge_sort((tip_revision,))
+        for node in nodes:
+            node.key = node.key[0]
+        return nodes
 
     def add_node(self, revision, parents):
         self._graph.add_node((revision,), [(p,) for p in parents])

=== modified file 'bzrlib/help.py'
--- bzrlib/help.py	2011-05-11 16:52:02 +0000
+++ bzrlib/help.py	2011-05-18 18:32:22 +0000
@@ -22,7 +22,6 @@
 # TODO: `help commands --all` should show hidden commands
 
 import sys
-import textwrap
 
 from bzrlib import (
     commands as _mod_commands,
@@ -30,6 +29,7 @@
     help_topics,
     osutils,
     plugin,
+    utextwrap,
     )
 
 
@@ -96,7 +96,7 @@
         else:
             firstline = ''
         helpstring = '%-*s %s%s' % (max_name, cmd_name, firstline, plugin_name)
-        lines = textwrap.wrap(
+        lines = utextwrap.wrap(
             helpstring, subsequent_indent=indent,
             width=width,
             break_long_words=False)

=== modified file 'bzrlib/help_topics/__init__.py'
--- bzrlib/help_topics/__init__.py	2010-11-06 10:55:58 +0000
+++ bzrlib/help_topics/__init__.py	2011-05-28 13:46:51 +0000
@@ -316,6 +316,7 @@
 --builtin      Use the built-in version of a command, not the plugin version.
                This does not suppress other plugin effects.
 --no-plugins   Do not process any plugins.
+--no-l10n      Do not translate messages.
 --concurrency  Number of processes that can be run concurrently (selftest).
 
 --profile      Profile execution using the hotshot profiler.

=== modified file 'bzrlib/help_topics/en/configuration.txt'
--- bzrlib/help_topics/en/configuration.txt	2011-05-24 14:40:29 +0000
+++ bzrlib/help_topics/en/configuration.txt	2011-06-14 10:47:20 +0000
@@ -386,7 +386,8 @@
 check_signatures
 ~~~~~~~~~~~~~~~~
 
-Defines the behavior for signatures.
+Reserved for future use.  These options will allow a policy for branches to
+require signatures.
 
 require
     The gnupg signature for revisions must be present and must be valid.
@@ -402,18 +403,22 @@
 create_signatures
 ~~~~~~~~~~~~~~~~~
 
-Defines the behaviour of signing revisions.
+Defines the behaviour of signing revisions on commits.  By default bzr will not
+sign new commits.
 
 always
-    Sign every new revision that is committed.
+    Sign every new revision that is committed.  If the signing fails then the
+    commit will not be made.
 
 when-required
-    (default) Sign newly committed revisions only when the branch requires
-    signed revisions.
+    Reserved for future use.
 
 never
-    Refuse to sign newly committed revisions, even if the branch
-    requires signatures.
+    Reserved for future use.
+
+In future it is planned that ``when-required`` will sign newly
+committed revisions only when the branch requires them.  ``never`` will refuse
+to sign newly committed revisions, even if the branch requires signatures.
 
 recurse
 ~~~~~~~
@@ -462,6 +467,14 @@
 These settings are only needed if the SMTP server requires authentication
 to send mail.
 
+locks.steal_dead
+~~~~~~~~~~~~~~~~
+
+If set to true, bzr will automatically break locks held by processes from
+the same machine and user that are no longer alive.  Otherwise, it will
+print a message and you can break the lock manually, if you are satisfied
+the object is no longer in use.
+
 mail_client
 ~~~~~~~~~~~
 

=== modified file 'bzrlib/hooks.py'
--- bzrlib/hooks.py	2011-04-07 10:36:24 +0000
+++ bzrlib/hooks.py	2011-06-14 10:07:16 +0000
@@ -72,6 +72,7 @@
     ('bzrlib.branch', 'Branch.hooks', 'BranchHooks'),
     ('bzrlib.bzrdir', 'BzrDir.hooks', 'BzrDirHooks'),
     ('bzrlib.commands', 'Command.hooks', 'CommandHooks'),
+    ('bzrlib.config', 'ConfigHooks', '_ConfigHooks'),
     ('bzrlib.info', 'hooks', 'InfoHooks'),
     ('bzrlib.lock', 'Lock.hooks', 'LockHooks'),
     ('bzrlib.merge', 'Merger.hooks', 'MergeHooks'),
@@ -256,8 +257,7 @@
         try:
             uninstall = getattr(hook, "uninstall")
         except AttributeError:
-            raise errors.UnsupportedOperation(self.install_named_hook_lazy,
-                self)
+            raise errors.UnsupportedOperation(self.uninstall_named_hook, self)
         else:
             uninstall(label)
 

=== modified file 'bzrlib/i18n.py'
--- bzrlib/i18n.py	2011-05-27 21:38:33 +0000
+++ bzrlib/i18n.py	2011-06-27 15:08:26 +0000
@@ -26,7 +26,7 @@
 import os
 import sys
 
-_translation = _gettext.NullTranslations()
+_translations = None
 
 
 def gettext(message):
@@ -34,7 +34,7 @@
     
     :returns: translated message as unicode.
     """
-    return _translation.ugettext(message)
+    return _translations.ugettext(message)
 
 
 def ngettext(s, p, n):
@@ -42,7 +42,7 @@
 
     :returns: translated message as unicode.
     """
-    return _translation.ungettext(s, p, n)
+    return _translations.ungettext(s, p, n)
 
 
 def N_(msg):
@@ -56,17 +56,21 @@
     :returns: concatenated translated message as unicode.
     """
     paragraphs = message.split(u'\n\n')
-    ugettext = _translation.ugettext
+    ugettext = _translations.ugettext
     # Be careful not to translate the empty string -- it holds the
     # meta data of the .po file.
     return u'\n\n'.join(ugettext(p) if p else u'' for p in paragraphs)
 
 
+def installed():
+    return _translations is not None
+
+
 def install(lang=None):
-    global _translation
+    global _translations
     if lang is None:
         lang = _get_current_locale()
-    _translation = _gettext.translation(
+    _translations = _gettext.translation(
             'bzr',
             localedir=_get_locale_dir(),
             languages=lang.split(':'),
@@ -74,8 +78,8 @@
 
 
 def uninstall():
-    global _translation
-    _translation = _null_translation
+    global _translations
+    _translations = None
 
 
 def _get_locale_dir():

=== modified file 'bzrlib/lockdir.py'
--- bzrlib/lockdir.py	2011-05-16 13:39:39 +0000
+++ bzrlib/lockdir.py	2011-06-13 01:13:22 +0000
@@ -92,6 +92,10 @@
 >>> # do something here
 >>> l.unlock()
 
+Some classes of stale locks can be predicted by checking: the host name is the
+same as the local host name; the user name is the same as the local user; the
+process id no longer exists.  The check on user name is not strictly necessary
+but helps protect against colliding host names.
 """
 
 
@@ -103,16 +107,19 @@
 # the existing locking code and needs a new format of the containing object.
 # -- robertc, mbp 20070628
 
+import errno
 import os
 import time
 
 from bzrlib import (
+    config,
     debug,
     errors,
     lock,
     osutils,
+    ui,
+    urlutils,
     )
-import bzrlib.config
 from bzrlib.decorators import only_raises
 from bzrlib.errors import (
         DirectoryNotEmpty,
@@ -130,7 +137,6 @@
         )
 from bzrlib.trace import mutter, note
 from bzrlib.osutils import format_delta, rand_chars, get_host_name
-import bzrlib.ui
 
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
@@ -162,7 +168,8 @@
 
     __INFO_NAME = '/info'
 
-    def __init__(self, transport, path, file_modebits=0644, dir_modebits=0755):
+    def __init__(self, transport, path, file_modebits=0644, dir_modebits=0755,
+        extra_holder_info=None):
         """Create a new LockDir object.
 
         The LockDir is initially unlocked - this just creates the object.
@@ -171,6 +178,10 @@
 
         :param path: Path to the lock within the base directory of the
             transport.
+
+        :param extra_holder_info: If passed, {str:str} dict of extra or
+            updated information to insert into the info file when the lock is
+            taken.
         """
         self.transport = transport
         self.path = path
@@ -181,8 +192,9 @@
         self._held_info_path = self._held_dir + self.__INFO_NAME
         self._file_modebits = file_modebits
         self._dir_modebits = dir_modebits
-
         self._report_function = note
+        self.extra_holder_info = extra_holder_info
+        self._warned_about_lock_holder = None
 
     def __repr__(self):
         return '%s(%s%s)' % (self.__class__.__name__,
@@ -203,7 +215,6 @@
         except (TransportError, PathError), e:
             raise LockFailed(self, e)
 
-
     def _attempt_lock(self):
         """Make the pending directory and attempt to rename into place.
 
@@ -216,8 +227,8 @@
 
         :returns: The nonce of the lock, if it was successfully acquired.
 
-        :raises LockContention: If the lock is held by someone else.  The exception
-            contains the info of the current holder of the lock.
+        :raises LockContention: If the lock is held by someone else.  The
+            exception contains the info of the current holder of the lock.
         """
         self._trace("lock_write...")
         start_time = time.time()
@@ -226,17 +237,24 @@
         except (errors.TransportError, PathError), e:
             self._trace("... failed to create pending dir, %s", e)
             raise LockFailed(self, e)
-        try:
-            self.transport.rename(tmpname, self._held_dir)
-        except (errors.TransportError, PathError, DirectoryNotEmpty,
-                FileExists, ResourceBusy), e:
-            self._trace("... contention, %s", e)
-            self._remove_pending_dir(tmpname)
-            raise LockContention(self)
-        except Exception, e:
-            self._trace("... lock failed, %s", e)
-            self._remove_pending_dir(tmpname)
-            raise
+        while True:
+            try:
+                self.transport.rename(tmpname, self._held_dir)
+                break
+            except (errors.TransportError, PathError, DirectoryNotEmpty,
+                    FileExists, ResourceBusy), e:
+                self._trace("... contention, %s", e)
+                other_holder = self.peek()
+                self._trace("other holder is %r" % other_holder)
+                try:
+                    self._handle_lock_contention(other_holder)
+                except:
+                    self._remove_pending_dir(tmpname)
+                    raise
+            except Exception, e:
+                self._trace("... lock failed, %s", e)
+                self._remove_pending_dir(tmpname)
+                raise
         # We must check we really got the lock, because Launchpad's sftp
         # server at one time had a bug were the rename would successfully
         # move the new directory into the existing directory, which was
@@ -262,6 +280,33 @@
                 (time.time() - start_time) * 1000)
         return self.nonce
 
+    def _handle_lock_contention(self, other_holder):
+        """A lock we want to take is held by someone else.
+
+        This function can: tell the user about it; possibly detect that it's
+        safe or appropriate to steal the lock, or just raise an exception.
+
+        If this function returns (without raising an exception) the lock will
+        be attempted again.
+
+        :param other_holder: A LockHeldInfo for the current holder; note that
+            it might be None if the lock can be seen to be held but the info
+            can't be read.
+        """
+        if (other_holder is not None):
+            if other_holder.is_lock_holder_known_dead():
+                if self.get_config().get_user_option_as_bool(
+                    'locks.steal_dead',
+                    default=False):
+                    ui.ui_factory.show_user_warning(
+                        'locks_steal_dead',
+                        lock_url=urlutils.join(self.transport.base, self.path),
+                        other_holder_info=unicode(other_holder))
+                    self.force_break(other_holder)
+                    self._trace("stole lock from dead holder")
+                    return
+        raise LockContention(self)
+
     def _remove_pending_dir(self, tmpname):
         """Remove the pending directory
 
@@ -287,14 +332,14 @@
             self.create(mode=self._dir_modebits)
             # After creating the lock directory, try again
             self.transport.mkdir(tmpname)
-        self.nonce = rand_chars(20)
-        info_bytes = self._prepare_info()
+        info = LockHeldInfo.for_this_process(self.extra_holder_info)
+        self.nonce = info.get('nonce')
         # We use put_file_non_atomic because we just created a new unique
         # directory so we don't have to worry about files existing there.
         # We'll rename the whole directory into place to get atomic
         # properties
         self.transport.put_bytes_non_atomic(tmpname + self.__INFO_NAME,
-                                            info_bytes)
+            info.to_bytes())
         return tmpname
 
     @only_raises(LockNotHeld, LockBroken)
@@ -344,9 +389,10 @@
     def break_lock(self):
         """Break a lock not held by this instance of LockDir.
 
-        This is a UI centric function: it uses the bzrlib.ui.ui_factory to
+        This is a UI centric function: it uses the ui.ui_factory to
         prompt for input if a lock is detected and there is any doubt about
-        it possibly being still active.
+        it possibly being still active.  force_break is the non-interactive
+        version.
 
         :returns: LockResult for the broken lock.
         """
@@ -355,16 +401,16 @@
             holder_info = self.peek()
         except LockCorrupt, e:
             # The lock info is corrupt.
-            if bzrlib.ui.ui_factory.get_boolean(u"Break (corrupt %r)" % (self,)):
+            if ui.ui_factory.get_boolean(u"Break (corrupt %r)" % (self,)):
                 self.force_break_corrupt(e.file_data)
             return
         if holder_info is not None:
-            lock_info = '\n'.join(self._format_lock_info(holder_info))
-            if bzrlib.ui.ui_factory.confirm_action(
-                "Break %(lock_info)s", 'bzrlib.lockdir.break', 
-                dict(lock_info=lock_info)):
+            if ui.ui_factory.confirm_action(
+                u"Break %(lock_info)s",
+                'bzrlib.lockdir.break',
+                dict(lock_info=unicode(holder_info))):
                 result = self.force_break(holder_info)
-                bzrlib.ui.ui_factory.show_message(
+                ui.ui_factory.show_message(
                     "Broke lock %s" % result.lock_url)
 
     def force_break(self, dead_holder_info):
@@ -374,18 +420,19 @@
         it still thinks it has the lock there will be two concurrent writers.
         In general the user's approval should be sought for lock breaks.
 
-        dead_holder_info must be the result of a previous LockDir.peek() call;
-        this is used to check that it's still held by the same process that
-        the user decided was dead.  If this is not the current holder,
-        LockBreakMismatch is raised.
-
         After the lock is broken it will not be held by any process.
         It is possible that another process may sneak in and take the
         lock before the breaking process acquires it.
 
+        :param dead_holder_info:
+            Must be the result of a previous LockDir.peek() call; this is used
+            to check that it's still held by the same process that the user
+            decided was dead.  If this is not the current holder,
+            LockBreakMismatch is raised.
+
         :returns: LockResult for the broken lock.
         """
-        if not isinstance(dead_holder_info, dict):
+        if not isinstance(dead_holder_info, LockHeldInfo):
             raise ValueError("dead_holder_info: %r" % dead_holder_info)
         self._check_not_locked()
         current_info = self.peek()
@@ -413,10 +460,10 @@
 
     def force_break_corrupt(self, corrupt_info_lines):
         """Release a lock that has been corrupted.
-        
+
         This is very similar to force_break, it except it doesn't assume that
         self.peek() can work.
-        
+
         :param corrupt_info_lines: the lines of the corrupted info file, used
             to check that the lock hasn't changed between reading the (corrupt)
             info file and calling force_break_corrupt.
@@ -470,7 +517,8 @@
 
         peek() reads the info file of the lock holder, if any.
         """
-        return self._parse_info(self.transport.get_bytes(path))
+        return LockHeldInfo.from_info_file_bytes(
+            self.transport.get_bytes(path))
 
     def peek(self):
         """Check if the lock is held by anyone.
@@ -489,35 +537,6 @@
     def _prepare_info(self):
         """Write information about a pending lock to a temporary file.
         """
-        # XXX: is creating this here inefficient?
-        config = bzrlib.config.GlobalConfig()
-        try:
-            user = config.username()
-        except errors.NoWhoami:
-            user = osutils.getuser_unicode()
-        s = rio.Stanza(hostname=get_host_name(),
-                   pid=str(os.getpid()),
-                   start_time=str(int(time.time())),
-                   nonce=self.nonce,
-                   user=user,
-                   )
-        return s.to_string()
-
-    def _parse_info(self, info_bytes):
-        lines = osutils.split_lines(info_bytes)
-        try:
-            stanza = rio.read_stanza(lines)
-        except ValueError, e:
-            mutter('Corrupt lock info file: %r', lines)
-            raise LockCorrupt("could not parse lock info file: " + str(e),
-                              lines)
-        if stanza is None:
-            # see bug 185013; we fairly often end up with the info file being
-            # empty after an interruption; we could log a message here but
-            # there may not be much we can say
-            return {}
-        else:
-            return stanza.as_dict()
 
     def attempt_lock(self):
         """Take the lock; fail if it's already held.
@@ -598,24 +617,16 @@
                 else:
                     start = 'Lock owner changed for'
                 last_info = new_info
-                formatted_info = self._format_lock_info(new_info)
+                msg = u'%s lock %s %s.' % (start, lock_url, new_info)
                 if deadline_str is None:
                     deadline_str = time.strftime('%H:%M:%S',
-                                                 time.localtime(deadline))
-                user, hostname, pid, time_ago = formatted_info
-                msg = ('%s lock %s '        # lock_url
-                    'held by '              # start
-                    '%s\n'                  # user
-                    'at %s '                # hostname
-                    '[process #%s], '       # pid
-                    'acquired %s.')         # time ago
-                msg_args = [start, lock_url, user, hostname, pid, time_ago]
+                                                    time.localtime(deadline))
                 if timeout > 0:
                     msg += ('\nWill continue to try until %s, unless '
-                        'you press Ctrl-C.')
-                    msg_args.append(deadline_str)
+                        'you press Ctrl-C.'
+                        % deadline_str)
                 msg += '\nSee "bzr help break-lock" for more.'
-                self._report_function(msg, *msg_args)
+                self._report_function(msg)
             if (max_attempts is not None) and (attempt_count >= max_attempts):
                 self._trace("exceeded %d attempts")
                 raise LockContention(self)
@@ -676,23 +687,6 @@
             raise LockContention(self)
         self._fake_read_lock = True
 
-    def _format_lock_info(self, info):
-        """Turn the contents of peek() into something for the user"""
-        start_time = info.get('start_time')
-        if start_time is None:
-            time_ago = '(unknown)'
-        else:
-            time_ago = format_delta(time.time() - int(info['start_time']))
-        user = info.get('user', '<unknown>')
-        hostname = info.get('hostname', '<unknown>')
-        pid = info.get('pid', '<unknown>')
-        return [
-            user,
-            hostname,
-            pid,
-            time_ago,
-            ]
-
     def validate_token(self, token):
         if token is not None:
             info = self.peek()
@@ -710,3 +704,158 @@
         if 'lock' not in debug.debug_flags:
             return
         mutter(str(self) + ": " + (format % args))
+
+    def get_config(self):
+        """Get the configuration that governs this lockdir."""
+        # XXX: This really should also use the locationconfig at least, but
+        # that seems a bit hard to hook up at the moment. -- mbp 20110329
+        return config.GlobalConfig()
+
+
+class LockHeldInfo(object):
+    """The information recorded about a held lock.
+
+    This information is recorded into the lock when it's taken, and it can be
+    read back by any process with access to the lockdir.  It can be used, for
+    example, to tell the user who holds the lock, or to try to detect whether
+    the lock holder is still alive.
+
+    Prior to bzr 2.4 a simple dict was used instead of an object.
+    """
+
+    def __init__(self, info_dict):
+        self.info_dict = info_dict
+
+    def __repr__(self):
+        """Return a debugging representation of this object."""
+        return "%s(%r)" % (self.__class__.__name__, self.info_dict)
+
+    def __unicode__(self):
+        """Return a user-oriented description of this object."""
+        d = self.to_readable_dict()
+        return (
+            u'held by %(user)s on %(hostname)s (process #%(pid)s), '
+            u'acquired %(time_ago)s' % d)
+
+    def to_readable_dict(self):
+        """Turn the holder info into a dict of human-readable attributes.
+
+        For example, the start time is presented relative to the current time,
+        rather than as seconds since the epoch.
+
+        Returns a list of [user, hostname, pid, time_ago] all as readable
+        strings.
+        """
+        start_time = self.info_dict.get('start_time')
+        if start_time is None:
+            time_ago = '(unknown)'
+        else:
+            time_ago = format_delta(
+                time.time() - int(self.info_dict['start_time']))
+        user = self.info_dict.get('user', '<unknown>')
+        hostname = self.info_dict.get('hostname', '<unknown>')
+        pid = self.info_dict.get('pid', '<unknown>')
+        return dict(
+            user=user,
+            hostname=hostname,
+            pid=pid,
+            time_ago=time_ago)
+
+    def get(self, field_name):
+        """Return the contents of a field from the lock info, or None."""
+        return self.info_dict.get(field_name)
+
+    @classmethod
+    def for_this_process(cls, extra_holder_info):
+        """Return a new LockHeldInfo for a lock taken by this process.
+        """
+        info = dict(
+            hostname=get_host_name(),
+            pid=str(os.getpid()),
+            nonce=rand_chars(20),
+            start_time=str(int(time.time())),
+            user=get_username_for_lock_info(),
+            )
+        if extra_holder_info is not None:
+            info.update(extra_holder_info)
+        return cls(info)
+
+    def to_bytes(self):
+        s = rio.Stanza(**self.info_dict)
+        return s.to_string()
+
+    @classmethod
+    def from_info_file_bytes(cls, info_file_bytes):
+        """Construct from the contents of the held file."""
+        lines = osutils.split_lines(info_file_bytes)
+        try:
+            stanza = rio.read_stanza(lines)
+        except ValueError, e:
+            mutter('Corrupt lock info file: %r', lines)
+            raise LockCorrupt("could not parse lock info file: " + str(e),
+                lines)
+        if stanza is None:
+            # see bug 185013; we fairly often end up with the info file being
+            # empty after an interruption; we could log a message here but
+            # there may not be much we can say
+            return cls({})
+        else:
+            return cls(stanza.as_dict())
+
+    def __cmp__(self, other):
+        """Value comparison of lock holders."""
+        return (
+            cmp(type(self), type(other))
+            or cmp(self.info_dict, other.info_dict))
+
+    def is_locked_by_this_process(self):
+        """True if this process seems to be the current lock holder."""
+        return (
+            self.get('hostname') == get_host_name()
+            and self.get('pid') == str(os.getpid())
+            and self.get('user') == get_username_for_lock_info())
+
+    def is_lock_holder_known_dead(self):
+        """True if the lock holder process is known to be dead.
+
+        False if it's either known to be still alive, or if we just can't tell.
+
+        We can be fairly sure the lock holder is dead if it declared the same
+        hostname and there is no process with the given pid alive.  If people
+        have multiple machines with the same hostname this may cause trouble.
+
+        This doesn't check whether the lock holder is in fact the same process
+        calling this method.  (In that case it will return true.)
+        """
+        if self.get('hostname') != get_host_name():
+            return False
+        if self.get('hostname') == 'localhost':
+            # Too ambiguous.
+            return False
+        if self.get('user') != get_username_for_lock_info():
+            # Could well be another local process by a different user, but
+            # just to be safe we won't conclude about this either.
+            return False
+        pid_str = self.info_dict.get('pid', None)
+        if not pid_str:
+            mutter("no pid recorded in %r" % (self, ))
+            return False
+        try:
+            pid = int(pid_str)
+        except ValueError:
+            mutter("can't parse pid %r from %r"
+                % (pid_str, self))
+            return False
+        return osutils.is_local_pid_dead(pid)
+
+
+def get_username_for_lock_info():
+    """Get a username suitable for putting into a lock.
+
+    It's ok if what's written here is not a proper email address as long
+    as it gives some clue who the user is.
+    """
+    try:
+        return config.GlobalConfig().username()
+    except errors.NoWhoami:
+        return osutils.getuser_unicode()

=== modified file 'bzrlib/log.py'
--- bzrlib/log.py	2011-05-19 14:45:40 +0000
+++ bzrlib/log.py	2011-06-16 12:50:32 +0000
@@ -678,16 +678,19 @@
     """
     br_revno, br_rev_id = branch.last_revision_info()
     repo = branch.repository
+    graph = repo.get_graph()
     if start_rev_id is None and end_rev_id is None:
         cur_revno = br_revno
-        for revision_id in repo.iter_reverse_revision_history(br_rev_id):
+        for revision_id in graph.iter_lefthand_ancestry(br_rev_id,
+            (_mod_revision.NULL_REVISION,)):
             yield revision_id, str(cur_revno), 0
             cur_revno -= 1
     else:
         if end_rev_id is None:
             end_rev_id = br_rev_id
         found_start = start_rev_id is None
-        for revision_id in repo.iter_reverse_revision_history(end_rev_id):
+        for revision_id in graph.iter_lefthand_ancestry(end_rev_id,
+                (_mod_revision.NULL_REVISION,)):
             revno_str = _compute_revno_str(branch, revision_id)
             if not found_start and revision_id == start_rev_id:
                 if not exclude_common_ancestry:
@@ -1086,8 +1089,9 @@
     cur_revno = branch_revno
     rev_nos = {}
     mainline_revs = []
-    for revision_id in branch.repository.iter_reverse_revision_history(
-                        branch_last_revision):
+    graph = branch.repository.get_graph()
+    for revision_id in graph.iter_lefthand_ancestry(
+            branch_last_revision, (_mod_revision.NULL_REVISION,)):
         if cur_revno < start_revno:
             # We have gone far enough, but we always add 1 more revision
             rev_nos[revision_id] = cur_revno
@@ -1945,8 +1949,9 @@
     old_revisions = set()
     new_history = []
     new_revisions = set()
-    new_iter = repository.iter_reverse_revision_history(new_revision_id)
-    old_iter = repository.iter_reverse_revision_history(old_revision_id)
+    graph = repository.get_graph()
+    new_iter = graph.iter_lefthand_ancestry(new_revision_id)
+    old_iter = graph.iter_lefthand_ancestry(old_revision_id)
     stop_revision = None
     do_old = True
     do_new = True

=== modified file 'bzrlib/merge.py'
--- bzrlib/merge.py	2011-05-21 16:43:19 +0000
+++ bzrlib/merge.py	2011-06-21 22:02:33 +0000
@@ -868,7 +868,7 @@
                     executable3, file_status, resolver=resolver)
         finally:
             child_pb.finished()
-        self.fix_root()
+        self.tt.fixup_new_roots()
         self._finish_computing_transform()
 
     def _finish_computing_transform(self):
@@ -1093,6 +1093,7 @@
                           ))
         return result
 
+    @deprecated_method(deprecated_in((2, 4, 0)))
     def fix_root(self):
         if self.tt.final_kind(self.tt.root) is None:
             self.tt.cancel_deletion(self.tt.root)
@@ -1316,16 +1317,26 @@
             self._raw_conflicts.append(('path conflict', trans_id, file_id,
                                         this_parent, this_name,
                                         other_parent, other_name))
-        if other_name is None:
+        if not self.other_tree.has_id(file_id):
             # it doesn't matter whether the result was 'other' or
-            # 'conflict'-- if there's no 'other', we leave it alone.
+            # 'conflict'-- if it has no file id, we leave it alone.
             return
         parent_id = parents[self.winner_idx[parent_id_winner]]
-        if parent_id is not None:
+        name = names[self.winner_idx[name_winner]]
+        if parent_id is not None or name is not None:
             # if we get here, name_winner and parent_winner are set to safe
             # values.
-            self.tt.adjust_path(names[self.winner_idx[name_winner]],
-                                self.tt.trans_id_file_id(parent_id),
+            if parent_id is None and name is not None:
+                # if parent_id is None and name is non-None, current file is
+                # the tree root.
+                if names[self.winner_idx[parent_id_winner]] != '':
+                    raise AssertionError(
+                        'File looks like a root, but named %s' %
+                        names[self.winner_idx[parent_id_winner]])
+                parent_trans_id = transform.ROOT_PARENT
+            else:
+                parent_trans_id = self.tt.trans_id_file_id(parent_id)
+            self.tt.adjust_path(name, parent_trans_id,
                                 self.tt.trans_id_file_id(file_id))
 
     def _do_merge_contents(self, file_id):
@@ -1605,8 +1616,8 @@
 
     def cook_conflicts(self, fs_conflicts):
         """Convert all conflicts into a form that doesn't depend on trans_id"""
-        self.cooked_conflicts.extend(transform.cook_conflicts(
-                fs_conflicts, self.tt))
+        content_conflict_file_ids = set()
+        cooked_conflicts = transform.cook_conflicts(fs_conflicts, self.tt)
         fp = transform.FinalPaths(self.tt)
         for conflict in self._raw_conflicts:
             conflict_type = conflict[0]
@@ -1642,6 +1653,7 @@
                         break
                 c = _mod_conflicts.Conflict.factory(conflict_type,
                                                     path=path, file_id=file_id)
+                content_conflict_file_ids.add(file_id)
             elif conflict_type == 'text conflict':
                 trans_id = conflict[1]
                 path = fp.get_path(trans_id)
@@ -1650,6 +1662,17 @@
                                                     path=path, file_id=file_id)
             else:
                 raise AssertionError('bad conflict type: %r' % (conflict,))
+            cooked_conflicts.append(c)
+
+        self.cooked_conflicts = []
+        # We want to get rid of path conflicts when a corresponding contents
+        # conflict exists. This can occur when one branch deletes a file while
+        # the other renames *and* modifies it. In this case, the content
+        # conflict is enough.
+        for c in cooked_conflicts:
+            if (c.typestring == 'path conflict'
+                and c.file_id in content_conflict_file_ids):
+                continue
             self.cooked_conflicts.append(c)
         self.cooked_conflicts.sort(key=_mod_conflicts.Conflict.sort_key)
 

=== modified file 'bzrlib/mutabletree.py'
--- bzrlib/mutabletree.py	2011-05-20 15:27:35 +0000
+++ bzrlib/mutabletree.py	2011-06-16 13:28:23 +0000
@@ -554,6 +554,8 @@
         entry = self._invdelta.get(inv_path)
         if entry is not None:
             return entry[3]
+        # Find a 'best fit' match if the filesystem is case-insensitive
+        inv_path = self.tree._fix_case_of_inventory_path(inv_path)
         file_id = self.tree.path2id(inv_path)
         if file_id is not None:
             return self.tree.iter_entries_by_dir([file_id]).next()[1]

=== modified file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	2011-05-26 08:05:45 +0000
+++ bzrlib/osutils.py	2011-06-16 18:34:26 +0000
@@ -2381,6 +2381,18 @@
     except UnicodeDecodeError:
         raise errors.BzrError("Can't decode username as %s." % \
                 user_encoding)
+    except ImportError, e:
+        if sys.platform != 'win32':
+            raise
+        if str(e) != 'No module named pwd':
+            raise
+        # https://bugs.launchpad.net/bzr/+bug/660174
+        # getpass.getuser() is unable to return username on Windows
+        # if there is no USERNAME environment variable set.
+        # That could be true if bzr is running as a service,
+        # e.g. running `bzr serve` as a service on Windows.
+        # We should not fail with traceback in this case.
+        username = u'UNKNOWN'
     return username
 
 
@@ -2448,3 +2460,30 @@
             if os.access(f, os.X_OK):
                 return f
     return None
+
+
+def _posix_is_local_pid_dead(pid):
+    """True if pid doesn't correspond to live process on this machine"""
+    try:
+        # Special meaning of unix kill: just check if it's there.
+        os.kill(pid, 0)
+    except OSError, e:
+        if e.errno == errno.ESRCH:
+            # On this machine, and really not found: as sure as we can be
+            # that it's dead.
+            return True
+        elif e.errno == errno.EPERM:
+            # exists, though not ours
+            return False
+        else:
+            mutter("os.kill(%d, 0) failed: %s" % (pid, e))
+            # Don't really know.
+            return False
+    else:
+        # Exists and our process: not dead.
+        return False
+
+if sys.platform == "win32":
+    is_local_pid_dead = win32utils.is_local_pid_dead
+else:
+    is_local_pid_dead = _posix_is_local_pid_dead

=== modified file 'bzrlib/reconcile.py'
--- bzrlib/reconcile.py	2011-05-18 16:42:48 +0000
+++ bzrlib/reconcile.py	2011-06-16 16:06:33 +0000
@@ -29,6 +29,7 @@
 from bzrlib import (
     cleanup,
     errors,
+    revision as _mod_revision,
     ui,
     )
 from bzrlib.trace import mutter
@@ -144,12 +145,12 @@
         self._reconcile_revision_history()
 
     def _reconcile_revision_history(self):
-        repo = self.branch.repository
         last_revno, last_revision_id = self.branch.last_revision_info()
         real_history = []
+        graph = self.branch.repository.get_graph()
         try:
-            for revid in repo.iter_reverse_revision_history(
-                    last_revision_id):
+            for revid in graph.iter_lefthand_ancestry(
+                    last_revision_id, (_mod_revision.NULL_REVISION,)):
                 real_history.append(revid)
         except errors.RevisionNotPresent:
             pass # Hit a ghost left hand parent

=== modified file 'bzrlib/remote.py'
--- bzrlib/remote.py	2011-05-20 13:28:35 +0000
+++ bzrlib/remote.py	2011-06-18 13:57:17 +0000
@@ -32,7 +32,6 @@
     static_tuple,
     symbol_versioning,
     urlutils,
-    versionedfile,
     vf_repository,
     )
 from bzrlib.branch import BranchReferenceFormat, BranchWriteLockResult
@@ -1661,6 +1660,8 @@
         self._real_repository.create_bundle(target, base, fileobj, format)
 
     @needs_read_lock
+    @symbol_versioning.deprecated_method(
+        symbol_versioning.deprecated_in((2, 4, 0)))
     def get_ancestry(self, revision_id, topo_sorted=True):
         self._ensure_real()
         return self._real_repository.get_ancestry(revision_id, topo_sorted)
@@ -1974,6 +1975,7 @@
     def supports_rich_root(self):
         return self._format.rich_root_data
 
+    @symbol_versioning.deprecated_method(symbol_versioning.deprecated_in((2, 4, 0)))
     def iter_reverse_revision_history(self, revision_id):
         self._ensure_real()
         return self._real_repository.iter_reverse_revision_history(revision_id)
@@ -3101,22 +3103,32 @@
         """
         try:
             configobj = self._get_configobj()
+            section_obj = None
             if section is None:
                 section_obj = configobj
             else:
                 try:
                     section_obj = configobj[section]
                 except KeyError:
-                    return default
-            return section_obj.get(name, default)
+                    pass
+            if section_obj is None:
+                value = default
+            else:
+                value = section_obj.get(name, default)
         except errors.UnknownSmartMethod:
-            return self._vfs_get_option(name, section, default)
+            value = self._vfs_get_option(name, section, default)
+        for hook in config.OldConfigHooks['get']:
+            hook(self, name, value)
+        return value
 
     def _response_to_configobj(self, response):
         if len(response[0]) and response[0][0] != 'ok':
             raise errors.UnexpectedSmartServerResponse(response)
         lines = response[1].read_body_bytes().splitlines()
-        return config.ConfigObj(lines, encoding='utf-8')
+        conf = config.ConfigObj(lines, encoding='utf-8')
+        for hook in config.OldConfigHooks['load']:
+            hook(self)
+        return conf
 
 
 class RemoteBranchConfig(RemoteConfig):

=== modified file 'bzrlib/repofmt/knitrepo.py'
--- bzrlib/repofmt/knitrepo.py	2011-05-18 10:41:51 +0000
+++ bzrlib/repofmt/knitrepo.py	2011-06-16 14:54:47 +0000
@@ -181,31 +181,6 @@
         result.get_parent_map([('A',)])
         return result
 
-    def fileid_involved_between_revs(self, from_revid, to_revid):
-        """Find file_id(s) which are involved in the changes between revisions.
-
-        This determines the set of revisions which are involved, and then
-        finds all file ids affected by those revisions.
-        """
-        vf = self._get_revision_vf()
-        from_set = set(vf.get_ancestry(from_revid))
-        to_set = set(vf.get_ancestry(to_revid))
-        changed = to_set.difference(from_set)
-        return self._fileid_involved_by_set(changed)
-
-    def fileid_involved(self, last_revid=None):
-        """Find all file_ids modified in the ancestry of last_revid.
-
-        :param last_revid: If None, last_revision() will be used.
-        """
-        if not last_revid:
-            changed = set(self.all_revision_ids())
-        else:
-            changed = set(self.get_ancestry(last_revid))
-        if None in changed:
-            changed.remove(None)
-        return self._fileid_involved_by_set(changed)
-
     @needs_read_lock
     def get_revision(self, revision_id):
         """Return the Revision object for a named revision"""

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2011-05-18 10:24:05 +0000
+++ bzrlib/repository.py	2011-06-18 13:57:17 +0000
@@ -522,21 +522,21 @@
         if revid and committers:
             result['committers'] = 0
         if revid and revid != _mod_revision.NULL_REVISION:
+            graph = self.get_graph()
             if committers:
                 all_committers = set()
-            revisions = self.get_ancestry(revid)
-            # pop the leading None
-            revisions.pop(0)
-            first_revision = None
+            revisions = [r for (r, p) in graph.iter_ancestry([revid])
+                        if r != _mod_revision.NULL_REVISION]
+            last_revision = None
             if not committers:
                 # ignore the revisions in the middle - just grab first and last
                 revisions = revisions[0], revisions[-1]
             for revision in self.get_revisions(revisions):
-                if not first_revision:
-                    first_revision = revision
+                if not last_revision:
+                    last_revision = revision
                 if committers:
                     all_committers.add(revision.committer)
-            last_revision = revision
+            first_revision = revision
             if committers:
                 result['committers'] = len(all_committers)
             result['firstrev'] = (first_revision.timestamp,
@@ -1002,6 +1002,7 @@
             raise AssertionError('_iter_for_revno returned too much history')
         return (True, partial_history[-1])
 
+    @symbol_versioning.deprecated_method(symbol_versioning.deprecated_in((2, 4, 0)))
     def iter_reverse_revision_history(self, revision_id):
         """Iterate backwards through revision ids in the lefthand history
 
@@ -1055,6 +1056,8 @@
         raise NotImplementedError(self.revision_trees)
 
     @needs_read_lock
+    @symbol_versioning.deprecated_method(
+        symbol_versioning.deprecated_in((2, 4, 0)))
     def get_ancestry(self, revision_id, topo_sorted=True):
         """Return a list of revision-ids integrated by a revision.
 
@@ -1064,6 +1067,8 @@
 
         This is topologically sorted.
         """
+        if 'evil' in debug.debug_flags:
+            mutter_callsite(2, "get_ancestry is linear with history.")
         if _mod_revision.is_null(revision_id):
             return [None]
         if not self.has_revision(revision_id):
@@ -1833,9 +1838,11 @@
         it is encountered, history extension will stop.
     """
     start_revision = partial_history_cache[-1]
-    iterator = repo.iter_reverse_revision_history(start_revision)
+    graph = repo.get_graph()
+    iterator = graph.iter_lefthand_ancestry(start_revision,
+        (_mod_revision.NULL_REVISION,))
     try:
-        #skip the last revision in the list
+        # skip the last revision in the list
         iterator.next()
         while True:
             if (stop_index is not None and

=== modified file 'bzrlib/revisiontree.py'
--- bzrlib/revisiontree.py	2011-05-13 14:02:14 +0000
+++ bzrlib/revisiontree.py	2011-06-16 11:58:04 +0000
@@ -179,10 +179,6 @@
     def _file_size(self, entry, stat_value):
         return entry.text_size
 
-    def _get_ancestors(self, default_revision):
-        return set(self._repository.get_ancestry(self._revision_id,
-                                                 topo_sorted=False))
-
     def walkdirs(self, prefix=""):
         _directory = 'directory'
         inv = self.inventory

=== modified file 'bzrlib/sign_my_commits.py'
--- bzrlib/sign_my_commits.py	2011-04-07 10:36:24 +0000
+++ bzrlib/sign_my_commits.py	2011-06-19 17:10:31 +0000
@@ -21,7 +21,9 @@
 lazy_import(globals(), """
 from bzrlib import (
     bzrdir as _mod_bzrdir,
+    errors,
     gpg,
+    revision as _mod_revision,
     )
 """)
 from bzrlib.commands import Command
@@ -64,9 +66,16 @@
         count = 0
         repo.lock_write()
         try:
+            graph = repo.get_graph()
             repo.start_write_group()
             try:
-                for rev_id in repo.get_ancestry(branch.last_revision())[1:]:
+                for rev_id, parents in graph.iter_ancestry(
+                        [branch.last_revision()]):
+                    if _mod_revision.is_null(rev_id):
+                        continue
+                    if parents is None:
+                        # Ignore ghosts
+                        continue
                     if repo.has_signature_for_revision_id(rev_id):
                         continue
                     rev = repo.get_revision(rev_id)

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2011-06-09 15:13:54 +0000
+++ bzrlib/tests/__init__.py	2011-06-27 15:42:09 +0000
@@ -142,7 +142,10 @@
     'BZREMAIL': None, # may still be present in the environment
     'EMAIL': 'jrandom@example.com', # set EMAIL as bzr does not guess
     'BZR_PROGRESS_BAR': None,
-    'BZR_LOG': None,
+    # This should trap leaks to ~/.bzr.log. This occurs when tests use TestCase
+    # as a base class instead of TestCaseInTempDir. Tests inheriting from
+    # TestCase should not use disk resources, BZR_LOG is one.
+    'BZR_LOG': '/you-should-use-TestCaseInTempDir-if-you-need-a-log-file',
     'BZR_PLUGIN_PATH': None,
     'BZR_DISABLE_PLUGINS': None,
     'BZR_PLUGINS_AT': None,
@@ -928,7 +931,7 @@
 
     The method is really a factory and users are expected to use it as such.
     """
-    
+
     kwargs['setUp'] = isolated_doctest_setUp
     kwargs['tearDown'] = isolated_doctest_tearDown
     return doctest.DocTestSuite(*args, **kwargs)
@@ -989,6 +992,12 @@
         # is addressed -- vila 20110219
         self.overrideAttr(config, '_expand_default_value', None)
         self._log_files = set()
+        # Each key in the ``_counters`` dict holds a value for a different
+        # counter. When the test ends, addDetail() should be used to output the
+        # counter values. This happens in install_counter_hook().
+        self._counters = {}
+        if 'config_stats' in selftest_debug_flags:
+            self._install_config_stats_hooks()
 
     def debug(self):
         # debug a frame up.
@@ -1011,6 +1020,50 @@
         if name in details:
             del details[name]
 
+    def install_counter_hook(self, hooks, name, counter_name=None):
+        """Install a counting hook.
+
+        Any hook can be counted as long as it doesn't need to return a value.
+
+        :param hooks: Where the hook should be installed.
+
+        :param name: The hook name that will be counted.
+
+        :param counter_name: The counter identifier in ``_counters``, defaults
+            to ``name``.
+        """
+        _counters = self._counters # Avoid closing over self
+        if counter_name is None:
+            counter_name = name
+        if _counters.has_key(counter_name):
+            raise AssertionError('%s is already used as a counter name'
+                                  % (counter_name,))
+        _counters[counter_name] = 0
+        self.addDetail(counter_name, content.Content(content.UTF8_TEXT,
+            lambda: ['%d' % (_counters[counter_name],)]))
+        def increment_counter(*args, **kwargs):
+            _counters[counter_name] += 1
+        label = 'count %s calls' % (counter_name,)
+        hooks.install_named_hook(name, increment_counter, label)
+        self.addCleanup(hooks.uninstall_named_hook, name, label)
+
+    def _install_config_stats_hooks(self):
+        """Install config hooks to count hook calls.
+
+        """
+        for hook_name in ('get', 'set', 'remove', 'load', 'save'):
+            self.install_counter_hook(config.ConfigHooks, hook_name,
+                                       'config.%s' % (hook_name,))
+
+        # The OldConfigHooks are private and need special handling to protect
+        # against recursive tests (tests that run other tests), so we just do
+        # manually what registering them into _builtin_known_hooks will provide
+        # us.
+        self.overrideAttr(config, 'OldConfigHooks', config._OldConfigHooks())
+        for hook_name in ('get', 'set', 'remove', 'load', 'save'):
+            self.install_counter_hook(config.OldConfigHooks, hook_name,
+                                      'old_config.%s' % (hook_name,))
+
     def _clear_debug_flags(self):
         """Prevent externally set debug flags affecting tests.
 
@@ -1070,9 +1123,13 @@
         # break some locks on purpose and should be taken into account by
         # considering that breaking a lock is just a dirty way of releasing it.
         if len(acquired_locks) != (len(released_locks) + len(broken_locks)):
-            message = ('Different number of acquired and '
-                       'released or broken locks. (%s, %s + %s)' %
-                       (acquired_locks, released_locks, broken_locks))
+            message = (
+                'Different number of acquired and '
+                'released or broken locks.\n'
+                'acquired=%s\n'
+                'released=%s\n'
+                'broken=%s\n' %
+                (acquired_locks, released_locks, broken_locks))
             if not self._lock_check_thorough:
                 # Rather than fail, just warn
                 print "Broken test %s: %s" % (self, message)
@@ -1650,7 +1707,7 @@
     def _finishLogFile(self):
         """Finished with the log file.
 
-        Close the file and delete it, unless setKeepLogfile was called.
+        Close the file and delete it.
         """
         if trace._trace_file:
             # flush the log file, to get all content
@@ -2271,20 +2328,21 @@
 class TestCaseWithMemoryTransport(TestCase):
     """Common test class for tests that do not need disk resources.
 
-    Tests that need disk resources should derive from TestCaseWithTransport.
+    Tests that need disk resources should derive from TestCaseInTempDir
+    orTestCaseWithTransport.
 
     TestCaseWithMemoryTransport sets the TEST_ROOT variable for all bzr tests.
 
-    For TestCaseWithMemoryTransport the test_home_dir is set to the name of
+    For TestCaseWithMemoryTransport the ``test_home_dir`` is set to the name of
     a directory which does not exist. This serves to help ensure test isolation
-    is preserved. test_dir is set to the TEST_ROOT, as is cwd, because they
-    must exist. However, TestCaseWithMemoryTransport does not offer local
-    file defaults for the transport in tests, nor does it obey the command line
+    is preserved. ``test_dir`` is set to the TEST_ROOT, as is cwd, because they
+    must exist. However, TestCaseWithMemoryTransport does not offer local file
+    defaults for the transport in tests, nor does it obey the command line
     override, so tests that accidentally write to the common directory should
     be rare.
 
-    :cvar TEST_ROOT: Directory containing all temporary directories, plus
-    a .bzr directory that stops us ascending higher into the filesystem.
+    :cvar TEST_ROOT: Directory containing all temporary directories, plus a
+        ``.bzr`` directory that stops us ascending higher into the filesystem.
     """
 
     TEST_ROOT = None
@@ -2614,6 +2672,12 @@
 
     OVERRIDE_PYTHON = 'python'
 
+    def setUp(self):
+        super(TestCaseInTempDir, self).setUp()
+        # Remove the protection set in isolated_environ, we have a proper
+        # access to disk resources now.
+        self.overrideEnv('BZR_LOG', None)
+
     def check_file_contents(self, filename, expect):
         self.log("check contents of file %s" % filename)
         f = file(filename)
@@ -3510,6 +3574,8 @@
 #                           with proper exclusion rules.
 #   -Ethreads               Will display thread ident at creation/join time to
 #                           help track thread leaks
+
+#   -Econfig_stats          Will collect statistics using addDetail
 selftest_debug_flags = set()
 
 

=== modified file 'bzrlib/tests/blackbox/test_ancestry.py'
--- bzrlib/tests/blackbox/test_ancestry.py	2009-08-18 16:53:50 +0000
+++ bzrlib/tests/blackbox/test_ancestry.py	2011-06-13 15:16:17 +0000
@@ -17,9 +17,6 @@
 import os
 
 from bzrlib.tests import TestCaseWithTransport
-from bzrlib.workingtree import WorkingTree
-from bzrlib.branch import Branch
-
 
 class TestAncestry(TestCaseWithTransport):
 

=== modified file 'bzrlib/tests/blackbox/test_annotate.py'
--- bzrlib/tests/blackbox/test_annotate.py	2011-02-09 08:37:08 +0000
+++ bzrlib/tests/blackbox/test_annotate.py	2011-06-13 15:18:27 +0000
@@ -24,9 +24,11 @@
 """
 
 
-from bzrlib import tests
+from bzrlib import (
+    config,
+    tests,
+    )
 
-from bzrlib.config import extract_email_address
 from bzrlib.urlutils import joinpath
 
 
@@ -160,7 +162,6 @@
         tree.add('file')
         tree.commit('add file', committer="test@host", rev_id="rev1")
         self.build_tree_contents([(file_relpath, 'foo\nbar\ngam\n')])
-        tree.branch.get_config().set_user_option('email', 'current@host2')
         return tree
 
     def test_annotate_cmd_revspec_branch(self):
@@ -176,6 +177,7 @@
 
     def test_annotate_edited_file(self):
         tree = self._setup_edited_file()
+        tree.branch.get_config().set_user_option('email', 'current@host2')
         out, err = self.run_bzr('annotate file')
         self.assertEqual(
             '1   test@ho | foo\n'
@@ -183,8 +185,24 @@
             '1   test@ho | gam\n',
             out)
 
+    def test_annotate_edited_file_no_default(self):
+        # Ensure that when no username is available annotate still works.
+        self.overrideEnv('EMAIL', None)
+        self.overrideEnv('BZR_EMAIL', None)
+        # Also, make sure that it's not inferred from mailname.
+        self.overrideAttr(config, '_auto_user_id',
+            lambda: (None, None))
+        tree = self._setup_edited_file()
+        out, err = self.run_bzr('annotate file')
+        self.assertEqual(
+            '1   test@ho | foo\n'
+            '2?  local u | bar\n'
+            '1   test@ho | gam\n',
+            out)
+
     def test_annotate_edited_file_show_ids(self):
         tree = self._setup_edited_file()
+        tree.branch.get_config().set_user_option('email', 'current@host2')
         out, err = self.run_bzr('annotate file --show-ids')
         self.assertEqual(
             '    rev1 | foo\n'
@@ -214,7 +232,7 @@
     def test_annotated_edited_merged_file_revnos(self):
         wt = self._create_merged_file()
         out, err = self.run_bzr(['annotate', 'file'])
-        email = extract_email_address(wt.branch.get_config().username())
+        email = config.extract_email_address(wt.branch.get_config().username())
         self.assertEqual(
             '3?    %-7s | local\n'
             '1     test@ho | foo\n'

=== modified file 'bzrlib/tests/blackbox/test_exceptions.py'
--- bzrlib/tests/blackbox/test_exceptions.py	2011-06-02 09:21:46 +0000
+++ bzrlib/tests/blackbox/test_exceptions.py	2011-06-17 13:53:59 +0000
@@ -30,10 +30,8 @@
     )
 from bzrlib.repofmt.groupcompress_repo import RepositoryFormat2a
 
-from bzrlib.tests import TestCase
-
-
-class TestExceptionReporting(TestCase):
+
+class TestExceptionReporting(tests.TestCaseInTempDir):
 
     def test_exception_exitcode(self):
         # we must use a subprocess, because the normal in-memory mechanism
@@ -63,7 +61,7 @@
         self.assertEquals(out, "")
 
 
-class TestOptParseBugHandling(TestCase):
+class TestOptParseBugHandling(tests.TestCase):
     "Test that we handle http://bugs.python.org/issue2931"
 
     def test_nonascii_optparse(self):

=== modified file 'bzrlib/tests/blackbox/test_export.py'
--- bzrlib/tests/blackbox/test_export.py	2011-06-07 13:33:41 +0000
+++ bzrlib/tests/blackbox/test_export.py	2011-06-10 14:47:25 +0000
@@ -387,6 +387,16 @@
         har_st = os.stat('t/har')
         self.assertEquals(315532800, har_st.st_mtime)
 
+    def test_dir_export_partial_tree_per_file_timestamps(self):
+        tree = self.example_branch()
+        self.build_tree(['branch/subdir/', 'branch/subdir/foo.txt'])
+        tree.smart_add(['branch'])
+        # Earliest allowable date on FAT32 filesystems is 1980-01-01
+        tree.commit('setup', timestamp=315532800)
+        self.run_bzr('export --per-file-timestamps tpart branch/subdir')
+        foo_st = os.stat('tpart/foo.txt')
+        self.assertEquals(315532800, foo_st.st_mtime)
+
     def test_export_directory(self):
         """Test --directory option"""
         self.example_branch()

=== modified file 'bzrlib/tests/blackbox/test_merge.py'
--- bzrlib/tests/blackbox/test_merge.py	2011-05-27 09:45:44 +0000
+++ bzrlib/tests/blackbox/test_merge.py	2011-06-13 14:32:13 +0000
@@ -34,7 +34,13 @@
     urlutils,
     workingtree,
     )
-from bzrlib.tests import script
+from bzrlib.tests import (
+    scenarios,
+    script,
+    )
+
+
+load_tests = scenarios.load_tests_apply_scenarios
 
 
 class TestMerge(tests.TestCaseWithTransport):
@@ -662,18 +668,6 @@
         this.lock_write()
         this.unlock()
 
-    def test_merge_reversed_revision_range(self):
-        tree = self.make_branch_and_tree(".")
-        for f in ("a", "b"):
-            self.build_tree([f])
-            tree.add(f)
-            tree.commit("added "+f)
-        for context in (".", "", "a"):
-            self.run_bzr("merge -r 1..0 " + context)
-            self.assertPathDoesNotExist("a")
-            tree.revert()
-            self.assertPathExists("a")
-
     def test_merge_fetches_tags(self):
         """Tags are updated by merge, and revisions named in those tags are
         fetched.
@@ -695,6 +689,27 @@
         self.assertEqual('rev-2a', target.tags.lookup_tag('tag-a'))
         target.repository.get_revision('rev-2a')
 
+
+class TestMergeRevisionRange(tests.TestCaseWithTransport):
+
+    scenarios = (('whole-tree', dict(context='.')),
+                 ('file-only', dict(context='a')))
+
+    def setUp(self):
+        super(TestMergeRevisionRange, self).setUp()
+        self.tree = self.make_branch_and_tree(".")
+        self.tree.commit('initial commit')
+        for f in ("a", "b"):
+            self.build_tree([f])
+            self.tree.add(f)
+            self.tree.commit("added " + f)
+
+    def test_merge_reversed_revision_range(self):
+        self.run_bzr("merge -r 2..1 " + self.context)
+        self.assertPathDoesNotExist("a")
+        self.assertPathExists("b")
+
+
 class TestMergeScript(script.TestCaseWithTransportAndScript):
     def test_merge_empty_branch(self):
         source = self.make_branch_and_tree('source')

=== modified file 'bzrlib/tests/blackbox/test_resolve.py'
--- bzrlib/tests/blackbox/test_resolve.py	2011-01-10 22:20:12 +0000
+++ bzrlib/tests/blackbox/test_resolve.py	2011-06-15 11:36:05 +0000
@@ -73,6 +73,44 @@
 """)
 
 
+class TestBug788000(script.TestCaseWithTransportAndScript):
+
+    def test_bug_788000(self):
+        self.run_script('''\
+$ bzr init a
+$ mkdir a/dir
+$ echo foo > a/dir/file
+$ bzr add a/dir
+$ cd a
+$ bzr commit -m one
+$ cd ..
+$ bzr clone a b
+$ echo bar > b/dir/file
+$ cd a
+$ rm -r dir
+$ bzr commit -m two
+$ cd ../b
+''',
+                        null_output_matches_anything=True)
+
+        self.run_script('''\
+$ bzr pull
+Using saved parent location:...
+Now on revision 2.
+2>RM  dir/file => dir/file.THIS
+2>Conflict: can't delete dir because it is not empty.  Not deleting.
+2>Conflict because dir is not versioned, but has versioned children...
+2>Contents conflict in dir/file
+2>3 conflicts encountered.
+''')
+        self.run_script('''\
+$ bzr resolve --take-other
+2>deleted dir/file.THIS
+2>deleted dir
+2>3 conflict(s) resolved, 0 remaining
+''')
+
+
 class TestResolveAuto(tests.TestCaseWithTransport):
 
     def test_auto_resolve(self):

=== modified file 'bzrlib/tests/blackbox/test_sign_my_commits.py'
--- bzrlib/tests/blackbox/test_sign_my_commits.py	2010-01-25 17:48:22 +0000
+++ bzrlib/tests/blackbox/test_sign_my_commits.py	2011-06-19 16:31:57 +0000
@@ -17,14 +17,10 @@
 
 """Black-box tests for bzr sign-my-commits."""
 
-import os
-
 from bzrlib import (
     gpg,
     tests,
     )
-from bzrlib.testament import Testament
-from bzrlib.workingtree import WorkingTree
 
 
 class SignMyCommits(tests.TestCaseWithTransport):
@@ -45,7 +41,8 @@
         wt.commit("base C", allow_pointless=True, rev_id='C')
         wt.commit("base D", allow_pointless=True, rev_id='D',
                 committer='Alternate <alt@foo.com>')
-
+        wt.add_parent_tree_id("aghost")
+        wt.commit("base E", allow_pointless=True, rev_id='E')
         return wt
 
     def assertUnsigned(self, repo, revision_id):
@@ -109,8 +106,11 @@
 
         out = self.run_bzr('sign-my-commits --dry-run')[0]
 
-        self.assertEquals('A\nB\nC\nSigned 3 revisions\n', out)
+        outlines = out.splitlines()
+        self.assertEquals(5, len(outlines))
+        self.assertEquals('Signed 4 revisions', outlines[-1])
         self.assertUnsigned(repo, 'A')
         self.assertUnsigned(repo, 'B')
         self.assertUnsigned(repo, 'C')
         self.assertUnsigned(repo, 'D')
+        self.assertUnsigned(repo, 'E')

=== modified file 'bzrlib/tests/blackbox/test_whoami.py'
--- bzrlib/tests/blackbox/test_whoami.py	2011-04-06 11:07:17 +0000
+++ bzrlib/tests/blackbox/test_whoami.py	2011-06-22 09:09:35 +0000
@@ -20,18 +20,26 @@
 import bzrlib
 from bzrlib import (
     config,
+    tests,
     )
-from bzrlib.tests import TestCaseWithTransport
-
-
-class TestWhoami(TestCaseWithTransport):
-
-    def test_whoami(self):
+
+
+class TestWhoami(tests.TestCaseWithTransport):
+
+    def assertWhoAmI(self, expected, *cmd_args, **kwargs):
+        out, err = self.run_bzr(('whoami',) + cmd_args, **kwargs)
+        self.assertEquals('', err)
+        lines = out.splitlines()
+        self.assertLength(1, lines)
+        self.assertEquals(expected, lines[0].rstrip())
+
+    def test_whoami_no_args_no_conf(self):
         # this should always identify something, if only "john@localhost"
         out = self.run_bzr("whoami")[0]
         self.assertTrue(len(out) > 0)
         self.assertEquals(1, out.count('@'))
 
+    def test_whoami_email_no_args(self):
         out = self.run_bzr("whoami --email")[0]
         self.assertTrue(len(out) > 0)
         self.assertEquals(1, out.count('@'))
@@ -47,29 +55,22 @@
         b = bzrlib.branch.Branch.open('.')
         b.get_config().set_user_option('email',
                                        'Branch Identity <branch@identi.ty>')
-        whoami = self.run_bzr("whoami")[0]
-        self.assertEquals('Branch Identity <branch@identi.ty>\n', whoami)
-        whoami_email = self.run_bzr("whoami --email")[0]
-        self.assertEquals('branch@identi.ty\n', whoami_email)
+        self.assertWhoAmI('Branch Identity <branch@identi.ty>')
+        self.assertWhoAmI('branch@identi.ty', '--email')
 
         # Verify that the environment variable overrides the value
         # in the file
         self.overrideEnv('BZR_EMAIL', 'Different ID <other@environ.ment>')
-        whoami = self.run_bzr("whoami")[0]
-        self.assertEquals('Different ID <other@environ.ment>\n', whoami)
-        whoami_email = self.run_bzr("whoami --email")[0]
-        self.assertEquals('other@environ.ment\n', whoami_email)
+        self.assertWhoAmI('Different ID <other@environ.ment>')
+        self.assertWhoAmI('other@environ.ment', '--email')
 
     def test_whoami_utf8(self):
         """verify that an identity can be in utf-8."""
-        wt = self.make_branch_and_tree('.')
         self.run_bzr(['whoami', u'Branch Identity \u20ac <branch@identi.ty>'],
                      encoding='utf-8')
-        whoami = self.run_bzr("whoami", encoding='utf-8')[0]
-        self.assertEquals('Branch Identity \xe2\x82\xac ' +
-                          '<branch@identi.ty>\n', whoami)
-        whoami_email = self.run_bzr("whoami --email", encoding='utf-8')[0]
-        self.assertEquals('branch@identi.ty\n', whoami_email)
+        self.assertWhoAmI('Branch Identity \xe2\x82\xac <branch@identi.ty>',
+                          encoding='utf-8')
+        self.assertWhoAmI('branch@identi.ty', '--email')
 
     def test_whoami_ascii(self):
         """
@@ -80,10 +81,10 @@
         b = bzrlib.branch.Branch.open('.')
         b.get_config().set_user_option('email', u'Branch Identity \u20ac ' +
                                        '<branch@identi.ty>')
-        whoami = self.run_bzr("whoami", encoding='ascii')[0]
-        self.assertEquals('Branch Identity ? <branch@identi.ty>\n', whoami)
-        whoami_email = self.run_bzr("whoami --email", encoding='ascii')[0]
-        self.assertEquals('branch@identi.ty\n', whoami_email)
+        self.assertWhoAmI('Branch Identity ? <branch@identi.ty>',
+                          encoding='ascii')
+        self.assertWhoAmI('branch@identi.ty', '--email',
+                          encoding='ascii')
 
     def test_warning(self):
         """verify that a warning is displayed if no email is given."""
@@ -109,10 +110,11 @@
         wt = self.make_branch_and_tree('subdir')
         c = wt.branch.get_config()
         c.set_user_option('email', 'Branch Identity <branch@identi.ty>')
-        out, err = self.run_bzr("whoami --directory subdir")
-        self.assertEquals('Branch Identity <branch@identi.ty>\n', out)
+        self.assertWhoAmI('Branch Identity <branch@identi.ty>',
+                          '--directory', 'subdir')
         self.run_bzr(['whoami', '--directory', 'subdir', '--branch',
                       'Changed Identity <changed@identi.ty>'])
+        c = wt.branch.get_config()
         self.assertEquals('Changed Identity <changed@identi.ty>',
                           c.get_user_option('email'))
 
@@ -122,13 +124,14 @@
         c = wt.branch.get_config()
         c.set_user_option('email', 'Branch Identity <branch@identi.ty>')
         url = self.get_readonly_url() + '/subdir'
-        out, err = self.run_bzr(['whoami', '--directory', url])
-        self.assertEquals('Branch Identity <branch@identi.ty>\n', out)
+        self.assertWhoAmI('Branch Identity <branch@identi.ty>',
+                          '--directory', url)
         url = self.get_url('subdir')
         self.run_bzr(['whoami', '--directory', url, '--branch',
                       'Changed Identity <changed@identi.ty>'])
         # The identity has been set in the branch config (but not the global
         # config)
+        c = wt.branch.get_config()
         self.assertEquals('Changed Identity <changed@identi.ty>',
                           c.get_user_option('email'))
         global_conf = config.GlobalConfig()

=== modified file 'bzrlib/tests/features.py'
--- bzrlib/tests/features.py	2011-03-14 12:43:19 +0000
+++ bzrlib/tests/features.py	2011-06-16 20:47:33 +0000
@@ -18,6 +18,7 @@
 
 import os
 import stat
+import sys
 
 from bzrlib import (
     osutils,
@@ -49,6 +50,7 @@
 pywintypes = tests.ModuleAvailableFeature('pywintypes')
 sphinx = tests.ModuleAvailableFeature('sphinx')
 subunit = tests.ModuleAvailableFeature('subunit')
+testtools = tests.ModuleAvailableFeature('testtools')
 
 
 class _BackslashDirSeparatorFeature(tests.Feature):
@@ -127,3 +129,17 @@
 bash_feature = ExecutableFeature('bash')
 sed_feature = ExecutableFeature('sed')
 diff_feature = ExecutableFeature('diff')
+
+
+class Win32Feature(tests.Feature):
+    """Feature testing whether we're running selftest on Windows
+    or Windows-like platform.
+    """
+
+    def _probe(self):
+        return sys.platform == 'win32'
+
+    def feature_name(self):
+        return "win32 platform"
+
+win32_feature = Win32Feature()

=== modified file 'bzrlib/tests/matchers.py'
--- bzrlib/tests/matchers.py	2010-05-06 11:08:10 +0000
+++ bzrlib/tests/matchers.py	2011-06-19 12:45:11 +0000
@@ -27,9 +27,14 @@
 """
 
 __all__ = [
+    'MatchesAncestry',
     'ReturnsUnlockable',
     ]
 
+from bzrlib import (
+    revision as _mod_revision,
+    )
+
 from testtools.matchers import Mismatch, Matcher
 
 
@@ -66,3 +71,45 @@
 
     def describe(self):
         return "%s is locked" % self.lockable_thing
+
+
+class _AncestryMismatch(Mismatch):
+    """Ancestry matching mismatch."""
+
+    def __init__(self, tip_revision, got, expected):
+        self.tip_revision = tip_revision
+        self.got = got
+        self.expected = expected
+
+    def describe(self):
+        return "mismatched ancestry for revision %r was %r, expected %r" % (
+            self.tip_revision, self.got, self.expected)
+
+
+class MatchesAncestry(Matcher):
+    """A matcher that checks the ancestry of a particular revision.
+
+    :ivar graph: Graph in which to check the ancestry
+    :ivar revision_id: Revision id of the revision
+    """
+
+    def __init__(self, repository, revision_id):
+        Matcher.__init__(self)
+        self.repository = repository
+        self.revision_id = revision_id
+
+    def __str__(self):
+        return ('MatchesAncestry(repository=%r, revision_id=%r)' % (
+            self.repository, self.revision_id))
+
+    def match(self, expected):
+        self.repository.lock_read()
+        try:
+            graph = self.repository.get_graph()
+            got = [r for r, p in graph.iter_ancestry([self.revision_id])]
+            if _mod_revision.NULL_REVISION in got:
+                got.remove(_mod_revision.NULL_REVISION)
+        finally:
+            self.repository.unlock()
+        if sorted(got) != sorted(expected):
+            return _AncestryMismatch(self.revision_id, sorted(got), sorted(expected))

=== modified file 'bzrlib/tests/per_interrepository/test_interrepository.py'
--- bzrlib/tests/per_interrepository/test_interrepository.py	2011-05-18 11:43:37 +0000
+++ bzrlib/tests/per_interrepository/test_interrepository.py	2011-06-16 15:29:10 +0000
@@ -27,6 +27,7 @@
     TestNotApplicable,
     TestSkipped,
     )
+from bzrlib.tests.matchers import MatchesAncestry
 from bzrlib.tests.per_interrepository import (
     TestCaseWithInterRepository,
     )
@@ -200,5 +201,5 @@
         rev = missing_ghost.get_revision('ghost')
         inv = missing_ghost.get_inventory('ghost')
         # rev must not be corrupt now
-        self.assertEqual([None, 'ghost', 'references', 'tip'],
-            missing_ghost.get_ancestry('tip'))
+        self.assertThat(['ghost', 'references', 'tip'],
+            MatchesAncestry(missing_ghost, 'tip'))

=== modified file 'bzrlib/tests/per_repository/test_commit_builder.py'
--- bzrlib/tests/per_repository/test_commit_builder.py	2011-05-17 15:19:59 +0000
+++ bzrlib/tests/per_repository/test_commit_builder.py	2011-06-13 15:16:17 +0000
@@ -21,7 +21,6 @@
 from bzrlib import (
     config,
     errors,
-    graph,
     inventory,
     osutils,
     repository,

=== modified file 'bzrlib/tests/per_repository/test_fileid_involved.py'
--- bzrlib/tests/per_repository/test_fileid_involved.py	2011-04-19 10:42:59 +0000
+++ bzrlib/tests/per_repository/test_fileid_involved.py	2011-06-16 13:29:36 +0000
@@ -214,16 +214,15 @@
 
         if len(history) < 2: return
 
+        graph = self.branch.repository.get_graph()
         for start in range(0,len(history)-1):
             start_id = history[start]
             for end in range(start+1,len(history)):
                 end_id = history[end]
-                old_revs = set(self.branch.repository.get_ancestry(start_id))
-                new_revs = set(self.branch.repository.get_ancestry(end_id))
+                unique_revs = graph.find_unique_ancestors(end_id, [start_id])
                 l1 = self.branch.repository.fileids_altered_by_revision_ids(
-                    new_revs.difference(old_revs))
+                    unique_revs)
                 l1 = set(l1.keys())
-
                 l2 = self.compare_tree_fileids(self.branch, start_id, end_id)
                 self.assertEquals(l1, l2)
 
@@ -305,11 +304,11 @@
         history = self.branch.revision_history()
         old_rev = history[0]
         new_rev = history[1]
-        old_revs = set(self.branch.repository.get_ancestry(old_rev))
-        new_revs = set(self.branch.repository.get_ancestry(new_rev))
+        graph = self.branch.repository.get_graph()
+        unique_revs = graph.find_unique_ancestors(new_rev, [old_rev])
 
         l1 = self.branch.repository.fileids_altered_by_revision_ids(
-            new_revs.difference(old_revs))
+            unique_revs)
         l1 = set(l1.keys())
 
         l2 = self.compare_tree_fileids(self.branch, old_rev, new_rev)

=== modified file 'bzrlib/tests/per_repository/test_iter_reverse_revision_history.py'
--- bzrlib/tests/per_repository/test_iter_reverse_revision_history.py	2011-04-09 21:00:33 +0000
+++ bzrlib/tests/per_repository/test_iter_reverse_revision_history.py	2011-06-16 12:50:32 +0000
@@ -20,6 +20,7 @@
     errors,
     tests,
     )
+from bzrlib.symbol_versioning import deprecated_in
 from bzrlib.tests.per_repository import TestCaseWithRepository
 
 
@@ -91,7 +92,8 @@
         repo = tree.branch.repository
         repo.lock_read()
         self.addCleanup(repo.unlock)
-        rev_history = repo.iter_reverse_revision_history('rev4')
+        rev_history = self.applyDeprecated(deprecated_in((2, 4, 0)),
+            repo.iter_reverse_revision_history, 'rev4')
         self.assertEqual('rev4', rev_history.next())
         self.assertEqual('rev3', rev_history.next())
         self.assertEqual('rev2', rev_history.next())
@@ -102,7 +104,8 @@
         """Assert the return values of iter_reverse_revision_history."""
         repo.lock_read()
         try:
-            actual = list(repo.iter_reverse_revision_history(revision_id))
+            actual = list(self.applyDeprecated(deprecated_in((2, 4, 0)),
+                repo.iter_reverse_revision_history, revision_id))
         finally:
             repo.unlock()
         self.assertEqual(expected, actual)
@@ -155,7 +158,8 @@
             tree.commit('2', rev_id='rev2')
         finally:
             tree.unlock()
-        iter = tree.branch.repository.iter_reverse_revision_history('rev2')
+        iter = self.applyDeprecated(deprecated_in((2, 4, 0)),
+            tree.branch.repository.iter_reverse_revision_history, 'rev2')
         tree.branch.repository.lock_read()
         try:
             self.assertEquals('rev2', iter.next())

=== modified file 'bzrlib/tests/per_repository_vf/test_reconcile.py'
--- bzrlib/tests/per_repository_vf/test_reconcile.py	2011-04-19 12:41:46 +0000
+++ bzrlib/tests/per_repository_vf/test_reconcile.py	2011-06-19 15:23:02 +0000
@@ -30,6 +30,7 @@
     TestCaseWithRepository,
     all_repository_vf_format_scenarios,
     )
+from bzrlib.tests.matchers import MatchesAncestry
 from bzrlib.tests.scenarios import load_tests_apply_scenarios
 from bzrlib.uncommit import uncommit
 
@@ -194,8 +195,7 @@
             **kwargs):
         # actual low level test.
         repo = aBzrDir.open_repository()
-        if ([None, 'missing', 'references_missing']
-            != repo.get_ancestry('references_missing')):
+        if not repo.has_revision('missing'):
             # the repo handles ghosts without corruption, so reconcile has
             # nothing to do here. Specifically, this test has the inventory
             # 'missing' present and the revision 'missing' missing, so clearly
@@ -215,8 +215,7 @@
         self.check_missing_was_removed(repo)
         # and the parent list for 'references_missing' should have that
         # revision a ghost now.
-        self.assertEqual([None, 'references_missing'],
-                         repo.get_ancestry('references_missing'))
+        self.assertFalse(repo.has_revision('missing'))
 
     def check_missing_was_removed(self, repo):
         if repo._reconcile_backsup_inventory:
@@ -265,17 +264,17 @@
         # now the current inventory should still have 'ghost'
         repo = d.open_repository()
         repo.get_inventory('ghost')
-        self.assertEqual([None, 'ghost'], repo.get_ancestry('ghost'))
+        self.assertThat(['ghost', 'the_ghost'], MatchesAncestry(repo, 'ghost'))
 
     def test_reweave_inventory_fixes_ancestryfor_a_present_ghost(self):
         d = bzrlib.bzrdir.BzrDir.open(self.get_url('inventory_ghost_present'))
         repo = d.open_repository()
-        ghost_ancestry = repo.get_ancestry('ghost')
-        if ghost_ancestry == [None, 'the_ghost', 'ghost']:
+        m = MatchesAncestry(repo, 'ghost')
+        if m.match(['the_ghost', 'ghost']) is None:
             # the repo handles ghosts without corruption, so reconcile has
             # nothing to do
             return
-        self.assertEqual([None, 'ghost'], ghost_ancestry)
+        self.assertThat(['ghost'], m)
         reconciler = repo.reconcile()
         # this is a data corrupting error, so a normal reconcile should fix it.
         # one inconsistent parents should have been found : the
@@ -287,8 +286,10 @@
         repo = d.open_repository()
         repo.get_inventory('ghost')
         repo.get_inventory('the_ghost')
-        self.assertEqual([None, 'the_ghost', 'ghost'], repo.get_ancestry('ghost'))
-        self.assertEqual([None, 'the_ghost'], repo.get_ancestry('the_ghost'))
+        self.assertThat(['the_ghost', 'ghost'],
+            MatchesAncestry(repo, 'ghost'))
+        self.assertThat(['the_ghost'],
+            MatchesAncestry(repo, 'the_ghost'))
 
     def test_text_from_ghost_revision(self):
         repo = self.make_repository('text-from-ghost')

=== modified file 'bzrlib/tests/per_repository_vf/test_repository.py'
--- bzrlib/tests/per_repository_vf/test_repository.py	2011-05-13 09:35:35 +0000
+++ bzrlib/tests/per_repository_vf/test_repository.py	2011-06-19 15:23:02 +0000
@@ -27,6 +27,8 @@
     vf_repository,
     )
 
+from bzrlib.symbol_versioning import deprecated_in
+from bzrlib.tests.matchers import MatchesAncestry
 from bzrlib.tests.per_repository_vf import (
     TestCaseWithRepository,
     all_repository_vf_format_scenarios,
@@ -384,13 +386,18 @@
     def test_get_ancestry_missing_revision(self):
         # get_ancestry(revision that is in some data but not fully installed
         # -> NoSuchRevision
+        repo = self.bzrdir.open_repository()
         self.assertRaises(errors.NoSuchRevision,
-                          self.bzrdir.open_repository().get_ancestry, 'orphan')
+            self.applyDeprecated, deprecated_in((2, 4, 0)),
+            repo.get_ancestry, 'orphan')
 
     def test_get_unordered_ancestry(self):
         repo = self.bzrdir.open_repository()
-        self.assertEqual(set(repo.get_ancestry('rev3')),
-                         set(repo.get_ancestry('rev3', topo_sorted=False)))
+        self.assertEqual(
+            set(self.applyDeprecated(deprecated_in((2, 4, 0)),
+                repo.get_ancestry, 'rev3')),
+            set(self.applyDeprecated(deprecated_in((2, 4, 0)),
+                repo.get_ancestry, 'rev3', topo_sorted=False)))
 
     def test_reserved_id(self):
         repo = self.make_repository('repository')
@@ -457,9 +464,10 @@
     def test_corrupt_revision_access_asserts_if_reported_wrong(self):
         repo_url = self.get_url('inventory_with_unnecessary_ghost')
         repo = _mod_repository.Repository.open(repo_url)
+        m = MatchesAncestry(repo, 'ghost')
         reported_wrong = False
         try:
-            if repo.get_ancestry('ghost') != [None, 'the_ghost', 'ghost']:
+            if m.match(['the_ghost', 'ghost']) is not None:
                 reported_wrong = True
         except errors.CorruptRepository:
             # caught the bad data:

=== modified file 'bzrlib/tests/per_workingtree/test_merge_from_branch.py'
--- bzrlib/tests/per_workingtree/test_merge_from_branch.py	2011-04-13 14:33:56 +0000
+++ bzrlib/tests/per_workingtree/test_merge_from_branch.py	2011-06-13 13:41:49 +0000
@@ -162,6 +162,8 @@
         outer.commit('added foo')
         inner = self.make_inner_branch()
         outer.merge_from_branch(inner, to_revision='1', from_revision='null:')
+        #retain original root id.
+        outer.set_root_id(outer.basis_tree().get_root_id())
         outer.commit('merge inner branch')
         outer.mkdir('dir-outer', 'dir-outer-id')
         outer.move(['dir', 'file3'], to_dir='dir-outer')

=== modified file 'bzrlib/tests/per_workingtree/test_rename_one.py'
--- bzrlib/tests/per_workingtree/test_rename_one.py	2011-05-13 12:51:05 +0000
+++ bzrlib/tests/per_workingtree/test_rename_one.py	2011-05-24 21:09:26 +0000
@@ -220,6 +220,51 @@
         self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id')],
                               tree.basis_tree())
 
+    def test_rename_one_after_dest_versioned(self):
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['a'])
+        tree.add(['a'], ['a-id'])
+        tree.commit('initial', rev_id='rev-1')
+        root_id = tree.get_root_id()
+        os.rename('a', 'b')
+        tree.add(['b'], ['b-id'])
+
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id')],
+                               tree)
+        e = self.assertRaises(errors.BzrMoveFailedError,
+            tree.rename_one, 'a', 'b')
+        self.assertIsInstance(e.extra, errors.AlreadyVersionedError)
+
+    def test_rename_one_after_with_after_dest_versioned(self):
+        ''' using after with an already versioned file should fail '''
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['a', 'b'])
+        tree.add(['a', 'b'], ['a-id', 'b-id'])
+        tree.commit('initial', rev_id='rev-1')
+        root_id = tree.get_root_id()
+        os.unlink('a')
+
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id')],
+                               tree)
+        e = self.assertRaises(errors.BzrMoveFailedError,
+            tree.rename_one, 'a', 'b', after=True)
+        self.assertIsInstance(e.extra, errors.AlreadyVersionedError)
+
+    def test_rename_one_after_with_after_dest_added(self):
+        ''' using after with a newly added file should work '''
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['a'])
+        tree.add(['a'], ['a-id'])
+        tree.commit('initial', rev_id='rev-1')
+        root_id = tree.get_root_id()
+        os.rename('a', 'b')
+        tree.add(['b'], ['b-id'])
+
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id')],
+                               tree)
+        tree.rename_one('a', 'b', after=True)
+        self.assertTreeLayout([('', root_id), ('b', 'a-id')], tree)
+
     def test_rename_one_after_source_removed(self):
         """Rename even if the source was already unversioned."""
         tree = self.make_branch_and_tree('.')

=== modified file 'bzrlib/tests/test_ancestry.py'
--- bzrlib/tests/test_ancestry.py	2011-01-12 01:01:53 +0000
+++ bzrlib/tests/test_ancestry.py	2011-06-16 15:29:10 +0000
@@ -15,25 +15,22 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 
+from bzrlib.branchbuilder import BranchBuilder
 from bzrlib.tests import TestCaseWithMemoryTransport
-from bzrlib.branchbuilder import BranchBuilder
+from bzrlib.tests.matchers import MatchesAncestry
 
 
 class TestAncestry(TestCaseWithMemoryTransport):
 
-    def assertAncestryEqual(self, expected, revision_id, branch):
-        """Assert that the ancestry of revision_id in branch is as expected."""
-        ancestry = branch.repository.get_ancestry(revision_id)
-        self.assertEqual(expected, ancestry)
-
     def test_straightline_ancestry(self):
         """Test ancestry file when just committing."""
         builder = BranchBuilder(self.get_transport())
         rev_id_one = builder.build_commit()
         rev_id_two = builder.build_commit()
         branch = builder.get_branch()
-        self.assertAncestryEqual([None, rev_id_one, rev_id_two],
-            rev_id_two, branch)
-        self.assertAncestryEqual([None, rev_id_one], rev_id_one, branch)
+        self.assertThat([rev_id_one, rev_id_two],
+            MatchesAncestry(branch.repository, rev_id_two))
+        self.assertThat([rev_id_one],
+            MatchesAncestry(branch.repository, rev_id_one))
 
 # TODO: check that ancestry is updated to include indirectly merged revisions

=== modified file 'bzrlib/tests/test_chk_map.py'
--- bzrlib/tests/test_chk_map.py	2011-05-26 08:05:45 +0000
+++ bzrlib/tests/test_chk_map.py	2011-06-20 07:23:28 +0000
@@ -224,7 +224,7 @@
             "      ('ddd',) 'initial ddd content'\n",
             c_map._dump_tree())
 
-    def test_one_deep_map_16(self):
+    def test_root_only_aaa_ddd_16(self):
         c_map = self.make_root_only_aaa_ddd_map(
                 search_key_func=chk_map._search_key_16)
         # We use 'aaa' and 'ddd' because they happen to map to 'F' when using

=== modified file 'bzrlib/tests/test_commit.py'
--- bzrlib/tests/test_commit.py	2011-05-13 12:51:05 +0000
+++ bzrlib/tests/test_commit.py	2011-06-16 15:29:10 +0000
@@ -26,13 +26,18 @@
 from bzrlib.bzrdir import BzrDirMetaFormat1
 from bzrlib.commit import Commit, NullCommitReporter
 from bzrlib.config import BranchConfig
-from bzrlib.errors import (PointlessCommit, BzrError, SigningFailed,
-                           LockContention)
+from bzrlib.errors import (
+    PointlessCommit,
+    BzrError,
+    SigningFailed,
+    LockContention,
+    )
 from bzrlib.tests import (
     SymlinkFeature,
     TestCaseWithTransport,
     test_foreign,
     )
+from bzrlib.tests.matchers import MatchesAncestry
 
 
 # TODO: Test commit with some added, and added-but-missing files
@@ -357,8 +362,8 @@
         eq = self.assertEquals
         eq(b.revision_history(), rev_ids)
         for i in range(4):
-            anc = b.repository.get_ancestry(rev_ids[i])
-            eq(anc, [None] + rev_ids[:i+1])
+            self.assertThat(rev_ids[:i+1],
+                MatchesAncestry(b.repository, rev_ids[i]))
 
     def test_commit_new_subdir_child_selective(self):
         wt = self.make_branch_and_tree('.')

=== modified file 'bzrlib/tests/test_config.py'
--- bzrlib/tests/test_config.py	2011-06-01 12:31:04 +0000
+++ bzrlib/tests/test_config.py	2011-06-20 13:58:54 +0000
@@ -37,6 +37,7 @@
     ui,
     urlutils,
     registry,
+    remote,
     tests,
     trace,
     transport,
@@ -45,11 +46,11 @@
     deprecated_in,
     deprecated_method,
     )
-from bzrlib.transport import remote
+from bzrlib.transport import remote as transport_remote
 from bzrlib.tests import (
     features,
-    TestSkipped,
     scenarios,
+    test_server,
     )
 from bzrlib.util.configobj import configobj
 
@@ -118,7 +119,8 @@
 def build_remote_branch_store(test):
     # There is only one permutation (but we won't be able to handle more with
     # this design anyway)
-    (transport_class, server_class) = remote.get_test_permutations()[0]
+    (transport_class,
+     server_class) = transport_remote.get_test_permutations()[0]
     build_backing_branch(test, 'branch', transport_class, server_class)
     b = branch.Branch.open(test.get_url('branch'))
     return config.BranchStore(b)
@@ -142,7 +144,8 @@
 def build_remote_branch_stack(test):
     # There is only one permutation (but we won't be able to handle more with
     # this design anyway)
-    (transport_class, server_class) = remote.get_test_permutations()[0]
+    (transport_class,
+     server_class) = transport_remote.get_test_permutations()[0]
     build_backing_branch(test, 'branch', transport_class, server_class)
     b = branch.Branch.open(test.get_url('branch'))
     return config.BranchStack(b)
@@ -1868,9 +1871,34 @@
 
 class TestTransportConfig(tests.TestCaseWithTransport):
 
+    def test_load_utf8(self):
+        """Ensure we can load an utf8-encoded file."""
+        t = self.get_transport()
+        unicode_user = u'b\N{Euro Sign}ar'
+        unicode_content = u'user=%s' % (unicode_user,)
+        utf8_content = unicode_content.encode('utf8')
+        # Store the raw content in the config file
+        t.put_bytes('foo.conf', utf8_content)
+        conf = config.TransportConfig(t, 'foo.conf')
+        self.assertEquals(unicode_user, conf.get_option('user'))
+
+    def test_load_non_ascii(self):
+        """Ensure we display a proper error on non-ascii, non utf-8 content."""
+        t = self.get_transport()
+        t.put_bytes('foo.conf', 'user=foo\n#\xff\n')
+        conf = config.TransportConfig(t, 'foo.conf')
+        self.assertRaises(errors.ConfigContentError, conf._get_configobj)
+
+    def test_load_erroneous_content(self):
+        """Ensure we display a proper error on content that can't be parsed."""
+        t = self.get_transport()
+        t.put_bytes('foo.conf', '[open_section\n')
+        conf = config.TransportConfig(t, 'foo.conf')
+        self.assertRaises(errors.ParseConfigError, conf._get_configobj)
+
     def test_get_value(self):
         """Test that retreiving a value from a section is possible"""
-        bzrdir_config = config.TransportConfig(transport.get_transport('.'),
+        bzrdir_config = config.TransportConfig(self.get_transport('.'),
                                                'control.conf')
         bzrdir_config.set_option('value', 'key', 'SECTION')
         bzrdir_config.set_option('value2', 'key2')
@@ -1906,6 +1934,247 @@
         self.assertIs(None, bzrdir_config.get_default_stack_on())
 
 
+class TestOldConfigHooks(tests.TestCaseWithTransport):
+
+    def setUp(self):
+        super(TestOldConfigHooks, self).setUp()
+        create_configs_with_file_option(self)
+
+    def assertGetHook(self, conf, name, value):
+        calls = []
+        def hook(*args):
+            calls.append(args)
+        config.OldConfigHooks.install_named_hook('get', hook, None)
+        self.addCleanup(
+            config.OldConfigHooks.uninstall_named_hook, 'get', None)
+        self.assertLength(0, calls)
+        actual_value = conf.get_user_option(name)
+        self.assertEquals(value, actual_value)
+        self.assertLength(1, calls)
+        self.assertEquals((conf, name, value), calls[0])
+
+    def test_get_hook_bazaar(self):
+        self.assertGetHook(self.bazaar_config, 'file', 'bazaar')
+
+    def test_get_hook_locations(self):
+        self.assertGetHook(self.locations_config, 'file', 'locations')
+
+    def test_get_hook_branch(self):
+        # Since locations masks branch, we define a different option
+        self.branch_config.set_user_option('file2', 'branch')
+        self.assertGetHook(self.branch_config, 'file2', 'branch')
+
+    def assertSetHook(self, conf, name, value):
+        calls = []
+        def hook(*args):
+            calls.append(args)
+        config.OldConfigHooks.install_named_hook('set', hook, None)
+        self.addCleanup(
+            config.OldConfigHooks.uninstall_named_hook, 'set', None)
+        self.assertLength(0, calls)
+        conf.set_user_option(name, value)
+        self.assertLength(1, calls)
+        # We can't assert the conf object below as different configs use
+        # different means to implement set_user_option and we care only about
+        # coverage here.
+        self.assertEquals((name, value), calls[0][1:])
+
+    def test_set_hook_bazaar(self):
+        self.assertSetHook(self.bazaar_config, 'foo', 'bazaar')
+
+    def test_set_hook_locations(self):
+        self.assertSetHook(self.locations_config, 'foo', 'locations')
+
+    def test_set_hook_branch(self):
+        self.assertSetHook(self.branch_config, 'foo', 'branch')
+
+    def assertRemoveHook(self, conf, name, section_name=None):
+        calls = []
+        def hook(*args):
+            calls.append(args)
+        config.OldConfigHooks.install_named_hook('remove', hook, None)
+        self.addCleanup(
+            config.OldConfigHooks.uninstall_named_hook, 'remove', None)
+        self.assertLength(0, calls)
+        conf.remove_user_option(name, section_name)
+        self.assertLength(1, calls)
+        # We can't assert the conf object below as different configs use
+        # different means to implement remove_user_option and we care only about
+        # coverage here.
+        self.assertEquals((name,), calls[0][1:])
+
+    def test_remove_hook_bazaar(self):
+        self.assertRemoveHook(self.bazaar_config, 'file')
+
+    def test_remove_hook_locations(self):
+        self.assertRemoveHook(self.locations_config, 'file',
+                              self.locations_config.location)
+
+    def test_remove_hook_branch(self):
+        self.assertRemoveHook(self.branch_config, 'file')
+
+    def assertLoadHook(self, name, conf_class, *conf_args):
+        calls = []
+        def hook(*args):
+            calls.append(args)
+        config.OldConfigHooks.install_named_hook('load', hook, None)
+        self.addCleanup(
+            config.OldConfigHooks.uninstall_named_hook, 'load', None)
+        self.assertLength(0, calls)
+        # Build a config
+        conf = conf_class(*conf_args)
+        # Access an option to trigger a load
+        conf.get_user_option(name)
+        self.assertLength(1, calls)
+        # Since we can't assert about conf, we just use the number of calls ;-/
+
+    def test_load_hook_bazaar(self):
+        self.assertLoadHook('file', config.GlobalConfig)
+
+    def test_load_hook_locations(self):
+        self.assertLoadHook('file', config.LocationConfig, self.tree.basedir)
+
+    def test_load_hook_branch(self):
+        self.assertLoadHook('file', config.BranchConfig, self.tree.branch)
+
+    def assertSaveHook(self, conf):
+        calls = []
+        def hook(*args):
+            calls.append(args)
+        config.OldConfigHooks.install_named_hook('save', hook, None)
+        self.addCleanup(
+            config.OldConfigHooks.uninstall_named_hook, 'save', None)
+        self.assertLength(0, calls)
+        # Setting an option triggers a save
+        conf.set_user_option('foo', 'bar')
+        self.assertLength(1, calls)
+        # Since we can't assert about conf, we just use the number of calls ;-/
+
+    def test_save_hook_bazaar(self):
+        self.assertSaveHook(self.bazaar_config)
+
+    def test_save_hook_locations(self):
+        self.assertSaveHook(self.locations_config)
+
+    def test_save_hook_branch(self):
+        self.assertSaveHook(self.branch_config)
+
+
+class TestOldConfigHooksForRemote(tests.TestCaseWithTransport):
+    """Tests config hooks for remote configs.
+
+    No tests for the remove hook as this is not implemented there.
+    """
+
+    def setUp(self):
+        super(TestOldConfigHooksForRemote, self).setUp()
+        self.transport_server = test_server.SmartTCPServer_for_testing
+        create_configs_with_file_option(self)
+
+    def assertGetHook(self, conf, name, value):
+        calls = []
+        def hook(*args):
+            calls.append(args)
+        config.OldConfigHooks.install_named_hook('get', hook, None)
+        self.addCleanup(
+            config.OldConfigHooks.uninstall_named_hook, 'get', None)
+        self.assertLength(0, calls)
+        actual_value = conf.get_option(name)
+        self.assertEquals(value, actual_value)
+        self.assertLength(1, calls)
+        self.assertEquals((conf, name, value), calls[0])
+
+    def test_get_hook_remote_branch(self):
+        remote_branch = branch.Branch.open(self.get_url('tree'))
+        self.assertGetHook(remote_branch._get_config(), 'file', 'branch')
+
+    def test_get_hook_remote_bzrdir(self):
+        remote_bzrdir = bzrdir.BzrDir.open(self.get_url('tree'))
+        conf = remote_bzrdir._get_config()
+        conf.set_option('remotedir', 'file')
+        self.assertGetHook(conf, 'file', 'remotedir')
+
+    def assertSetHook(self, conf, name, value):
+        calls = []
+        def hook(*args):
+            calls.append(args)
+        config.OldConfigHooks.install_named_hook('set', hook, None)
+        self.addCleanup(
+            config.OldConfigHooks.uninstall_named_hook, 'set', None)
+        self.assertLength(0, calls)
+        conf.set_option(value, name)
+        self.assertLength(1, calls)
+        # We can't assert the conf object below as different configs use
+        # different means to implement set_user_option and we care only about
+        # coverage here.
+        self.assertEquals((name, value), calls[0][1:])
+
+    def test_set_hook_remote_branch(self):
+        remote_branch = branch.Branch.open(self.get_url('tree'))
+        self.addCleanup(remote_branch.lock_write().unlock)
+        self.assertSetHook(remote_branch._get_config(), 'file', 'remote')
+
+    def test_set_hook_remote_bzrdir(self):
+        remote_branch = branch.Branch.open(self.get_url('tree'))
+        self.addCleanup(remote_branch.lock_write().unlock)
+        remote_bzrdir = bzrdir.BzrDir.open(self.get_url('tree'))
+        self.assertSetHook(remote_bzrdir._get_config(), 'file', 'remotedir')
+
+    def assertLoadHook(self, expected_nb_calls, name, conf_class, *conf_args):
+        calls = []
+        def hook(*args):
+            calls.append(args)
+        config.OldConfigHooks.install_named_hook('load', hook, None)
+        self.addCleanup(
+            config.OldConfigHooks.uninstall_named_hook, 'load', None)
+        self.assertLength(0, calls)
+        # Build a config
+        conf = conf_class(*conf_args)
+        # Access an option to trigger a load
+        conf.get_option(name)
+        self.assertLength(expected_nb_calls, calls)
+        # Since we can't assert about conf, we just use the number of calls ;-/
+
+    def test_load_hook_remote_branch(self):
+        remote_branch = branch.Branch.open(self.get_url('tree'))
+        self.assertLoadHook(1, 'file', remote.RemoteBranchConfig, remote_branch)
+
+    def test_load_hook_remote_bzrdir(self):
+        remote_bzrdir = bzrdir.BzrDir.open(self.get_url('tree'))
+        # The config file doesn't exist, set an option to force its creation
+        conf = remote_bzrdir._get_config()
+        conf.set_option('remotedir', 'file')
+        # We get one call for the server and one call for the client, this is
+        # caused by the differences in implementations betwen
+        # SmartServerBzrDirRequestConfigFile (in smart/bzrdir.py) and
+        # SmartServerBranchGetConfigFile (in smart/branch.py)
+        self.assertLoadHook(2 ,'file', remote.RemoteBzrDirConfig, remote_bzrdir)
+
+    def assertSaveHook(self, conf):
+        calls = []
+        def hook(*args):
+            calls.append(args)
+        config.OldConfigHooks.install_named_hook('save', hook, None)
+        self.addCleanup(
+            config.OldConfigHooks.uninstall_named_hook, 'save', None)
+        self.assertLength(0, calls)
+        # Setting an option triggers a save
+        conf.set_option('foo', 'bar')
+        self.assertLength(1, calls)
+        # Since we can't assert about conf, we just use the number of calls ;-/
+
+    def test_save_hook_remote_branch(self):
+        remote_branch = branch.Branch.open(self.get_url('tree'))
+        self.addCleanup(remote_branch.lock_write().unlock)
+        self.assertSaveHook(remote_branch._get_config())
+
+    def test_save_hook_remote_bzrdir(self):
+        remote_branch = branch.Branch.open(self.get_url('tree'))
+        self.addCleanup(remote_branch.lock_write().unlock)
+        remote_bzrdir = bzrdir.BzrDir.open(self.get_url('tree'))
+        self.assertSaveHook(remote_bzrdir._get_config())
+
+
 class TestOption(tests.TestCase):
 
     def test_default_value(self):
@@ -2084,6 +2353,84 @@
         self.assertRaises(AssertionError, store._load_from_string, 'bar=baz')
 
 
+class TestIniFileStoreContent(tests.TestCaseWithTransport):
+    """Simulate loading a config store without content of various encodings.
+
+    All files produced by bzr are in utf8 content.
+
+    Users may modify them manually and end up with a file that can't be
+    loaded. We need to issue proper error messages in this case.
+    """
+
+    invalid_utf8_char = '\xff'
+
+    def test_load_utf8(self):
+        """Ensure we can load an utf8-encoded file."""
+        t = self.get_transport()
+        # From http://pad.lv/799212
+        unicode_user = u'b\N{Euro Sign}ar'
+        unicode_content = u'user=%s' % (unicode_user,)
+        utf8_content = unicode_content.encode('utf8')
+        # Store the raw content in the config file
+        t.put_bytes('foo.conf', utf8_content)
+        store = config.IniFileStore(t, 'foo.conf')
+        store.load()
+        stack = config.Stack([store.get_sections], store)
+        self.assertEquals(unicode_user, stack.get('user'))
+
+    def test_load_non_ascii(self):
+        """Ensure we display a proper error on non-ascii, non utf-8 content."""
+        t = self.get_transport()
+        t.put_bytes('foo.conf', 'user=foo\n#%s\n' % (self.invalid_utf8_char,))
+        store = config.IniFileStore(t, 'foo.conf')
+        self.assertRaises(errors.ConfigContentError, store.load)
+
+    def test_load_erroneous_content(self):
+        """Ensure we display a proper error on content that can't be parsed."""
+        t = self.get_transport()
+        t.put_bytes('foo.conf', '[open_section\n')
+        store = config.IniFileStore(t, 'foo.conf')
+        self.assertRaises(errors.ParseConfigError, store.load)
+
+
+class TestIniConfigContent(tests.TestCaseWithTransport):
+    """Simulate loading a IniBasedConfig without content of various encodings.
+
+    All files produced by bzr are in utf8 content.
+
+    Users may modify them manually and end up with a file that can't be
+    loaded. We need to issue proper error messages in this case.
+    """
+
+    invalid_utf8_char = '\xff'
+
+    def test_load_utf8(self):
+        """Ensure we can load an utf8-encoded file."""
+        # From http://pad.lv/799212
+        unicode_user = u'b\N{Euro Sign}ar'
+        unicode_content = u'user=%s' % (unicode_user,)
+        utf8_content = unicode_content.encode('utf8')
+        # Store the raw content in the config file
+        with open('foo.conf', 'wb') as f:
+            f.write(utf8_content)
+        conf = config.IniBasedConfig(file_name='foo.conf')
+        self.assertEquals(unicode_user, conf.get_user_option('user'))
+
+    def test_load_badly_encoded_content(self):
+        """Ensure we display a proper error on non-ascii, non utf-8 content."""
+        with open('foo.conf', 'wb') as f:
+            f.write('user=foo\n#%s\n' % (self.invalid_utf8_char,))
+        conf = config.IniBasedConfig(file_name='foo.conf')
+        self.assertRaises(errors.ConfigContentError, conf._get_parser)
+
+    def test_load_erroneous_content(self):
+        """Ensure we display a proper error on content that can't be parsed."""
+        with open('foo.conf', 'wb') as f:
+            f.write('[open_section\n')
+        conf = config.IniBasedConfig(file_name='foo.conf')
+        self.assertRaises(errors.ParseConfigError, conf._get_parser)
+
+
 class TestMutableStore(TestStore):
 
     scenarios = [(key, {'store_id': key, 'get_store': builder}) for key, builder
@@ -2167,6 +2514,36 @@
         self.assertLength(1, sections)
         self.assertSectionContent(('baz', {'foo': 'bar'}), sections[0])
 
+    def test_load_hook(self):
+        # We first needs to ensure that the store exists
+        store = self.get_store(self)
+        section = store.get_mutable_section('baz')
+        section.set('foo', 'bar')
+        store.save()
+        # Now we can try to load it
+        store = self.get_store(self)
+        calls = []
+        def hook(*args):
+            calls.append(args)
+        config.ConfigHooks.install_named_hook('load', hook, None)
+        self.assertLength(0, calls)
+        store.load()
+        self.assertLength(1, calls)
+        self.assertEquals((store,), calls[0])
+
+    def test_save_hook(self):
+        calls = []
+        def hook(*args):
+            calls.append(args)
+        config.ConfigHooks.install_named_hook('save', hook, None)
+        self.assertLength(0, calls)
+        store = self.get_store(self)
+        section = store.get_mutable_section('baz')
+        section.set('foo', 'bar')
+        store.save()
+        self.assertLength(1, calls)
+        self.assertEquals((store,), calls[0])
+
 
 class TestIniFileStore(TestStore):
 
@@ -2494,10 +2871,6 @@
         conf_stack = config.Stack([conf1, conf2])
         self.assertEquals('baz', conf_stack.get('foo'))
 
-    def test_get_for_empty_stack(self):
-        conf_stack = config.Stack([])
-        self.assertEquals(None, conf_stack.get('foo'))
-
     def test_get_for_empty_section_callable(self):
         conf_stack = config.Stack([lambda : []])
         self.assertEquals(None, conf_stack.get('foo'))
@@ -2521,6 +2894,26 @@
         stack = self.get_stack(self)
 
 
+class TestStackGet(TestStackWithTransport):
+
+    def test_get_for_empty_stack(self):
+        conf = self.get_stack(self)
+        self.assertEquals(None, conf.get('foo'))
+
+    def test_get_hook(self):
+        conf = self.get_stack(self)
+        conf.store._load_from_string('foo=bar')
+        calls = []
+        def hook(*args):
+            calls.append(args)
+        config.ConfigHooks.install_named_hook('get', hook, None)
+        self.assertLength(0, calls)
+        value = conf.get('foo')
+        self.assertEquals('bar', value)
+        self.assertLength(1, calls)
+        self.assertEquals((conf, 'foo', 'bar'), calls[0])
+
+
 class TestStackSet(TestStackWithTransport):
 
     def test_simple_set(self):
@@ -2536,6 +2929,17 @@
         conf.set('foo', 'baz')
         self.assertEquals, 'baz', conf.get('foo')
 
+    def test_set_hook(self):
+        calls = []
+        def hook(*args):
+            calls.append(args)
+        config.ConfigHooks.install_named_hook('set', hook, None)
+        self.assertLength(0, calls)
+        conf = self.get_stack(self)
+        conf.set('foo', 'bar')
+        self.assertLength(1, calls)
+        self.assertEquals((conf, 'foo', 'bar'), calls[0])
+
 
 class TestStackRemove(TestStackWithTransport):
 
@@ -2551,6 +2955,18 @@
         conf = self.get_stack(self)
         self.assertRaises(KeyError, conf.remove, 'I_do_not_exist')
 
+    def test_remove_hook(self):
+        calls = []
+        def hook(*args):
+            calls.append(args)
+        config.ConfigHooks.install_named_hook('remove', hook, None)
+        self.assertLength(0, calls)
+        conf = self.get_stack(self)
+        conf.store._load_from_string('foo=bar')
+        conf.remove('foo')
+        self.assertLength(1, calls)
+        self.assertEquals((conf, 'foo'), calls[0])
+
 
 class TestConfigGetOptions(tests.TestCaseWithTransport, TestOptionsMixin):
 
@@ -2723,6 +3139,11 @@
         self.assertEquals({}, conf._get_config())
         self._got_user_passwd(None, None, conf, 'http', 'foo.net')
 
+    def test_non_utf8_config(self):
+        conf = config.AuthenticationConfig(_file=StringIO(
+                'foo = bar\xff'))
+        self.assertRaises(errors.ConfigContentError, conf._get_config)
+        
     def test_missing_auth_section_header(self):
         conf = config.AuthenticationConfig(_file=StringIO('foo = bar'))
         self.assertRaises(ValueError, conf.get_credentials, 'ftp', 'foo.net')
@@ -3216,7 +3637,8 @@
         to be able to choose a user name with no configuration.
         """
         if sys.platform == 'win32':
-            raise TestSkipped("User name inference not implemented on win32")
+            raise tests.TestSkipped(
+                "User name inference not implemented on win32")
         realname, address = config._auto_user_id()
         if os.path.exists('/etc/mailname'):
             self.assertIsNot(None, realname)

=== modified file 'bzrlib/tests/test_conflicts.py'
--- bzrlib/tests/test_conflicts.py	2011-05-21 16:29:38 +0000
+++ bzrlib/tests/test_conflicts.py	2011-06-21 15:19:18 +0000
@@ -446,6 +446,14 @@
               dict(actions='modify_file', check='file_has_more_content')),
              ('file_deleted',
               dict(actions='delete_file', check='file_doesnt_exist')),),
+            # File renamed-modified/deleted
+            (dict(_base_actions='create_file',
+                  _path='new-file', _file_id='file-id'),
+             ('file_renamed_and_modified',
+              dict(actions='modify_and_rename_file',
+                   check='file_renamed_and_more_content')),
+             ('file_deleted',
+              dict(actions='delete_file', check='file_doesnt_exist')),),
             # File modified/deleted in dir
             (dict(_base_actions='create_file_in_dir',
                   _path='dir/file', _file_id='file-id'),
@@ -463,9 +471,16 @@
     def do_modify_file(self):
         return [('modify', ('file-id', 'trunk content\nmore content\n'))]
 
+    def do_modify_and_rename_file(self):
+        return [('modify', ('file-id', 'trunk content\nmore content\n')),
+                ('rename', ('file', 'new-file'))]
+
     def check_file_has_more_content(self):
         self.assertFileEqual('trunk content\nmore content\n', 'branch/file')
 
+    def check_file_renamed_and_more_content(self):
+        self.assertFileEqual('trunk content\nmore content\n', 'branch/new-file')
+
     def do_delete_file(self):
         return [('unversion', 'file-id')]
 

=== modified file 'bzrlib/tests/test_export.py'
--- bzrlib/tests/test_export.py	2011-06-13 17:03:20 +0000
+++ bzrlib/tests/test_export.py	2011-06-28 13:55:39 +0000
@@ -149,6 +149,25 @@
         self.assertEqual(a_time, t.stat('a').st_mtime)
         self.assertEqual(b_time, t.stat('b').st_mtime)
 
+    def test_subdir_files_per_timestamps(self):
+        builder = self.make_branch_builder('source')
+        builder.start_series()
+        foo_time = time.mktime((1999, 12, 12, 0, 0, 0, 0, 0, 0))
+        builder.build_snapshot(None, None, [
+            ('add', ('', 'root-id', 'directory', '')),
+            ('add', ('subdir', 'subdir-id', 'directory', '')),
+            ('add', ('subdir/foo.txt', 'foo-id', 'file', 'content\n'))],
+            timestamp=foo_time)
+        builder.finish_series()
+        b = builder.get_branch()
+        b.lock_read()
+        self.addCleanup(b.unlock)
+        tree = b.basis_tree()
+        export.export(tree, 'target', format='dir', subdir='subdir',
+            per_file_timestamps=True)
+        t = self.get_transport('target')
+        self.assertEquals(foo_time, t.stat('foo.txt').st_mtime)
+
 
 class TarExporterTests(tests.TestCaseWithTransport):
 

=== modified file 'bzrlib/tests/test_export_pot.py'
--- bzrlib/tests/test_export_pot.py	2011-05-11 17:44:45 +0000
+++ bzrlib/tests/test_export_pot.py	2011-06-28 10:28:29 +0000
@@ -18,10 +18,14 @@
 import textwrap
 
 from bzrlib import (
+    commands,
     export_pot,
     tests,
     )
 
+import re
+
+
 class TestEscape(tests.TestCase):
 
     def test_simple_escape(self):
@@ -145,3 +149,45 @@
                 "EGG\\n"
                 msgstr ""\n
                 ''')
+
+
+class TestExportCommandHelp(PoEntryTestCase):
+
+    def test_command_help(self):
+
+        class cmd_Demo(commands.Command):
+            __doc__ = """A sample command.
+
+            :Usage:
+                bzr demo
+
+            :Examples:
+                Example 1::
+
+                    cmd arg1
+
+            Blah Blah Blah
+            """
+
+        export_pot._write_command_help(self._outf, cmd_Demo())
+        result = self._outf.getvalue()
+        # We don't care about filename and lineno here.
+        result = re.sub(r'(?m)^#: [^\n]+\n', '', result)
+
+        self.assertEqualDiff(
+                'msgid "A sample command."\n'
+                'msgstr ""\n'
+                '\n'                # :Usage: should not be translated.
+                'msgid ""\n'
+                '":Examples:\\n"\n'
+                '"    Example 1::"\n'
+                'msgstr ""\n'
+                '\n'
+                'msgid "        cmd arg1"\n'
+                'msgstr ""\n'
+                '\n'
+                'msgid "Blah Blah Blah"\n'
+                'msgstr ""\n'
+                '\n',
+                result
+                )

=== modified file 'bzrlib/tests/test_foreign.py'
--- bzrlib/tests/test_foreign.py	2011-05-17 10:29:43 +0000
+++ bzrlib/tests/test_foreign.py	2011-06-16 12:50:32 +0000
@@ -163,12 +163,13 @@
         result.old_revno, result.old_revid = self.target.last_revision_info()
         self.source.lock_read()
         try:
+            graph = self.source.repository.get_graph()
             # This just handles simple cases, but that's good enough for tests
             my_history = self.target.revision_history()
             if stop_revision is None:
                 stop_revision = self.source.last_revision()
-            their_history = list(
-                self.source.repository.iter_reverse_revision_history(stop_revision))
+            their_history = list(graph.iter_lefthand_ancestry(stop_revision,
+                (revision.NULL_REVISION,)))
             their_history.reverse()
             if their_history[:min(len(my_history), len(their_history))] != my_history:
                 raise errors.DivergedBranches(self.target, self.source)
@@ -181,7 +182,7 @@
                     return (tree.get_file(file_id), None)
                 tree.get_file_with_stat = get_file_with_stat
                 new_revid = self.target.mapping.revision_id_foreign_to_bzr(
-                    (str(rev.timestamp), str(rev.timezone), 
+                    (str(rev.timestamp), str(rev.timezone),
                         str(self.target.revno())))
                 parent_revno, parent_revid= self.target.last_revision_info()
                 if parent_revid == revision.NULL_REVISION:

=== modified file 'bzrlib/tests/test_graph.py'
--- bzrlib/tests/test_graph.py	2011-02-19 17:37:45 +0000
+++ bzrlib/tests/test_graph.py	2011-06-20 11:03:53 +0000
@@ -21,7 +21,6 @@
     )
 from bzrlib.revision import NULL_REVISION
 from bzrlib.tests import TestCaseWithMemoryTransport
-from bzrlib.symbol_versioning import deprecated_in
 
 
 # Ancestry 1:
@@ -1647,6 +1646,15 @@
         self.assertEqual(['B', 'D'],
             sorted(graph_thunk.heads(['D', 'B', 'A'])))
 
+    def test_merge_sort(self):
+        d = {('C',):[('A',)], ('B',): [('A',)], ('A',): []}
+        g = _mod_graph.KnownGraph(d)
+        graph_thunk = _mod_graph.GraphThunkIdsToKeys(g)
+        graph_thunk.add_node("D", ["A", "C"])
+        self.assertEqual([('C', 0, (2,), False), ('A', 0, (1,), True)],
+            [(n.key, n.merge_depth, n.revno, n.end_of_merge)
+                 for n in graph_thunk.merge_sort('C')])
+
 
 class TestPendingAncestryResultGetKeys(TestCaseWithMemoryTransport):
     """Tests for bzrlib.graph.PendingAncestryResult."""

=== modified file 'bzrlib/tests/test_help.py'
--- bzrlib/tests/test_help.py	2011-01-12 01:01:53 +0000
+++ bzrlib/tests/test_help.py	2011-06-27 15:36:58 +0000
@@ -16,45 +16,63 @@
 
 """Unit tests for the bzrlib.help module."""
 
+import textwrap
+
 from bzrlib import (
     builtins,
     commands,
     errors,
     help,
     help_topics,
+    i18n,
     plugin,
     tests,
     )
 
-
-class TestHelp(tests.TestCase):
-
-    def setUp(self):
-        tests.TestCase.setUp(self)
-        commands.install_bzr_command_hooks()
+from bzrlib.tests.test_i18n import ZzzTranslations
+import re
 
 
 class TestCommandHelp(tests.TestCase):
     """Tests for help on commands."""
 
+    def assertCmdHelp(self, expected, cmd):
+        self.assertEqualDiff(textwrap.dedent(expected), cmd.get_help_text())
+
     def test_command_help_includes_see_also(self):
         class cmd_WithSeeAlso(commands.Command):
             __doc__ = """A sample command."""
             _see_also = ['foo', 'bar']
-        cmd = cmd_WithSeeAlso()
-        helptext = cmd.get_help_text()
-        self.assertEndsWith(
-            helptext,
-            '  -v, --verbose  Display more information.\n'
-            '  -q, --quiet    Only display errors and warnings.\n'
-            '  -h, --help     Show help message.\n'
-            '\n'
-            'See also: bar, foo\n')
+        self.assertCmdHelp('''\
+            Purpose: A sample command.
+            Usage:   bzr WithSeeAlso
+            
+            Options:
+              --usage        Show usage message and options.
+              -v, --verbose  Display more information.
+              -q, --quiet    Only display errors and warnings.
+              -h, --help     Show help message.
+            
+            See also: bar, foo
+            ''',
+                           cmd_WithSeeAlso())
 
     def test_get_help_text(self):
         """Commands have a get_help_text method which returns their help."""
         class cmd_Demo(commands.Command):
             __doc__ = """A sample command."""
+        self.assertCmdHelp('''\
+            Purpose: A sample command.
+            Usage:   bzr Demo
+            
+            Options:
+              --usage        Show usage message and options.
+              -v, --verbose  Display more information.
+              -q, --quiet    Only display errors and warnings.
+              -h, --help     Show help message.
+
+            ''',
+                           cmd_Demo())
         cmd = cmd_Demo()
         helptext = cmd.get_help_text()
         self.assertStartsWith(helptext,
@@ -295,6 +313,182 @@
             '  Blah blah blah.\n\n')
 
 
+class ZzzTranslationsForDoc(ZzzTranslations):
+
+    _section_pat = re.compile(':\w+:\\n\\s+')
+    _indent_pat = re.compile('\\s+')
+
+    def zzz(self, s):
+        m = self._section_pat.match(s)
+        if m is None:
+            m = self._indent_pat.match(s)
+        if m:
+            return u'%szz{{%s}}' % (m.group(0), s[m.end():])
+        return u'zz{{%s}}' % s
+
+
+class TestCommandHelpI18n(tests.TestCase):
+    """Tests for help on translated commands."""
+
+    def setUp(self):
+        super(TestCommandHelpI18n, self).setUp()
+        self.overrideAttr(i18n, '_translations', ZzzTranslationsForDoc())
+
+    def assertCmdHelp(self, expected, cmd):
+        self.assertEqualDiff(textwrap.dedent(expected), cmd.get_help_text())
+
+    def test_command_help_includes_see_also(self):
+        class cmd_WithSeeAlso(commands.Command):
+            __doc__ = """A sample command."""
+            _see_also = ['foo', 'bar']
+        self.assertCmdHelp('''\
+            zz{{:Purpose: zz{{A sample command.}}
+            }}zz{{:Usage:   bzr WithSeeAlso
+            }}
+            zz{{:Options:
+              --usage        Show usage message and options.
+              -v, --verbose  Display more information.
+              -q, --quiet    Only display errors and warnings.
+              -h, --help     Show help message.
+            }}
+            zz{{:See also: bar, foo}}
+            ''',
+                           cmd_WithSeeAlso())
+
+    def test_get_help_text(self):
+        """Commands have a get_help_text method which returns their help."""
+        class cmd_Demo(commands.Command):
+            __doc__ = """A sample command."""
+        self.assertCmdHelp('''\
+            zz{{:Purpose: zz{{A sample command.}}
+            }}zz{{:Usage:   bzr Demo
+            }}
+            zz{{:Options:
+              --usage        Show usage message and options.
+              -v, --verbose  Display more information.
+              -q, --quiet    Only display errors and warnings.
+              -h, --help     Show help message.
+            }}
+            ''',
+                           cmd_Demo())
+
+    def test_command_with_additional_see_also(self):
+        class cmd_WithSeeAlso(commands.Command):
+            __doc__ = """A sample command."""
+            _see_also = ['foo', 'bar']
+        cmd = cmd_WithSeeAlso()
+        helptext = cmd.get_help_text(['gam'])
+        self.assertEndsWith(
+            helptext,
+            '  -v, --verbose  Display more information.\n'
+            '  -q, --quiet    Only display errors and warnings.\n'
+            '  -h, --help     Show help message.\n'
+            '}}\n'
+            'zz{{:See also: bar, foo, gam}}\n')
+
+    def test_command_only_additional_see_also(self):
+        class cmd_WithSeeAlso(commands.Command):
+            __doc__ = """A sample command."""
+        cmd = cmd_WithSeeAlso()
+        helptext = cmd.get_help_text(['gam'])
+        self.assertEndsWith(
+            helptext,
+            'zz{{:Options:\n'
+            '  --usage        Show usage message and options.\n'
+            '  -v, --verbose  Display more information.\n'
+            '  -q, --quiet    Only display errors and warnings.\n'
+            '  -h, --help     Show help message.\n'
+            '}}\n'
+            'zz{{:See also: gam}}\n')
+
+
+    def test_help_custom_section_ordering(self):
+        """Custom descriptive sections should remain in the order given."""
+        # The help formatter expect the class name to start with 'cmd_'
+        class cmd_Demo(commands.Command):
+            __doc__ = """A sample command.
+ 
+            Blah blah blah.
+
+            :Formats:
+              Interesting stuff about formats.
+
+            :Examples:
+              Example 1::
+ 
+                cmd arg1
+
+            :Tips:
+              Clever things to keep in mind.
+            """
+        self.assertCmdHelp('''\
+            zz{{:Purpose: zz{{A sample command.}}
+            }}zz{{:Usage:   bzr Demo
+            }}
+            zz{{:Options:
+              --usage        Show usage message and options.
+              -v, --verbose  Display more information.
+              -q, --quiet    Only display errors and warnings.
+              -h, --help     Show help message.
+            }}
+            Description:
+              zz{{zz{{Blah blah blah.}}
+            
+            }}:Formats:
+              zz{{Interesting stuff about formats.}}
+            
+            Examples:
+              zz{{Example 1::}}
+            
+                zz{{cmd arg1}}
+            
+            Tips:
+              zz{{Clever things to keep in mind.}}
+
+            ''',
+                           cmd_Demo())
+
+    def test_help_text_custom_usage(self):
+        """Help text may contain a custom usage section."""
+        class cmd_Demo(commands.Command):
+            __doc__ = """A sample command.
+
+            :Usage:
+                cmd Demo [opts] args
+
+                cmd Demo -h
+
+            Blah blah blah.
+            """
+        self.assertCmdHelp('''\
+            zz{{:Purpose: zz{{A sample command.}}
+            }}zz{{:Usage:
+                zz{{cmd Demo [opts] args}}
+            
+                zz{{cmd Demo -h}}
+
+            }}
+            zz{{:Options:
+              --usage        Show usage message and options.
+              -v, --verbose  Display more information.
+              -q, --quiet    Only display errors and warnings.
+              -h, --help     Show help message.
+            }}
+            Description:
+              zz{{zz{{Blah blah blah.}}
+
+            }}
+            ''',
+                           cmd_Demo())
+
+
+class TestHelp(tests.TestCase):
+
+    def setUp(self):
+        tests.TestCase.setUp(self)
+        commands.install_bzr_command_hooks()
+
+
 class TestRegisteredTopic(TestHelp):
     """Tests for the RegisteredTopic class."""
 
@@ -306,10 +500,10 @@
         self.assertEqual('basic', topic.topic)
 
     def test_get_help_text(self):
-        """A RegisteredTopic returns the get_detail results for get_help_text."""
+        """RegisteredTopic returns the get_detail results for get_help_text."""
         topic = help_topics.RegisteredTopic('commands')
         self.assertEqual(help_topics.topic_registry.get_detail('commands'),
-            topic.get_help_text())
+                         topic.get_help_text())
 
     def test_get_help_text_with_additional_see_also(self):
         topic = help_topics.RegisteredTopic('commands')
@@ -327,7 +521,7 @@
             '\n')
 
     def test_get_help_topic(self):
-        """The help topic for a RegisteredTopic is its topic from construction."""
+        """The help topic for RegisteredTopic is its topic from construction."""
         topic = help_topics.RegisteredTopic('foobar')
         self.assertEqual('foobar', topic.get_help_topic())
         topic = help_topics.RegisteredTopic('baz')

=== modified file 'bzrlib/tests/test_i18n.py'
--- bzrlib/tests/test_i18n.py	2011-05-27 06:37:07 +0000
+++ bzrlib/tests/test_i18n.py	2011-06-27 15:52:06 +0000
@@ -67,7 +67,7 @@
 
     def setUp(self):
         super(TestGetText, self).setUp()
-        self.overrideAttr(i18n, '_translation', ZzzTranslations())
+        self.overrideAttr(i18n, '_translations', ZzzTranslations())
 
     def test_oneline(self):
         self.assertEqual(u"zz{{spam ham eggs}}",
@@ -82,7 +82,7 @@
 
     def setUp(self):
         super(TestGetTextPerParagraph, self).setUp()
-        self.overrideAttr(i18n, '_translation', ZzzTranslations())
+        self.overrideAttr(i18n, '_translations', ZzzTranslations())
 
     def test_oneline(self):
         self.assertEqual(u"zz{{spam ham eggs}}",

=== modified file 'bzrlib/tests/test_lockdir.py'
--- bzrlib/tests/test_lockdir.py	2011-05-13 12:51:05 +0000
+++ bzrlib/tests/test_lockdir.py	2011-06-10 07:31:54 +0000
@@ -17,6 +17,7 @@
 """Tests for LockDir"""
 
 import os
+import sys
 import time
 
 import bzrlib
@@ -24,6 +25,7 @@
     config,
     errors,
     lock,
+    lockdir,
     osutils,
     tests,
     transport,
@@ -35,9 +37,13 @@
     LockFailed,
     LockNotHeld,
     )
-from bzrlib.lockdir import LockDir
+from bzrlib.lockdir import (
+    LockDir,
+    LockHeldInfo,
+    )
 from bzrlib.tests import (
     features,
+    TestCase,
     TestCaseWithTransport,
     )
 from bzrlib.trace import note
@@ -48,6 +54,7 @@
 # implementation are tested separately.  (The main requirement is just that
 # they don't allow overwriting nonempty directories.)
 
+
 class TestLockDir(TestCaseWithTransport):
     """Test LockDir operations"""
 
@@ -141,8 +148,8 @@
         self.addCleanup(lf1.unlock)
         # lock is held, should get some info on it
         info1 = lf1.peek()
-        self.assertEqual(set(info1.keys()),
-                         set(['user', 'nonce', 'hostname', 'pid', 'start_time']))
+        self.assertEqual(set(info1.info_dict.keys()),
+            set(['user', 'nonce', 'hostname', 'pid', 'start_time']))
         # should get the same info if we look at it through a different
         # instance
         info2 = LockDir(t, 'test_lock').peek()
@@ -161,7 +168,7 @@
         self.addCleanup(lf1.unlock)
         info2 = lf2.peek()
         self.assertTrue(info2)
-        self.assertEqual(info2['nonce'], lf1.nonce)
+        self.assertEqual(info2.get('nonce'), lf1.nonce)
 
     def test_30_lock_wait_fail(self):
         """Wait on a lock, then fail
@@ -184,24 +191,16 @@
             # it should only take about 0.4 seconds, but we allow more time in
             # case the machine is heavily loaded
             self.assertTrue(after - before <= 8.0,
-                    "took %f seconds to detect lock contention" % (after - before))
+                "took %f seconds to detect lock contention" % (after - before))
         finally:
             lf1.unlock()
         self.assertEqual(1, len(self._logged_reports))
-        self.assertEqual(self._logged_reports[0][0],
-            '%s lock %s held by %s\n'
-            'at %s [process #%s], acquired %s.\n'
-            'Will continue to try until %s, unless '
-            'you press Ctrl-C.\n'
-            'See "bzr help break-lock" for more.')
-        start, lock_url, user, hostname, pid, time_ago, deadline_str = \
-            self._logged_reports[0][1]
-        self.assertEqual(start, u'Unable to obtain')
-        self.assertEqual(user, u'jrandom@example.com')
-        # skip hostname
-        self.assertContainsRe(pid, r'\d+')
-        self.assertContainsRe(time_ago, r'.* ago')
-        self.assertContainsRe(deadline_str, r'\d{2}:\d{2}:\d{2}')
+        self.assertContainsRe(self._logged_reports[0][0],
+            r'Unable to obtain lock .* held by jrandom@example\.com on .*'
+            r' \(process #\d+\), acquired .* ago\.\n'
+            r'Will continue to try until \d{2}:\d{2}:\d{2}, unless '
+            r'you press Ctrl-C.\n'
+            r'See "bzr help break-lock" for more.')
 
     def test_31_lock_wait_easy(self):
         """Succeed when waiting on a lock with no contention.
@@ -349,12 +348,15 @@
         ld.create()
         ld.lock_write()
         ld.transport.put_bytes_non_atomic('test_lock/held/info', '\0')
+
         class LoggingUIFactory(bzrlib.ui.SilentUIFactory):
             def __init__(self):
                 self.prompts = []
+
             def get_boolean(self, prompt):
                 self.prompts.append(('boolean', prompt))
                 return True
+
         ui = LoggingUIFactory()
         self.overrideAttr(bzrlib.ui, 'ui_factory', ui)
         ld2.break_lock()
@@ -372,12 +374,15 @@
         ld.create()
         ld.lock_write()
         ld.transport.delete('test_lock/held/info')
+
         class LoggingUIFactory(bzrlib.ui.SilentUIFactory):
             def __init__(self):
                 self.prompts = []
+
             def get_boolean(self, prompt):
                 self.prompts.append(('boolean', prompt))
                 return True
+
         ui = LoggingUIFactory()
         orig_factory = bzrlib.ui.ui_factory
         bzrlib.ui.ui_factory = ui
@@ -416,18 +421,17 @@
         lf1.unlock()
         self.assertFalse(t.has('test_lock/held/info'))
 
-    def test__format_lock_info(self):
+    def test_display_form(self):
         ld1 = self.get_lock()
         ld1.create()
         ld1.lock_write()
         try:
-            info_list = ld1._format_lock_info(ld1.peek())
+            info_list = ld1.peek().to_readable_dict()
         finally:
             ld1.unlock()
-        self.assertEqual(info_list[0], u'jrandom@example.com')
-        # info_list[1] is hostname. we skip this.
-        self.assertContainsRe(info_list[2], '^\d+$') # pid
-        self.assertContainsRe(info_list[3], r'^\d+ seconds? ago$') # time_ago
+        self.assertEqual(info_list['user'], u'jrandom@example.com')
+        self.assertContainsRe(info_list['pid'], '^\d+$')
+        self.assertContainsRe(info_list['time_ago'], r'^\d+ seconds? ago$')
 
     def test_lock_without_email(self):
         global_config = config.GlobalConfig()
@@ -481,8 +485,10 @@
         # should be nothing before we start
         ld1.create()
         t = self.get_transport().clone('test_lock')
+
         def check_dir(a):
             self.assertEquals(a, t.list_dir('.'))
+
         check_dir([])
         # when held, that's all we see
         ld1.attempt_lock()
@@ -505,9 +511,10 @@
         t.put_bytes('test_lock/held/info', '')
         lf = LockDir(t, 'test_lock')
         info = lf.peek()
-        formatted_info = lf._format_lock_info(info)
+        formatted_info = info.to_readable_dict()
         self.assertEquals(
-            ['<unknown>', '<unknown>', '<unknown>', '(unknown)'],
+            dict(user='<unknown>', hostname='<unknown>', pid='<unknown>',
+                time_ago='(unknown)'),
             formatted_info)
 
     def test_corrupt_lockdir_info(self):
@@ -646,3 +653,108 @@
         ld2.force_break(holder_info)
         lock_path = ld.transport.abspath(ld.path)
         self.assertEqual([], self._calls)
+
+
+class TestLockHeldInfo(TestCase):
+    """Can get information about the lock holder, and detect whether they're
+    still alive."""
+
+    def test_repr(self):
+        info = LockHeldInfo.for_this_process(None)
+        self.assertContainsRe(repr(info), r"LockHeldInfo\(.*\)")
+
+    def test_unicode(self):
+        info = LockHeldInfo.for_this_process(None)
+        self.assertContainsRe(unicode(info),
+            r'held by .* on .* \(process #\d+\), acquired .* ago')
+
+    def test_is_locked_by_this_process(self):
+        info = LockHeldInfo.for_this_process(None)
+        self.assertTrue(info.is_locked_by_this_process())
+
+    def test_is_not_locked_by_this_process(self):
+        info = LockHeldInfo.for_this_process(None)
+        info.info_dict['pid'] = '123123123123123'
+        self.assertFalse(info.is_locked_by_this_process())
+
+    def test_lock_holder_live_process(self):
+        """Detect that the holder (this process) is still running."""
+        info = LockHeldInfo.for_this_process(None)
+        self.assertFalse(info.is_lock_holder_known_dead())
+
+    def test_lock_holder_dead_process(self):
+        """Detect that the holder (this process) is still running."""
+        info = LockHeldInfo.for_this_process(None)
+        info.info_dict['pid'] = '123123123'
+        if sys.platform == 'win32':
+            self.knownFailure(
+                'live lock holder detection not implemented yet on win32')
+        self.assertTrue(info.is_lock_holder_known_dead())
+
+    def test_lock_holder_other_machine(self):
+        """The lock holder isn't here so we don't know if they're alive."""
+        info = LockHeldInfo.for_this_process(None)
+        info.info_dict['hostname'] = 'egg.example.com'
+        info.info_dict['pid'] = '123123123'
+        self.assertFalse(info.is_lock_holder_known_dead())
+
+    def test_lock_holder_other_user(self):
+        """Only auto-break locks held by this user."""
+        info = LockHeldInfo.for_this_process(None)
+        info.info_dict['user'] = 'notme@example.com'
+        info.info_dict['pid'] = '123123123'
+        self.assertFalse(info.is_lock_holder_known_dead())
+
+    def test_no_good_hostname(self):
+        """Correctly handle ambiguous hostnames.
+
+        If the lock's recorded with just 'localhost' we can't really trust
+        it's the same 'localhost'.  (There are quite a few of them. :-)
+        So even if the process is known not to be alive, we can't say that's
+        known for sure.
+        """
+        self.overrideAttr(lockdir, 'get_host_name',
+            lambda: 'localhost')
+        info = LockHeldInfo.for_this_process(None)
+        info.info_dict['pid'] = '123123123'
+        self.assertFalse(info.is_lock_holder_known_dead())
+
+
+class TestStaleLockDir(TestCaseWithTransport):
+    """Can automatically break stale locks.
+
+    :see: https://bugs.launchpad.net/bzr/+bug/220464
+    """
+
+    def test_auto_break_stale_lock(self):
+        """Locks safely known to be stale are just cleaned up.
+
+        This generates a warning but no other user interaction.
+        """
+        # This is off by default at present; see the discussion in the bug.
+        # If you change the default, don't forget to update the docs.
+        config.GlobalConfig().set_user_option('locks.steal_dead', True)
+        # Create a lock pretending to come from a different nonexistent
+        # process on the same machine.
+        l1 = LockDir(self.get_transport(), 'a',
+            extra_holder_info={'pid': '12312313'})
+        token_1 = l1.attempt_lock()
+        l2 = LockDir(self.get_transport(), 'a')
+        token_2 = l2.attempt_lock()
+        # l1 will notice its lock was stolen.
+        self.assertRaises(errors.LockBroken,
+            l1.unlock)
+        l2.unlock()
+
+    def test_auto_break_stale_lock_configured_off(self):
+        """Automatic breaking can be turned off"""
+        l1 = LockDir(self.get_transport(), 'a',
+            extra_holder_info={'pid': '12312313'})
+        token_1 = l1.attempt_lock()
+        self.addCleanup(l1.unlock)
+        l2 = LockDir(self.get_transport(), 'a')
+        # This fails now, because dead lock breaking is off by default.
+        self.assertRaises(LockContention,
+            l2.attempt_lock)
+        # and it's in fact not broken
+        l1.confirm()

=== modified file 'bzrlib/tests/test_matchers.py'
--- bzrlib/tests/test_matchers.py	2010-05-06 11:08:10 +0000
+++ bzrlib/tests/test_matchers.py	2011-06-19 15:23:02 +0000
@@ -18,7 +18,10 @@
 
 from testtools.matchers import *
 
-from bzrlib.tests import TestCase
+from bzrlib.tests import (
+    TestCase,
+    TestCaseWithTransport,
+    )
 from bzrlib.tests.matchers import *
 
 
@@ -62,3 +65,42 @@
         self.assertNotEqual(None, mismatch)
         self.assertThat(mismatch.describe(), Equals("I am da tree is locked"))
 
+
+class TestMatchesAncestry(TestCaseWithTransport):
+
+    def test__str__(self):
+        matcher = MatchesAncestry("A repository", "arevid")
+        self.assertEqual(
+            "MatchesAncestry(repository='A repository', "
+            "revision_id='arevid')",
+            str(matcher))
+
+    def test_match(self):
+        b = self.make_branch_builder('.')
+        b.start_series()
+        revid1 = b.build_commit()
+        revid2 = b.build_commit()
+        b.finish_series()
+        branch = b.get_branch()
+        m = MatchesAncestry(branch.repository, revid2)
+        self.assertThat([revid2, revid1], m)
+        self.assertThat([revid1, revid2], m)
+        m = MatchesAncestry(branch.repository, revid1)
+        self.assertThat([revid1], m)
+        m = MatchesAncestry(branch.repository, "unknown")
+        self.assertThat(["unknown"], m)
+
+    def test_mismatch(self):
+        b = self.make_branch_builder('.')
+        b.start_series()
+        revid1 = b.build_commit()
+        revid2 = b.build_commit()
+        b.finish_series()
+        branch = b.get_branch()
+        m = MatchesAncestry(branch.repository, revid1)
+        mismatch = m.match([])
+        self.assertIsNot(None, mismatch)
+        self.assertEquals(
+            "mismatched ancestry for revision '%s' was ['%s'], expected []" % (
+                revid1, revid1),
+            mismatch.describe())

=== modified file 'bzrlib/tests/test_merge.py'
--- bzrlib/tests/test_merge.py	2011-05-26 08:05:45 +0000
+++ bzrlib/tests/test_merge.py	2011-06-09 18:56:55 +0000
@@ -121,6 +121,20 @@
         finally:
             wt1.unlock()
 
+    def test_merge_into_null_tree(self):
+        wt = self.make_branch_and_tree('tree')
+        null_tree = wt.basis_tree()
+        self.build_tree(['tree/file'])
+        wt.add('file')
+        wt.commit('tree with root')
+        merger = _mod_merge.Merge3Merger(null_tree, null_tree, null_tree, wt,
+                                         this_branch=wt.branch,
+                                         do_merge=False)
+        with merger.make_preview_transform() as tt:
+            self.assertEqual([], tt.find_conflicts())
+            preview = tt.get_preview_tree()
+            self.assertEqual(wt.get_root_id(), preview.get_root_id())
+
     def test_create_rename(self):
         """Rename an inventory entry while creating the file"""
         tree =self.make_branch_and_tree('.')
@@ -387,6 +401,26 @@
                              '>>>>>>> MERGE-SOURCE\n',
                              'this/file')
 
+    def test_merge_reverse_revision_range(self):
+        tree = self.make_branch_and_tree(".")
+        tree.lock_write()
+        self.addCleanup(tree.unlock)
+        self.build_tree(['a'])
+        tree.add('a')
+        tree.commit("added a")
+        first_rev = tree.branch.revision_history()[0]
+        merger = _mod_merge.Merger.from_revision_ids(None, tree,
+                                          _mod_revision.NULL_REVISION,
+                                          first_rev)
+        merger.merge_type = _mod_merge.Merge3Merger
+        merger.interesting_files = 'a'
+        conflict_count = merger.do_merge()
+        self.assertEqual(0, conflict_count)
+
+        self.assertPathDoesNotExist("a")
+        tree.revert()
+        self.assertPathExists("a")
+
     def test_make_merger(self):
         this_tree = self.make_branch_and_tree('this')
         this_tree.commit('rev1', rev_id='rev1')

=== modified file 'bzrlib/tests/test_osutils.py'
--- bzrlib/tests/test_osutils.py	2011-05-13 12:51:05 +0000
+++ bzrlib/tests/test_osutils.py	2011-06-16 18:34:26 +0000
@@ -2052,6 +2052,7 @@
         # Whatever the result is, if we don't raise an exception, it's ok.
         osutils.terminal_width()
 
+
 class TestCreationOps(tests.TestCaseInTempDir):
     _test_needs_features = [features.chown_feature]
 
@@ -2087,6 +2088,7 @@
         self.assertEquals(self.uid, s.st_uid)
         self.assertEquals(self.gid, s.st_gid)
 
+
 class TestGetuserUnicode(tests.TestCase):
 
     def test_ascii_user(self):
@@ -2107,6 +2109,13 @@
         self.overrideEnv('LOGNAME', u'jrandom\xb6'.encode(ue))
         self.assertEqual(u'jrandom\xb6', osutils.getuser_unicode())
 
+    def test_no_username_bug_660174(self):
+        self.requireFeature(features.win32_feature)
+        for name in ('LOGNAME', 'USER', 'LNAME', 'USERNAME'):
+            self.overrideEnv(name, None)
+        self.assertEqual(u'UNKNOWN', osutils.getuser_unicode())
+
+
 class TestBackupNames(tests.TestCase):
 
     def setUp(self):

=== modified file 'bzrlib/tests/test_remote.py'
--- bzrlib/tests/test_remote.py	2011-04-27 10:55:49 +0000
+++ bzrlib/tests/test_remote.py	2011-06-19 17:49:43 +0000
@@ -32,7 +32,7 @@
     config,
     controldir,
     errors,
-    graph,
+    graph as _mod_graph,
     inventory,
     inventory_delta,
     remote,
@@ -3244,12 +3244,14 @@
         _, stacked = branch_factory()
         source = stacked.repository._get_source(target_repository_format)
         tip = stacked.last_revision()
-        revs = stacked.repository.get_ancestry(tip)
-        search = graph.PendingAncestryResult([tip], stacked.repository)
+        stacked.repository._ensure_real()
+        graph = stacked.repository.get_graph()
+        revs = [r for (r,ps) in graph.iter_ancestry([tip])
+                if r != NULL_REVISION]
+        revs.reverse()
+        search = _mod_graph.PendingAncestryResult([tip], stacked.repository)
         self.reset_smart_call_log()
         stream = source.get_stream(search)
-        if None in revs:
-            revs.remove(None)
         # We trust that if a revision is in the stream the rest of the new
         # content for it is too, as per our main fetch tests; here we are
         # checking that the revisions are actually included at all, and their
@@ -3294,7 +3296,7 @@
         self.assertEqual(expected_revs, rev_ord)
         # Getting topological sort requires VFS calls still - one of which is
         # pushing up from the bound branch.
-        self.assertLength(13, self.hpss_calls)
+        self.assertLength(14, self.hpss_calls)
 
     def test_stacked_get_stream_groupcompress(self):
         # Repository._get_source.get_stream() from a stacked repository with
@@ -3358,7 +3360,7 @@
         remote_branch = bzrdir.BzrDir.open(remote_branch_url).open_branch()
         self.hpss_calls = []
         local.repository.fetch(remote_branch.repository,
-                fetch_spec=graph.EverythingResult(remote_branch.repository))
+                fetch_spec=_mod_graph.EverythingResult(remote_branch.repository))
         self.assertEqual(['Repository.get_stream_1.19'], self.hpss_calls)
 
     def override_verb(self, verb_name, verb):
@@ -3394,7 +3396,7 @@
         remote_branch = bzrdir.BzrDir.open(remote_branch_url).open_branch()
         self.hpss_calls = []
         local.repository.fetch(remote_branch.repository,
-                fetch_spec=graph.EverythingResult(remote_branch.repository))
+                fetch_spec=_mod_graph.EverythingResult(remote_branch.repository))
         # make sure the overridden verb was used
         self.assertLength(1, verb_log)
         # more than one HPSS call is needed, but because it's a VFS callback

=== modified file 'bzrlib/tests/test_revert.py'
--- bzrlib/tests/test_revert.py	2011-05-13 12:51:05 +0000
+++ bzrlib/tests/test_revert.py	2011-06-15 11:36:05 +0000
@@ -134,15 +134,6 @@
         self.assertPathDoesNotExist('file')
         self.assertEqual({}, tree.merge_modified())
 
-    def test_empty_deprecated(self):
-        tree = self.make_branch_and_tree('.')
-        self.build_tree(['file'])
-        tree.add('file')
-        self.callDeprecated(['Using [] to revert all files is deprecated'
-            ' as of bzr 0.91.  Please use None (the default) instead.'],
-            tree.revert, [])
-        self.assertIs(None, tree.path2id('file'))
-
     def test_revert_file_in_deleted_dir(self):
         tree = self.make_branch_and_tree('.')
         self.build_tree(['dir/', 'dir/file1', 'dir/file2'])

=== modified file 'bzrlib/tests/test_revision.py'
--- bzrlib/tests/test_revision.py	2011-01-12 01:01:53 +0000
+++ bzrlib/tests/test_revision.py	2011-06-19 16:23:04 +0000
@@ -27,7 +27,7 @@
     )
 from bzrlib.revision import NULL_REVISION
 from bzrlib.tests import TestCase, TestCaseWithTransport
-from bzrlib.trace import mutter
+from bzrlib.tests.matchers import MatchesAncestry
 
 # We're allowed to test deprecated interfaces
 warnings.filterwarnings('ignore',
@@ -104,7 +104,7 @@
              ('a@u-0-5', ['a@u-0-0', 'a@u-0-1', 'a@u-0-2', 'a@u-0-3', 'a@u-0-4',
                           'b@u-0-3', 'b@u-0-4',
                           'b@u-0-5', 'a@u-0-5']),
-             ('b@u-0-6', ['a@u-0-0', 'a@u-0-1', 'a@u-0-2',
+             ('b@u-0-6', ['a@u-0-0', 'a@u-0-1', 'a@u-0-2', 'a@u-0-4',
                           'b@u-0-3', 'b@u-0-4',
                           'b@u-0-5', 'b@u-0-6']),
              ]
@@ -116,10 +116,8 @@
                     continue
                 if rev_id in br2_only and not branch is br2:
                     continue
-                mutter('ancestry of {%s}: %r',
-                       rev_id, branch.repository.get_ancestry(rev_id))
-                result = sorted(branch.repository.get_ancestry(rev_id))
-                self.assertEquals(result, [None] + sorted(anc))
+                self.assertThat(anc,
+                    MatchesAncestry(branch.repository, rev_id))
 
 
 class TestIntermediateRevisions(TestCaseWithTransport):

=== modified file 'bzrlib/tests/test_selftest.py'
--- bzrlib/tests/test_selftest.py	2011-06-09 15:13:54 +0000
+++ bzrlib/tests/test_selftest.py	2011-06-17 13:59:23 +0000
@@ -43,6 +43,7 @@
     branchbuilder,
     bzrdir,
     errors,
+    hooks,
     lockdir,
     memorytree,
     osutils,
@@ -2505,17 +2506,24 @@
 
 
 class TestStartBzrSubProcess(tests.TestCase):
+    """Stub test start_bzr_subprocess."""
 
-    def check_popen_state(self):
-        """Replace to make assertions when popen is called."""
+    def _subprocess_log_cleanup(self):
+        """Inhibits the base version as we don't produce a log file."""
 
     def _popen(self, *args, **kwargs):
-        """Record the command that is run, so that we can ensure it is correct"""
+        """Override the base version to record the command that is run.
+
+        From there we can ensure it is correct without spawning a real process.
+        """
         self.check_popen_state()
         self._popen_args = args
         self._popen_kwargs = kwargs
         raise _DontSpawnProcess()
 
+    def check_popen_state(self):
+        """Replace to make assertions when popen is called."""
+
     def test_run_bzr_subprocess_no_plugins(self):
         self.assertRaises(_DontSpawnProcess, self.start_bzr_subprocess, [])
         command = self._popen_args[0]
@@ -2525,7 +2533,7 @@
 
     def test_allow_plugins(self):
         self.assertRaises(_DontSpawnProcess, self.start_bzr_subprocess, [],
-            allow_plugins=True)
+                          allow_plugins=True)
         command = self._popen_args[0]
         self.assertEqual([], command[2:])
 
@@ -2536,7 +2544,7 @@
             self.assertEqual('set variable', os.environ['EXISTANT_ENV_VAR'])
         self.check_popen_state = check_environment
         self.assertRaises(_DontSpawnProcess, self.start_bzr_subprocess, [],
-            env_changes={'EXISTANT_ENV_VAR':'set variable'})
+                          env_changes={'EXISTANT_ENV_VAR':'set variable'})
         # not set in theparent
         self.assertFalse('EXISTANT_ENV_VAR' in os.environ)
 
@@ -2548,7 +2556,7 @@
         os.environ['EXISTANT_ENV_VAR'] = 'set variable'
         self.check_popen_state = check_environment
         self.assertRaises(_DontSpawnProcess, self.start_bzr_subprocess, [],
-            env_changes={'EXISTANT_ENV_VAR':None})
+                          env_changes={'EXISTANT_ENV_VAR':None})
         # Still set in parent
         self.assertEqual('set variable', os.environ['EXISTANT_ENV_VAR'])
         del os.environ['EXISTANT_ENV_VAR']
@@ -2559,7 +2567,7 @@
             self.assertFalse('NON_EXISTANT_ENV_VAR' in os.environ)
         self.check_popen_state = check_environment
         self.assertRaises(_DontSpawnProcess, self.start_bzr_subprocess, [],
-            env_changes={'NON_EXISTANT_ENV_VAR':None})
+                          env_changes={'NON_EXISTANT_ENV_VAR':None})
 
     def test_working_dir(self):
         """Test that we can specify the working dir for the child"""
@@ -2568,18 +2576,12 @@
         chdirs = []
         def chdir(path):
             chdirs.append(path)
-        os.chdir = chdir
-        try:
-            def getcwd():
-                return 'current'
-            osutils.getcwd = getcwd
-            try:
-                self.assertRaises(_DontSpawnProcess, self.start_bzr_subprocess, [],
-                    working_dir='foo')
-            finally:
-                osutils.getcwd = orig_getcwd
-        finally:
-            os.chdir = orig_chdir
+        self.overrideAttr(os, 'chdir', chdir)
+        def getcwd():
+            return 'current'
+        self.overrideAttr(osutils, 'getcwd', getcwd)
+        self.assertRaises(_DontSpawnProcess, self.start_bzr_subprocess, [],
+                          working_dir='foo')
         self.assertEqual(['foo', 'current'], chdirs)
 
     def test_get_bzr_path_with_cwd_bzrlib(self):
@@ -3559,3 +3561,44 @@
 
     def test_mutiple_excludes(self):
         self.assertTestList(['c'], '-x', 'a', '-x', 'b')
+
+
+class TestCounterHooks(tests.TestCase, SelfTestHelper):
+
+    _test_needs_features = [features.subunit]
+
+    def setUp(self):
+        super(TestCounterHooks, self).setUp()
+        class Test(tests.TestCase):
+
+            def setUp(self):
+                super(Test, self).setUp()
+                self.hooks = hooks.Hooks()
+                self.hooks.add_hook('myhook', 'Foo bar blah', (2,4))
+                self.install_counter_hook(self.hooks, 'myhook')
+
+            def no_hook(self):
+                pass
+
+            def run_hook_once(self):
+                for hook in self.hooks['myhook']:
+                    hook(self)
+
+        self.test_class = Test
+
+    def assertHookCalls(self, expected_calls, test_name):
+        test = self.test_class(test_name)
+        result = unittest.TestResult()
+        test.run(result)
+        self.assertTrue(hasattr(test, '_counters'))
+        self.assertTrue(test._counters.has_key('myhook'))
+        self.assertEquals(expected_calls, test._counters['myhook'])
+
+    def test_no_hook(self):
+        self.assertHookCalls(0, 'no_hook')
+
+    def test_run_hook_once(self):
+        tt = features.testtools
+        if tt.module.__version__ < (0, 9, 8):
+            raise tests.TestSkipped('testtools-0.9.8 required for addDetail')
+        self.assertHookCalls(1, 'run_hook_once')

=== modified file 'bzrlib/tests/test_shelf.py'
--- bzrlib/tests/test_shelf.py	2011-05-13 12:51:05 +0000
+++ bzrlib/tests/test_shelf.py	2011-06-10 08:58:30 +0000
@@ -568,19 +568,18 @@
         list(creator.iter_shelvable())
         creator.shelve_deletion('foo-id')
         creator.shelve_deletion('bar-id')
-        shelf_file = open('shelf', 'w+b')
-        self.addCleanup(shelf_file.close)
-        creator.write_shelf(shelf_file)
-        creator.transform()
-        creator.finalize()
+        with open('shelf', 'w+b') as shelf_file:
+            creator.write_shelf(shelf_file)
+            creator.transform()
+            creator.finalize()
         # validate the test setup
         self.assertTrue('foo-id' in tree)
         self.assertTrue('bar-id' in tree)
         self.assertFileEqual('baz', 'tree/foo/bar')
-        shelf_file.seek(0)
-        unshelver = shelf.Unshelver.from_tree_and_shelf(tree, shelf_file)
-        self.addCleanup(unshelver.finalize)
-        unshelver.make_merger().do_merge()
+        with open('shelf', 'r+b') as shelf_file:
+            unshelver = shelf.Unshelver.from_tree_and_shelf(tree, shelf_file)
+            self.addCleanup(unshelver.finalize)
+            unshelver.make_merger().do_merge()
         self.assertFalse('foo-id' in tree)
         self.assertFalse('bar-id' in tree)
 

=== modified file 'bzrlib/tests/test_shelf_ui.py'
--- bzrlib/tests/test_shelf_ui.py	2011-06-09 14:40:22 +0000
+++ bzrlib/tests/test_shelf_ui.py	2011-06-13 14:44:22 +0000
@@ -311,10 +311,15 @@
                                 from_revision=revision.NULL_REVISION)
         tree1.commit('Replaced root entry')
         # This is essentially assertNotRaises(InconsistentDelta)
-        self.expectFailure('Cannot shelve replacing a root entry',
-                           self.assertRaises, AssertionError,
-                           self.assertRaises, errors.InconsistentDelta,
-                           self.shelve_all, tree1, rev2)
+        # With testtools 0.99, it can be rewritten as:
+        # with ExpectedException(AssertionError,
+        #                        'InconsistentDelta not raised'):
+        #     with ExpectedException(errors.InconsistentDelta, ''):
+        #         self.shelve_all(tree1, rev2)
+        e = self.assertRaises(AssertionError, self.assertRaises,
+                              errors.InconsistentDelta, self.shelve_all, tree1,
+                              rev2)
+        self.assertContainsRe('InconsistentDelta not raised', str(e))
 
     def test_shelve_split(self):
         outer_tree = self.make_branch_and_tree('outer')

=== modified file 'bzrlib/tests/test_source.py'
--- bzrlib/tests/test_source.py	2010-08-24 19:21:32 +0000
+++ bzrlib/tests/test_source.py	2011-06-13 22:10:04 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,7 +19,6 @@
 They are useful for testing code quality, checking coverage metric etc.
 """
 
-# import system imports here
 import os
 import parser
 import re
@@ -27,7 +26,6 @@
 import sys
 import token
 
-#import bzrlib specific imports here
 from bzrlib import (
     osutils,
     )
@@ -113,8 +111,9 @@
 
         # Avoid the case when bzrlib is packaged in a zip file
         if not os.path.isdir(source_dir):
-            raise TestSkipped('Cannot find bzrlib source directory. Expected %s'
-                              % source_dir)
+            raise TestSkipped(
+                'Cannot find bzrlib source directory. Expected %s'
+                % source_dir)
         return source_dir
 
     def get_source_files(self, extensions=None):
@@ -154,7 +153,7 @@
             yield fname, text
 
     def is_our_code(self, fname):
-        """Return true if it's a "real" part of bzrlib rather than external code"""
+        """True if it's a "real" part of bzrlib rather than external code"""
         if '/util/' in fname or '/plugins/' in fname:
             return False
         else:
@@ -194,10 +193,9 @@
 
         copyright_re = re.compile('#\\s*copyright.*(?=\n)', re.I)
         copyright_canonical_re = re.compile(
-            r'# Copyright \(C\) ' # Opening "# Copyright (C)"
-            r'(\d+)(, \d+)*' # Followed by a series of dates
-            r'.*Canonical Ltd' # And containing 'Canonical Ltd'
-            )
+            r'# Copyright \(C\) '  # Opening "# Copyright (C)"
+            r'(\d+)(, \d+)*'       # followed by a series of dates
+            r'.*Canonical Ltd')    # and containing 'Canonical Ltd'.
 
         for fname, text in self.get_source_file_contents(
                 extensions=('.py', '.pyx')):
@@ -230,7 +228,7 @@
                         ]
             for fname, comment in incorrect:
                 help_text.append(fname)
-                help_text.append((' '*4) + comment)
+                help_text.append((' ' * 4) + comment)
 
             self.fail('\n'.join(help_text))
 
@@ -269,10 +267,9 @@
                          " LICENSE_EXCEPTIONS in"
                          " bzrlib/tests/test_source.py",
                          "Or add the following text to the beginning:",
-                         gpl_txt
-                        ]
+                         gpl_txt]
             for fname in incorrect:
-                help_text.append((' '*4) + fname)
+                help_text.append((' ' * 4) + fname)
 
             self.fail('\n'.join(help_text))
 
@@ -283,7 +280,7 @@
             dict_[fname].append(line_no)
 
     def _format_message(self, dict_, message):
-        files = ["%s: %s" % (f, ', '.join([str(i+1) for i in lines]))
+        files = ["%s: %s" % (f, ', '.join([str(i + 1) for i in lines]))
                 for f, lines in dict_.items()]
         files.sort()
         return message + '\n\n    %s' % ('\n    '.join(files))
@@ -291,19 +288,16 @@
     def test_coding_style(self):
         """Check if bazaar code conforms to some coding style conventions.
 
-        Currently we assert that the following is not present:
-         * any tab characters
-         * non-unix newlines
-         * no newline at end of files
+        Generally we expect PEP8, but we do not generally strictly enforce
+        this, and there are existing files that do not comply.  The 'pep8'
+        tool, available separately, will check for more cases.
 
-        Print how many files have
-         * trailing white space
-         * lines longer than 79 chars
+        This test only enforces conditions that are globally true at the
+        moment, and that should cause a patch to be rejected: spaces rather
+        than tabs, unix newlines, and a newline at the end of the file.
         """
         tabs = {}
-        trailing_ws = {}
         illegal_newlines = {}
-        long_lines = {}
         no_newline_at_eof = []
         for fname, text in self.get_source_file_contents(
                 extensions=('.py', '.pyx')):
@@ -315,12 +309,8 @@
                 if '\t' in line:
                     self._push_file(tabs, fname, line_no)
                 if not line.endswith('\n') or line.endswith('\r\n'):
-                    if line_no != last_line_no: # not no_newline_at_eof
+                    if line_no != last_line_no:  # not no_newline_at_eof
                         self._push_file(illegal_newlines, fname, line_no)
-                if line.endswith(' \n'):
-                    self._push_file(trailing_ws, fname, line_no)
-                if len(line) > 80:
-                    self._push_file(long_lines, fname, line_no)
             if not lines[-1].endswith('\n'):
                 no_newline_at_eof.append(fname)
         problems = []
@@ -328,17 +318,9 @@
             problems.append(self._format_message(tabs,
                 'Tab characters were found in the following source files.'
                 '\nThey should either be replaced by "\\t" or by spaces:'))
-        if trailing_ws:
-            print ("There are %i lines with trailing white space in %i files."
-                % (sum([len(lines) for f, lines in trailing_ws.items()]),
-                    len(trailing_ws)))
         if illegal_newlines:
             problems.append(self._format_message(illegal_newlines,
                 'Non-unix newlines were found in the following source files:'))
-        if long_lines:
-            print ("There are %i lines longer than 79 characters in %i files."
-                % (sum([len(lines) for f, lines in long_lines.items()]),
-                    len(long_lines)))
         if no_newline_at_eof:
             no_newline_at_eof.sort()
             problems.append("The following source files doesn't have a "
@@ -382,9 +364,9 @@
     def test_extension_exceptions(self):
         """Extension functions should propagate exceptions.
 
-        Either they should return an object, have an 'except' clause, or have a
-        "# cannot_raise" to indicate that we've audited them and defined them as not
-        raising exceptions.
+        Either they should return an object, have an 'except' clause, or
+        have a "# cannot_raise" to indicate that we've audited them and
+        defined them as not raising exceptions.
         """
         both_exc_and_no_exc = []
         missing_except = []
@@ -392,13 +374,13 @@
                               r'(api\s+)?class (\w+).*:', re.MULTILINE)
         extern_class_re = re.compile(r'## extern cdef class (\w+)',
                                      re.MULTILINE)
-        except_re = re.compile(r'cdef\s+' # start with cdef
-                               r'([\w *]*?)\s*' # this is the return signature
-                               r'(\w+)\s*\(' # the function name
-                               r'[^)]*\)\s*' # parameters
-                               r'(.*)\s*:' # the except clause
-                               r'\s*(#\s*cannot[- _]raise)?' # cannot raise comment
-                              )
+        except_re = re.compile(
+            r'cdef\s+'        # start with cdef
+            r'([\w *]*?)\s*'  # this is the return signature
+            r'(\w+)\s*\('     # the function name
+            r'[^)]*\)\s*'     # parameters
+            r'(.*)\s*:'       # the except clause
+            r'\s*(#\s*cannot[- _]raise)?')  # cannot raise comment
         for fname, text in self.get_source_file_contents(
                 extensions=('.pyx',)):
             known_classes = set([m[-1] for m in class_re.findall(text)])
@@ -417,15 +399,18 @@
                     missing_except.append((fname, func))
         error_msg = []
         if both_exc_and_no_exc:
-            error_msg.append('The following functions had "cannot raise" comments'
-                             ' but did have an except clause set:')
+            error_msg.append(
+                'The following functions had "cannot raise" comments'
+                ' but did have an except clause set:')
             for fname, func in both_exc_and_no_exc:
                 error_msg.append('%s:%s' % (fname, func))
             error_msg.extend(('', ''))
         if missing_except:
-            error_msg.append('The following functions have fixed return types,'
-                             ' but no except clause.')
-            error_msg.append('Either add an except or append "# cannot_raise".')
+            error_msg.append(
+                'The following functions have fixed return types,'
+                ' but no except clause.')
+            error_msg.append(
+                'Either add an except or append "# cannot_raise".')
             for fname, func in missing_except:
                 error_msg.append('%s:%s' % (fname, func))
             error_msg.extend(('', ''))

=== modified file 'bzrlib/tests/test_transform.py'
--- bzrlib/tests/test_transform.py	2011-06-10 16:19:57 +0000
+++ bzrlib/tests/test_transform.py	2011-06-13 16:25:08 +0000
@@ -285,6 +285,23 @@
         new_trans_id = transform.new_directory('', ROOT_PARENT, 'alt-root-id')
         self.assertRaises(ValueError, transform.fixup_new_roots)
 
+    def test_add_unversioned_root(self):
+        transform, root = self.get_transform()
+        new_trans_id = transform.new_directory('', ROOT_PARENT, None)
+        transform.fixup_new_roots()
+        self.assertNotIn(transform.root, transform._new_id)
+
+    def test_apply_retains_root_directory(self):
+        # Do not attempt to delete the physical root directory, because that
+        # is impossible.
+        transform, root = self.get_transform()
+        with transform:
+            transform.delete_contents(root)
+            e = self.assertRaises(AssertionError, self.assertRaises,
+                                  errors.TransformRenameFailed,
+                                  transform.apply)
+        self.assertContainsRe('TransformRenameFailed not raised', str(e))
+
     def test_hardlink(self):
         self.requireFeature(HardlinkFeature)
         transform, root = self.get_transform()

=== modified file 'bzrlib/transform.py'
--- bzrlib/transform.py	2011-06-10 16:19:57 +0000
+++ bzrlib/transform.py	2011-06-21 15:19:18 +0000
@@ -32,6 +32,7 @@
     bencode,
     bzrdir,
     commit,
+    conflicts,
     delta,
     errors,
     inventory,
@@ -252,7 +253,8 @@
         if (self.tree_file_id(self._new_root) is not None and
             self._new_root not in self._removed_id):
             self.unversion_file(self._new_root)
-        self.version_file(file_id, self._new_root)
+        if file_id is not None:
+            self.version_file(file_id, self._new_root)
 
         # Now move children of new root into old root directory.
         # Ensure all children are registered with the transaction, but don't
@@ -1826,8 +1828,10 @@
         tree_paths.sort(reverse=True)
         child_pb = ui.ui_factory.nested_progress_bar()
         try:
-            for num, data in enumerate(tree_paths):
-                path, trans_id = data
+            for num, (path, trans_id) in enumerate(tree_paths):
+                # do not attempt to move root into a subdirectory of itself.
+                if path == '':
+                    continue
                 child_pb.update('removing file', num, len(tree_paths))
                 full_path = self._tree.abspath(path)
                 if trans_id in self._removed_contents:
@@ -3153,13 +3157,11 @@
 
 def cook_conflicts(raw_conflicts, tt):
     """Generate a list of cooked conflicts, sorted by file path"""
-    from bzrlib.conflicts import Conflict
     conflict_iter = iter_cook_conflicts(raw_conflicts, tt)
-    return sorted(conflict_iter, key=Conflict.sort_key)
+    return sorted(conflict_iter, key=conflicts.Conflict.sort_key)
 
 
 def iter_cook_conflicts(raw_conflicts, tt):
-    from bzrlib.conflicts import Conflict
     fp = FinalPaths(tt)
     for conflict in raw_conflicts:
         c_type = conflict[0]
@@ -3167,16 +3169,17 @@
         modified_path = fp.get_path(conflict[2])
         modified_id = tt.final_file_id(conflict[2])
         if len(conflict) == 3:
-            yield Conflict.factory(c_type, action=action, path=modified_path,
-                                     file_id=modified_id)
+            yield conflicts.Conflict.factory(
+                c_type, action=action, path=modified_path, file_id=modified_id)
 
         else:
             conflicting_path = fp.get_path(conflict[3])
             conflicting_id = tt.final_file_id(conflict[3])
-            yield Conflict.factory(c_type, action=action, path=modified_path,
-                                   file_id=modified_id,
-                                   conflict_path=conflicting_path,
-                                   conflict_file_id=conflicting_id)
+            yield conflicts.Conflict.factory(
+                c_type, action=action, path=modified_path,
+                file_id=modified_id,
+                conflict_path=conflicting_path,
+                conflict_file_id=conflicting_id)
 
 
 class _FileMover(object):

=== modified file 'bzrlib/tree.py'
--- bzrlib/tree.py	2011-05-20 15:15:53 +0000
+++ bzrlib/tree.py	2011-06-14 02:23:26 +0000
@@ -800,59 +800,6 @@
         return self.get_file(self._inventory.path2id(path), path)
 
 
-######################################################################
-# diff
-
-# TODO: Merge these two functions into a single one that can operate
-# on either a whole tree or a set of files.
-
-# TODO: Return the diff in order by filename, not by category or in
-# random order.  Can probably be done by lock-stepping through the
-# filenames from both trees.
-
-
-def file_status(filename, old_tree, new_tree):
-    """Return single-letter status, old and new names for a file.
-
-    The complexity here is in deciding how to represent renames;
-    many complex cases are possible.
-    """
-    old_inv = old_tree.inventory
-    new_inv = new_tree.inventory
-    new_id = new_inv.path2id(filename)
-    old_id = old_inv.path2id(filename)
-
-    if not new_id and not old_id:
-        # easy: doesn't exist in either; not versioned at all
-        if new_tree.is_ignored(filename):
-            return 'I', None, None
-        else:
-            return '?', None, None
-    elif new_id:
-        # There is now a file of this name, great.
-        pass
-    else:
-        # There is no longer a file of this name, but we can describe
-        # what happened to the file that used to have
-        # this name.  There are two possibilities: either it was
-        # deleted entirely, or renamed.
-        if new_inv.has_id(old_id):
-            return 'X', old_inv.id2path(old_id), new_inv.id2path(old_id)
-        else:
-            return 'D', old_inv.id2path(old_id), None
-
-    # if the file_id is new in this revision, it is added
-    if new_id and not old_inv.has_id(new_id):
-        return 'A'
-
-    # if there used to be a file of this name, but that ID has now
-    # disappeared, it is deleted
-    if old_id and not new_inv.has_id(old_id):
-        return 'D'
-
-    return 'wtf?'
-
-
 def find_ids_across_trees(filenames, trees, require_versioned=True):
     """Find the ids corresponding to specified filenames.
 

=== modified file 'bzrlib/ui/__init__.py'
--- bzrlib/ui/__init__.py	2011-05-27 05:16:48 +0000
+++ bzrlib/ui/__init__.py	2011-06-10 07:01:07 +0000
@@ -154,6 +154,8 @@
             "It is recommended that you upgrade by "
             "running the command\n"
             "  bzr upgrade %(basedir)s"),
+        locks_steal_dead=(
+            u"Stole dead lock %(lock_url)s %(other_holder_info)s."),
         )
 
     def __init__(self):
@@ -304,13 +306,13 @@
         try:
             template = self._user_warning_templates[warning_id]
         except KeyError:
-            fail = "failed to format warning %r, %r" % (warning_id, message_args)
-            warnings.warn(fail)   # so tests will fail etc
+            fail = "bzr warning: %r, %r" % (warning_id, message_args)
+            warnings.warn("no template for warning: " + fail)   # so tests will fail etc
             return fail
         try:
             return template % message_args
         except ValueError, e:
-            fail = "failed to format warning %r, %r: %s" % (
+            fail = "bzr unprintable warning: %r, %r, %s" % (
                 warning_id, message_args, e)
             warnings.warn(fail)   # so tests will fail etc
             return fail

=== modified file 'bzrlib/uncommit.py'
--- bzrlib/uncommit.py	2010-04-30 11:03:59 +0000
+++ bzrlib/uncommit.py	2011-06-16 12:50:32 +0000
@@ -66,12 +66,10 @@
             revno = old_revno
         new_revno = revno - 1
 
-        revid_iterator = branch.repository.iter_reverse_revision_history(
-                            old_tip)
         cur_revno = old_revno
         new_revision_id = old_tip
         graph = branch.repository.get_graph()
-        for rev_id in revid_iterator:
+        for rev_id in graph.iter_lefthand_ancestry(old_tip):
             if cur_revno == new_revno:
                 new_revision_id = rev_id
                 break

=== modified file 'bzrlib/win32utils.py'
--- bzrlib/win32utils.py	2011-05-18 16:42:48 +0000
+++ bzrlib/win32utils.py	2011-06-11 01:11:43 +0000
@@ -576,3 +576,41 @@
         return argv
 else:
     get_unicode_argv = None
+
+
+if has_win32api:
+    def _pywin32_is_local_pid_dead(pid):
+        """True if pid doesn't correspond to live process on this machine"""
+        try:
+            handle = win32api.OpenProcess(1, False, pid) # PROCESS_TERMINATE
+        except pywintypes.error, e:
+            if e[0] == 5: # ERROR_ACCESS_DENIED
+                # Probably something alive we're not allowed to kill
+                return False
+            elif e[0] == 87: # ERROR_INVALID_PARAMETER
+                return True
+            raise
+        handle.close()
+        return False
+    is_local_pid_dead = _pywin32_is_local_pid_dead
+elif has_ctypes and sys.platform == 'win32':
+    from ctypes.wintypes import BOOL, DWORD, HANDLE
+    _kernel32 = ctypes.windll.kernel32
+    _CloseHandle = ctypes.WINFUNCTYPE(BOOL, HANDLE)(
+        ("CloseHandle", _kernel32))
+    _OpenProcess = ctypes.WINFUNCTYPE(HANDLE, DWORD, BOOL, DWORD)(
+        ("OpenProcess", _kernel32))
+    def _ctypes_is_local_pid_dead(pid):
+        """True if pid doesn't correspond to live process on this machine"""
+        handle = _OpenProcess(1, False, pid) # PROCESS_TERMINATE
+        if not handle:
+            errorcode = ctypes.GetLastError()
+            if errorcode == 5: # ERROR_ACCESS_DENIED
+                # Probably something alive we're not allowed to kill
+                return False
+            elif errorcode == 87: # ERROR_INVALID_PARAMETER
+                return True
+            raise ctypes.WinError(errorcode)
+        _CloseHandle(handle)
+        return False
+    is_local_pid_dead = _ctypes_is_local_pid_dead

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2011-05-18 16:42:48 +0000
+++ bzrlib/workingtree.py	2011-06-18 13:57:17 +0000
@@ -496,13 +496,6 @@
         finally:
             file.close()
 
-    def _get_ancestors(self, default_revision):
-        ancestors = set([default_revision])
-        for parent_id in self.get_parent_ids():
-            ancestors.update(self.branch.repository.get_ancestry(
-                             parent_id, topo_sorted=False))
-        return ancestors
-
     def get_parent_ids(self):
         """See Tree.get_parent_ids.
 
@@ -973,8 +966,8 @@
         file and change the file_id. That is the normal mode. Second, it can
         only change the file_id without touching any physical file.
 
-        rename_one uses the second mode if 'after == True' and 'to_rel' is not
-        versioned but present in the working tree.
+        rename_one uses the second mode if 'after == True' and 'to_rel' is
+        either not versioned or newly added, and present in the working tree.
 
         rename_one uses the second mode if 'after == False' and 'from_rel' is
         versioned but no longer in the working tree, and 'to_rel' is not
@@ -1385,11 +1378,6 @@
     def revert(self, filenames=None, old_tree=None, backups=True,
                pb=None, report_changes=False):
         from bzrlib.conflicts import resolve
-        if filenames == []:
-            filenames = None
-            symbol_versioning.warn('Using [] to revert all files is deprecated'
-                ' as of bzr 0.91.  Please use None (the default) instead.',
-                DeprecationWarning, stacklevel=2)
         if old_tree is None:
             basis_tree = self.basis_tree()
             basis_tree.lock_read()
@@ -2573,8 +2561,8 @@
         inventory. The second mode only updates the inventory without
         touching the file on the filesystem.
 
-        move uses the second mode if 'after == True' and the target is not
-        versioned but present in the working tree.
+        move uses the second mode if 'after == True' and the target is
+        either not versioned or newly added, and present in the working tree.
 
         move uses the second mode if 'after == False' and the source is
         versioned but no longer in the working tree, and the target is not
@@ -2727,7 +2715,8 @@
 
     class _RenameEntry(object):
         def __init__(self, from_rel, from_id, from_tail, from_parent_id,
-                     to_rel, to_tail, to_parent_id, only_change_inv=False):
+                     to_rel, to_tail, to_parent_id, only_change_inv=False,
+                     change_id=False):
             self.from_rel = from_rel
             self.from_id = from_id
             self.from_tail = from_tail
@@ -2735,6 +2724,7 @@
             self.to_rel = to_rel
             self.to_tail = to_tail
             self.to_parent_id = to_parent_id
+            self.change_id = change_id
             self.only_change_inv = only_change_inv
 
     def _determine_mv_mode(self, rename_entries, after=False):
@@ -2752,14 +2742,27 @@
             to_rel = rename_entry.to_rel
             to_id = inv.path2id(to_rel)
             only_change_inv = False
+            change_id = False
 
             # check the inventory for source and destination
             if from_id is None:
                 raise errors.BzrMoveFailedError(from_rel,to_rel,
                     errors.NotVersionedError(path=from_rel))
             if to_id is not None:
-                raise errors.BzrMoveFailedError(from_rel,to_rel,
-                    errors.AlreadyVersionedError(path=to_rel))
+                allowed = False
+                # allow it with --after but only if dest is newly added
+                if after:
+                    basis = self.basis_tree()
+                    basis.lock_read()
+                    try:
+                        if not basis.has_id(to_id):
+                            rename_entry.change_id = True
+                            allowed = True
+                    finally:
+                        basis.unlock()
+                if not allowed:
+                    raise errors.BzrMoveFailedError(from_rel,to_rel,
+                        errors.AlreadyVersionedError(path=to_rel))
 
             # try to determine the mode for rename (only change inv or change
             # inv and file system)
@@ -2836,6 +2839,9 @@
             except OSError, e:
                 raise errors.BzrMoveFailedError(entry.from_rel,
                     entry.to_rel, e[1])
+        if entry.change_id:
+            to_id = inv.path2id(entry.to_rel)
+            inv.remove_recursive_id(to_id)
         inv.rename(entry.from_id, entry.to_parent_id, entry.to_tail)
 
     @needs_tree_write_lock

=== modified file 'bzrlib/workingtree_4.py'
--- bzrlib/workingtree_4.py	2011-05-20 10:49:09 +0000
+++ bzrlib/workingtree_4.py	2011-06-16 11:58:04 +0000
@@ -1713,9 +1713,6 @@
         annotations = self._repository.texts.annotate(text_key)
         return [(key[-1], line) for (key, line) in annotations]
 
-    def _get_ancestors(self, default_revision):
-        return set(self._repository.get_ancestry(self._revision_id,
-                                                 topo_sorted=False))
     def _comparison_data(self, entry, path):
         """See Tree._comparison_data."""
         if entry is None:

=== modified file 'doc/en/release-notes/bzr-2.3.txt'
--- doc/en/release-notes/bzr-2.3.txt	2011-06-09 15:13:54 +0000
+++ doc/en/release-notes/bzr-2.3.txt	2011-06-16 18:34:26 +0000
@@ -33,6 +33,10 @@
 .. Fixes for situations where bzr would previously crash or give incorrect
    or undesirable results.
 
+* Don't fail with traceback if `bzr serve` is running as a service on Windows,
+  and there is no USERNAME, nor BZR_EMAIL or other whoami-related environment
+  variables set. (Alexander Belchenko, Bug #660174)
+
 Documentation
 *************
 

=== modified file 'doc/en/release-notes/bzr-2.4.txt'
--- doc/en/release-notes/bzr-2.4.txt	2011-06-13 16:34:53 +0000
+++ doc/en/release-notes/bzr-2.4.txt	2011-06-28 13:55:39 +0000
@@ -5,28 +5,116 @@
 .. toctree::
    :maxdepth: 1
 
+bzr 2.4b5
+#########
+
+:2.4b5: NOT RELEASED YET
+
+External Compatibility Breaks
+*****************************
+
+.. These may require users to change the way they use Bazaar.
+
+New Features
+************
+
+.. New commands, options, etc that users may wish to try out.
+
+Improvements
+************
+
+.. Improvements to existing commands, especially improved performance 
+   or memory usage, or better results.
+
+Bug Fixes
+*********
+
+* Display a proper error message when a config file content cannot be
+  decoded as UTF-8 or when it cannot be parsed.
+  (Vincent Ladeuil, #502060, #688677, #792246)
+
+* Generate a single conflict (instead of two) when merging a branch
+  modifying and renaming a file in a branch that deleted it (or vice-versa).
+  (Vincent Ladeuil, #688101)
+
+* Properly load utf8-encoded config files. (Vincent Ladeuil, #799212)
+
+* ``GraphThunkIdsToKeys.merge_sort`` now properly returns
+  keys rather than ids. (Jelmer Vernooij, #799677)
+
+.. Fixes for situations where bzr would previously crash or give incorrect
+   or undesirable results.
+
+Documentation
+*************
+
+.. Improved or updated documentation.
+
+API Changes
+***********
+
+.. Changes that may require updates in plugins or other code that uses
+   bzrlib.
+
+Internals
+*********
+
+.. Major internal changes, unlikely to be visible to users or plugin 
+   developers, but interesting for bzr developers.
+
+* Start implementing localization, starting with command help text (but not
+  the command options themselves). This will allow bootstrapping the bzr
+  internationalization process. (Inada Naoki)
+
+Testing
+*******
+
+.. Fixes and changes that are only relevant to bzr's test framework and 
+   suite.  This can include new facilities for writing tests, fixes to 
+   spurious test failures and changes to the way things should be tested.
+
+* Fix test failures when running as a homeless user (debian buildd). Tests
+  leaking into ``${HOME}/.bzr.log`` should be detected properly now.
+  (Vincent Ladeuil, #798698)
+
 bzr 2.4b4
 #########
 
-:2.4b4: NOT RELEASED YET
+:2.4b4: 2011-06-16
+
+This is the fourth beta of the 2.4 series, leading to a 2.4.0 release in
+August 2011. Beta releases are suitable for everyday use but may cause some
+incompatibilities with plugins.
+
+This release includes all bug fixed in previous series known at the time of
+this release.
+
 
 External Compatibility Breaks
 *****************************
 
 .. These may require users to change the way they use Bazaar.
 
+* Do not treat configuration option 'check_signatures = require' as if
+  it were 'create_signatures = always' (Jonathan Riddell)
+
 New Features
 ************
 
 .. New commands, options, etc that users may wish to try out.
 
+* Hooks have been added for config stacks: ``get``, ``set`` and ``remove``
+  are called when an option is respectively read, modified or deleted. Also
+  added ``load`` and ``save`` hooks for config stores, called when the
+  stores are loaded or saved.  (Vincent Ladeuil)
+
 * New hook server_exception in bzrlib.smart.server to catch any
-  exception caused while running bzr serve.  (Jonathan Riddell,
-  #274578)
-
-* New hook set_commit_message in bzrlib.msgeditor to set
-  a commit message and revision properties.  (Jonathan Riddell,
-  #274578)
+  exception caused while running bzr serve.
+  (Jonathan Riddell, #274578)
+
+
+* New hook set_commit_message in bzrlib.msgeditor to set a commit message
+  and revision properties.  (Jonathan Riddell, #274578)
 
 * Support ``-S`` as an alias for ``--short`` for the ``log`` and
   ``missing`` commands. (Martin von Gagern, #38655)
@@ -37,18 +125,31 @@
 .. Improvements to existing commands, especially improved performance 
    or memory usage, or better results.
 
+* ``bzr annotate`` can be run without setting whoami data first.
+  (Jonathan Riddell, #667408)
+
 Bug Fixes
 *********
 
 .. Fixes for situations where bzr would previously crash or give incorrect
    or undesirable results.
 
+* Bazaar can now detect when a lock file is held by a dead process
+  originating from the same machine, and steal the lock after printing a
+  message to the user.  This is off by default, for safety, but can be
+  turned on by setting the configuration variable ``locks.steal_dead`` to
+  ``True``.
+  (Martin Pool, #220464)
+
 * Credentials in the log output produced by ``-Dhttp`` are masked so users
   can more freely post them in bug reports. (Vincent Ladeuil, #723074)
 
 * Fix a race condition for ``server_started`` hooks leading to a spurious
   test failure. (Vincent Ladeuil, #789167)
 
+* Fix exporting subdirectory with ``--per-file-timestamps``.
+  (Szilveszter Farkas, #795557)
+
 * Handle files that get created but don't get used during TreeTransform.
   ``open()`` can create a file, and still raise an exception before it
   returns. So anything we might have created, make sure we destroy during
@@ -63,6 +164,9 @@
   because ``build_mo`` became a required step that they didn't know about.
   (John Arbash Meinel, #787122)
 
+* Properly avoid re-adding a file after it changes case on CICP
+  filesystems. (John Arbash Meinel, #798130)
+
 * Reports the original error when an InvalidHttpResponse exception is
   encountered to facilitate debug. (Vincent Ladeuil, #788530)
 
@@ -73,6 +177,8 @@
   ``UIFactory.get_password`` and ``UIFactory.get_boolean`` now require a
   unicode prompt to be passed in. (Jelmer Vernooij, #592083)
 
+* Support merging into the empty tree. (Aaron Bentley, #595328)
+
 Documentation
 *************
 
@@ -81,6 +187,11 @@
 * Improve documentation of ``bzr merge --force``.
   (Neil Martinsen-Burrell, #767307)
 
+* Make docs for configuration options for digital signatures match 
+  reality. (Jonathan Riddell)
+
+* Add user-guide page on GPG signatures. (Jonathan Riddell)
+
 API Changes
 ***********
 
@@ -91,6 +202,22 @@
   functions, so that calling code can take stream the output.
   (Xaav, Martin Pool)
 
+* Information about held lockdir locks returned from eg `LockDir.peek` is
+  now represented as a `LockHeldInfo` object, rather than a plain
+  Python dict.
+  (Martin Pool)
+
+* Remove `file_status` function.
+  (Martin Pool)
+
+* ``Repository.iter_reverse_revision_history`` is now deprecated.
+  Use ``Graph.iter_lefthand_ancestry`` instead.
+  (Jelmer Vernooij, #739481)
+
+* ``Repository.get_ancestry`` has been deprecated. Use
+  ``Graph.iter_ancestry`` instead.
+  (Jelmer Vernooij, #784511)
+
 Internals
 *********
 
@@ -120,6 +247,10 @@
 * Fix spurious test failure on OSX for WorkingTreeFormat2.
   (Vincent Ladeuil, #787942)
 
+* Re-target ``bb.test_merge.TestMerge.test_merge_reversed_revision_range``
+  and rewrite it as a parameterized test to avoid unrelated failures.
+  (Vincent Ladeuil, #795456)
+
 * Show log file contents from subprocesses started by
   `start_bzr_subprocess` in test failure details.  This may help diagnose
   strange hangs and failures involving subprocesses.  (Andrew Bennetts)
@@ -249,8 +380,8 @@
 * Merging into empty branches now gives an error as this is currently
   not supported. (Jonathan Riddell, #242175)
 
-* Do not show exception to user on pointless commit error (Jonathan
-  Riddell #317357)
+* Do not show exception to user on pointless commit error.
+  (Jonathan Riddell #317357)
 
 * ``WT.update_basis_by_delta`` no longer requires that the deltas match
   the current WT state. This allows ``update_basis_by_delta`` to be used
@@ -258,6 +389,9 @@
   not load the whole inventory, which can take 10+s with large trees.
   (Jonathan Riddell, John Arbash Meinel, #781168)
 
+* ``bzr mv --after old_name new_name`` now works if "new_name" is newly
+  added. (Benoît Pierre)
+
 
 Documentation
 *************

=== added file 'doc/en/user-guide/gpg_signatures.txt'
--- doc/en/user-guide/gpg_signatures.txt	1970-01-01 00:00:00 +0000
+++ doc/en/user-guide/gpg_signatures.txt	2011-06-14 09:21:36 +0000
@@ -0,0 +1,95 @@
+GnuPG Signatures
+=============================
+
+Reasons to Sign Your Repository
+--------------------------------
+
+Bazaar can sign revisions using GnuPG, a Free Software implementation of the
+OpenPGP digital signature format.  By signing commits a person wanting to
+make use of a branch can be confident where the code came from, assuming the
+GnuPG keys used can be verified.  This could for example prevent worry about
+compromised code in the case where a server hosting Bazaar branches has been
+hacked into.  It could also be used to verify that all code is written by a
+select group of people, such as if contributor agreements are needed.
+
+Signatures are passed around with commits during branch, push, merge and other
+operations.
+
+Setting up GnuPG
+--------------
+
+There are many guides to creating a digital signature key with GnuPG.  See
+for example the `GnuPG Handbook
+<http://www.gnupg.org/gph/en/manual.html#AEN26>`_ or the `Launchpad Wiki
+<https://help.launchpad.net/YourAccount/ImportingYourPGPKey>`_.
+
+
+Signing Commits
+---------------
+
+To sign commits as they are made turn on the ``create_signatures``
+configuration option in your ``bazaar.conf`` or ``locations.conf`` file::
+
+  create_signatures = always
+
+When you next make a commit it will ask for the pass phrase for your GnuPG key. 
+If you want GnuPG to remember your password ensure you have ``gnupg-agent``
+installed.
+
+To sign previous commits to a branch use ``sign-my-commits``.  This will go
+through all revisions in the branch and sign any which match your
+commit name.  You can also pass the name of a contributor to ``sign-my-commits``
+to sign someone else's commits or if your GnuPG key does not match your Bazaar
+name and e-mail::
+
+  bzr sign-my-commits . "Amy Pond <amy@example.com>"
+
+It will not sign commits which already have a signature.
+
+To sign a single commit or a range of commits use the (hidden) command
+``re-sign``::
+
+  bzr re-sign -r 24
+
+``re-sign`` is also useful to change an existing signature.
+
+Verifying Commits
+-----------------
+
+Unfortunately there is currently no command for verifying signatures.  This can
+be done manually using bzrlib and Python::
+
+  >>> from bzrlib.branch import Branch
+  >>> b = Branch.open('/home/amy/src/daleks')
+  >>> b.last_revision()
+  'amy@example.com-20110527185938-hluafawphszb8dl1'
+  >>> print b.repository.get_signature_text(b.last_revision())
+  -----BEGIN PGP SIGNED MESSAGE-----
+  Hash: SHA1
+
+  bazaar-ng testament short form 1
+  revision-id: amy@example.com-20110527185938-hluafawphszb8dl1
+  sha1: 6411f9bdf6571200357140c9ce7c0f50106ac9a4
+  -----BEGIN PGP SIGNATURE-----
+  Version: GnuPG v1.4.11 (GNU/Linux)
+
+  iEYEARECAAYFAk32HFcACgkQpQbm1N1NUIiBXACg6ILsyvJp4+Twq190qk1I4v9K
+  PAAAoI8pg9s7uuqldqOwz6/uwH3ezdSX
+  =+NcB
+  -----END PGP SIGNATURE-----
+
+The signature can be saved to a file and verified with ``gpg --verify
+signature.txt``.
+
+The signed SHA1 checksum is made from the ``bzr testament --long`` command
+which includes information on the committer and a list of all files in that
+commit along with their checksums.  So if the signature is valid you can be
+sure you are looking at the same code as the person who made the signature.
+
+Work in Progress
+----------------
+
+There is still a number of digital signature related features which are hoped
+to be added to Bazaar soon.  These include easy verificiation, qbzr
+integration, signing with different keys and setting branches to require
+signatures.

=== modified file 'doc/en/user-guide/index.txt'
--- doc/en/user-guide/index.txt	2011-05-16 10:22:06 +0000
+++ doc/en/user-guide/index.txt	2011-06-13 14:08:28 +0000
@@ -103,6 +103,7 @@
    server
    hooks
    version_info
+   gpg_signatures
 
 
 A brief tour of some popular plugins

=== modified file 'setup.py'
--- setup.py	2011-05-26 21:24:36 +0000
+++ setup.py	2011-06-16 18:34:26 +0000
@@ -478,6 +478,12 @@
     packages.append('sqlite3')
 
 
+def get_fastimport_py2exe_info(includes, excludes, packages):
+    # This is the python-fastimport package, not to be confused with the
+    # bzr-fastimport plugin.
+    packages.append('fastimport')
+
+
 if 'bdist_wininst' in sys.argv:
     def find_docs():
         docs = []
@@ -670,6 +676,9 @@
     if 'svn' in plugins:
         get_svn_py2exe_info(includes, excludes, packages)
 
+    if 'fastimport' in plugins:
+        get_fastimport_py2exe_info(includes, excludes, packages)
+
     if "TBZR" in os.environ:
         # TORTOISE_OVERLAYS_MSI_WIN32 must be set to the location of the
         # TortoiseOverlays MSI installer file. It is in the TSVN svn repo and

=== added file 'tools/subunit-sum'
--- tools/subunit-sum	1970-01-01 00:00:00 +0000
+++ tools/subunit-sum	2011-06-16 10:45:17 +0000
@@ -0,0 +1,61 @@
+#!/usr/bin/env python
+# Copyright (C) 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Displays the sum of a counter found in a subunit stream.
+
+Each test have (or not) the counter as a named detail in the stream.
+"""
+
+import subunit
+import sys
+import testtools
+import unittest
+
+
+class TestSumCounter(testtools.TestResult):
+
+    def __init__(self, counter_names):
+        """Create a FilterResult object outputting to stream."""
+        testtools.TestResult.__init__(self)
+        self.counter_names = counter_names
+        self.totals = {}
+        self.longest = 0
+        for name in counter_names:
+            l = len(name)
+            if l > self.longest: self.longest = l
+            self.totals[name] = 0
+
+    def addSuccess(self, test, details):
+        for name in self.counter_names:
+            try:
+                counter_text = ''.join(details[name].iter_text())
+            except KeyError, e:
+                # this counter doesn't exist for the test
+                continue
+            self.totals[name] += int(counter_text)
+
+    def display_totals(self, stream):
+        for name in self.counter_names:
+            stream.write('%-*s: %s\n'
+                         % (self.longest, name, self.totals[name]))
+
+
+counter_names = sys.argv[1:]
+result = TestSumCounter(counter_names)
+test = subunit.ProtocolTestCase(sys.stdin)
+test.run(result)
+result.display_totals(sys.stdout)

